(* ::Package:: *)

Print["This is BSFH.m, version 1.0, 
a tool for computing bordered sutured Heegaard Floer homology,
based on an algorithm due to Rumen Zarev,
written in Mathematica 10.3.1.0 for Linux x86 (64-bit). 
A manual for this Mathematica package can be found in appendix D of

   C.B. Zibrowius, ",Style["On a Heegaard Floer theory for tangles", Italic],", PhD thesis (2016), Cambridge. 

For more background on bordered sutured Floer theory, see

   R. Zarev, ",Style["Bordered Floer homology for sutured manifolds", Italic],", arXiv: 0908.1106v2.

For questions, suggestions or comments, please email claudius.zibrowius","@","gmail.com. 

    Copyright (C) 2015-2016 Claudius Zibrowius

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>."
];


regionsInput::usage="Input Data: A table of regions of a nice Heegaard diagram. Each region corresponds to a row; its first entry is the region index; the remaining entries are the vertices of the region, ordered by boundary orientation, starting with a vertex which is the start of a segment of an \[Alpha]-curve/\[Alpha]-arc in the boundary of the region. Our convention are such that the boundary of a bigon with multiplicity 1 is oriented counter-clockwise. ";
alphacurvesInput::usage="Input Data: A table of \[Alpha]-curves in a nice Heegaard diagram. Each curve corresponds to a row; its first entry is the curve index; the second a list of all intersection points on this curve.";
alphaarcsInput::usage="Input Data: A table of \[Alpha]-arcs in a nice Heegaard diagram. Each arc corresponds to a row; its first entry is the arc index; the second a list of all intersection points on this arc. IMPORTANT: The rows need to be ordered by glueing surfaces. Those alpha arcs for S1 have lower index than those for S2!";
betacurvesInput::usage="Input Data: A table of \[Beta]-curves in a nice Heegaard diagram. Each curve corresponds to a row; its first entry is the curve index; the second a list of all intersection points on this curve.";
CancellationSortListInput::usage="Input Data: A table of pairs of intersection points that can be removed by finger-moves. Each row corresponds to such a pair, so it has two entries. The order of the intersection points is the same as for 'regionsInput', noting that there is a bigon connecting the two intersection points. Generators occupying these intersection points will later be cancelled along the differentials from these bigons, if possible, in the order specified in this list 'CSortListIn'.";
RelevantGeneratorGenerationQ::usage="RelevantGeneratorGenerationQ[v] takes a list of alpha curves as input and decides if it might be relevant in the sense that it should be used in the calculation of the bordered sutured Floer homology. This is a quick check used for speeding up the generator generation. USE WITH CARE! As default, set equal to True.";
RelevantGeneratorQ::usage="RelevantGeneratorQ[v] takes a generator and decides if if it is relevant in the sense that it should be used in the calculation of the bordered sutured Floer homology. For type A structures or bimodules, it can make sense to forget some generators in certain idempotents, but it should be used with care. As default, set equal to 'True'.";
S1BoundaryInput::usage="Input Data: A list of arcs in the arc diagram corresponding to the first glueing surface. The entry for each arc is a (possibly empty) list of adjacent regions, ordered opposite to the boundary orientation. ";
S2BoundaryInput::usage="Input Data: A list of arcs in the arc diagram corresponding to the second glueing surface. The entry for each arc is a (possibly empty) list of adjacent regions, ordered opposite to the boundary orientation. ";
ArcDiagram1MatchingsInput::usage="Input Data: A list of \[Alpha]-arcs parametrizing the first glueing surface, ordered by their indices. The entry for each \[Alpha]-arc is the pair of its endpoints. The index of each endpoint is determined by the order of arcs in 'S1BoundaryInput'.";
ArcDiagram2MatchingsInput::usage="Input Data: A list of \[Alpha]-arcs parametrizing the second glueing surface, ordered by their indices. The entry for each \[Alpha]-arc is the pair of its endpoints. The index of each endpoint is determined by the order of arcs in 'S2BoundaryInput'.";
TypeS1::usage="Input Data: TypeS1 takes one of two values, namely 'TypeA' or 'TypeD', depending on whether we want to compute a type A or type D structure.";
TypeS2::usage="Input Data: TypeS2 takes one of two values, namely 'TypeA' or 'TypeD', depending on whether we want to compute a type A or type D structure.";


Step0Preparation:=Module[{},
(* Input for glueing surfaces *)
Bpre1=regionsInput.({
 {1, 0, 0, 0, 0},
 {0, 1, 0, 0, 0},
 {0, 0, 0, 1, 0},
 {0, 0, 1, 0, 0},
 {0, 0, 0, 0, 1}
});
Bindpre1=Table[{i,Count[Bpre1[[i]],\[Placeholder]]},{i,1,NR}];
Bpre2X=Bpre1[[All,2;;3]];
Bpre3X=DeleteCases[Bpre2X,v_/;v==\[Placeholder],2];
Bpre2Y=Bpre1[[All,4;;5]];
Bpre3Y=DeleteCases[Bpre2Y,v_/;v==\[Placeholder],2];
Bpre4=Flatten[Table[Join[Table[{i,Bpre3X[[i,ii]]}->1,{ii,1,Length[Bpre3X[[i]]]}],Table[{i,Bpre3Y[[i,ii]]}->-1,{ii,1,Length[Bpre3Y[[i]]]}]],{i,1,NR}]];
(* B: ith row=vertices of the ith region, ith column=regions adjacent to the ith intersection point *)
B=SparseArray[Bpre4,{NR,NI},0];
AbsB=Abs[B];
Bind=Bindpre1[[All,2]];
(* Feedback *)
Print["CHECK THE FOLLOWING FEEDBACK CAREFULLY! "]
Print["#{regions}=",NR];
Print["#{intersection points}=",NI];
(*Print[MatrixForm[Join[Transpose[Bpre1/.{\[Placeholder]->""}],{ConstantArray["--",NR]},Transpose[Bindpre1/.{0->""}]]]];
Print[MatrixForm[{Table[i,{i,1,NI}],Table[1,{i,1,NR}].B }]];
*)
Print[" 0 vertices = ",Flatten[Position[Table[1,{i,1,NR}].B,0]]];
Print["+1 vertices = ",Flatten[Position[Table[1,{i,1,NR}].B,1]]];
Print["-1 vertices = ",Flatten[Position[Table[1,{i,1,NR}].B,-1]]];
Print["+2 vertices = ",Flatten[Position[Table[1,{i,1,NR}].B,2]]];
Print["-2 vertices = ",Flatten[Position[Table[1,{i,1,NR}].B,-2]]];
Print["bigon regions = ",Flatten[Position[Bind,2]]];
(* returns the index of the curve/arc occupied by vertex v *)
WhichAlphaQ[v_]:=WhichAlphaQ[v]=Position[alphas,v][[1,1]];
WhichBetaQ[v_]:=WhichBetaQ[v]=Position[betacurves,v][[1,1]];
(* Preparation *)
alphacurves=alphacurvesInput[[All,2]];
alphaarcs=alphaarcsInput[[All,2]];
alphas=Join[alphaarcs,alphacurves];
betacurves=betacurvesInput[[All,2]];
NAC=Length[alphacurves];
NAA=Length[alphaarcs];
NA=NAC+NAA;
NB=Length[betacurves];
(* feedback *)
Print[If[DuplicateFreeQ[Flatten[CancellationSortListInput]],"No multiple cancellation. Good! ","ERROR: multiple cancellation. Bad! Check your input 'CancellationSortListInput'."]];
Print["Intersection points whose cancellation will be attempted last: ",DeleteCases[Range[1,NI],v_/;MemberQ[Flatten[CancellationSortListInput],v]]];
Print[If[MemberQ[Table[DuplicateFreeQ[WhichAlphaQ/@CancellationSortListInput[[i]]],{i,1,Length[CancellationSortListInput]}],True]==False,"Cancelling pairs belong to same \[Alpha]s. Good!","ERROR: Cancelling pairs belong to DIFFERENT \[Alpha]s. Bad!"]];
Print[If[MemberQ[Table[DuplicateFreeQ[WhichBetaQ/@CancellationSortListInput[[i]]],{i,1,Length[CancellationSortListInput]}],True]==False,"Cancelling pairs belong to same \[Beta]s. Good!","ERROR: Cancelling pairs belong to DIFFERENT \[Beta]s. Bad!"]];
Print[If[Sort[Flatten[alphas]]==Range[1,NI],"All intersection points lie on exactly one \[Alpha]-curve. Good!","ERROR: Some intersection points lie on multiple \[Alpha]-curves and/or none. BAD!"]];
Print[If[Sort[Flatten[betacurves]]==Range[1,NI],"All intersection points lie on exactly one \[Beta]-curve. Good!","ERROR: Some intersection points lie on multiple \[Beta]-curves and/or none. BAD!"]];
Print["NAC=#{\[Alpha]-circles}=",NAC];
Print["NAA=#{\[Alpha]-arcs}=",NAA];
Print["NA=#{\[Alpha]-curves}=",NA];
Print["NB=#{\[Beta]-circles}=",NB];
(* Prepare generation of generators *)
AtMostOneOnEachAlpha[v_]:=DuplicateFreeQ[v,WhichAlphaQ[#1]==WhichAlphaQ[#2]&];
NumberOfAlphaCurvesCorrect[v_]:=Length[Cases[v,vv_/;MemberQ[Flatten[alphacurves],vv]]]==NAC;
S1Boundary=If[TypeS1===TypeD,S1BoundaryInput, Reverse[S1BoundaryInput,{1,2}]];
S2Boundary=If[TypeS2===TypeD,S2BoundaryInput, Reverse[S2BoundaryInput,{1,2}]];
ArcDiagram1Matchings=If[TypeS1===TypeD,ArcDiagram1MatchingsInput,2*Length[ArcDiagram1MatchingsInput]+1-Reverse[#]&/@ ArcDiagram1MatchingsInput];
ArcDiagram2Matchings=If[TypeS2===TypeD,ArcDiagram2MatchingsInput,2*Length[ArcDiagram2MatchingsInput]+1-Reverse[#]&/@ ArcDiagram2MatchingsInput];
S1BoundaryInput::usage="Input data for algebra: List of regions for each line segment in arc diagram for S1, ordered from bottom to top. For computing a type A structure, this ordering is reversed automatically for S1Boundary. This is opposite to Zarevs conventions.";
S2BoundaryInput::usage="Input data for algebra: List of regions for each line segment in arc diagram for S2, ordered from bottom to top. For computing a type A structure, this ordering is reversed automatically for S2Boundary. This is opposite to Zarevs conventions. Set equal to {} if there is no S2.";
ArcDiagram1MatchingsInput::usage="list of pairs of endpoints of alpha arcs for S1, ordered by indices of alpha arcs. Pairs are ordered by indices of endpoints. Note that indices of alpha arcs of the S1 must be strictly smaller than those of S2. For computing a type A structure, the endpoint indices are automatically inverted for ArcDiagram1Matchings. This is opposite to Zarevs conventions.";
ArcDiagram2MatchingsInput::usage="list of pairs of endpoints of alpha arcs for S2, ordered by indices of alpha arcs. Pairs are ordered by indices of endpoints. Note that indices of alpha arcs of the S1 must be strictly smaller than those of S2. For computing a type A structure, the endpoint indices are automatically inverted for ArcDiagram2Matchings. This is opposite to Zarevs conventions. Set equal to {} if there is no S2.";
(* Auxilliary function which maps a level of the arc diagram to the index of the alpha arc *)
S1ListLevelToAlphaArcIndex=SortBy[Flatten[Table[{{ArcDiagram1Matchings[[i,1]],i},{ArcDiagram1Matchings[[i,2]],i}},{i,1,Length[ArcDiagram1Matchings]}],1],#[[1]]&][[All,2]];
S2ListLevelToAlphaArcIndex=SortBy[Flatten[Table[{{ArcDiagram2Matchings[[i,1]],i+Length[ArcDiagram1Matchings]},{ArcDiagram2Matchings[[i,2]],i+Length[ArcDiagram1Matchings]}},{i,1,Length[ArcDiagram2Matchings]}],1],#[[1]]&][[All,2]];
S1LevelToAlphaArcIndex[v_]:=S1ListLevelToAlphaArcIndex[[v]];
S2LevelToAlphaArcIndex[v_]:=S2ListLevelToAlphaArcIndex[[v]];
DBoundaryRegionsS1=Flatten[S1Boundary];
DBoundaryRegionsS2=Flatten[S2Boundary];
DBoundaryRegions=Join[DBoundaryRegionsS1,DBoundaryRegionsS2];
DBoundary1NeighbourPairs=Flatten[Table[{Part[#,i],Part[#,i+1]},{i,1,Length[#]-1}]&/@DeleteCases[S1Boundary,{}],1];
DBoundary2NeighbourPairs=If[S2Boundary=={},{},Flatten[Table[{Part[#,i],Part[#,i+1]},{i,1,Length[#]-1}]&/@DeleteCases[S2Boundary,{}],1]];
DBoundaryNeighbourPairs=Join[DBoundary1NeighbourPairs,DBoundary2NeighbourPairs];
(*  *)
N1E=Length[Flatten[ArcDiagram1Matchings]];
N2E=Length[Flatten[ArcDiagram2Matchings]];
N1E1=N1E+1;
N2E1=N2E+1;
N1E2=N1E+2;
N2E2=N2E+2;
No1MS=If[TypeS1===TypeD,N1E/2-NS1oaa,NS1oaa];
No2MS=If[TypeS2===TypeD,N2E/2-NS2oaa,NS2oaa];
N1NE1=N1E-No1MS;
N2NE1=N2E-No2MS;
S1AlwaysMovingArcs=If[TypeS1===TypeD,S1NeverOccupiedAlphaArcs,S1AlwaysOccupiedAlphaArcs];
S2AlwaysMovingArcs=If[TypeS2===TypeD,S2NeverOccupiedAlphaArcs,S2AlwaysOccupiedAlphaArcs];
S1PotMovingArcs=S1PotentiallyOccupiedAlphaArcs;
S2PotMovingArcs=S2PotentiallyOccupiedAlphaArcs;
S1NeverMovingArcs=If[TypeS1===TypeA,S1NeverOccupiedAlphaArcs,S1AlwaysOccupiedAlphaArcs];
S2NeverMovingArcs=If[TypeS2===TypeA,S2NeverOccupiedAlphaArcs,S2AlwaysOccupiedAlphaArcs];
(* check *)
If[Sort[Join[S1AlwaysOccupiedAlphaArcs,S2AlwaysOccupiedAlphaArcs,S1PotentiallyOccupiedAlphaArcs,S2PotentiallyOccupiedAlphaArcs,S1NeverOccupiedAlphaArcs,S2NeverOccupiedAlphaArcs]]=!=Range[1,NAA]||Length[S1AlwaysOccupiedAlphaArcs]>NS1oaa||Length[S2AlwaysOccupiedAlphaArcs]>NS2oaa,Print["Something is wrong! Check your input of \[Alpha]-arcs."]];
Print["Surface 1 (",TypeS1,"): #occupied \[Alpha]-arcs = ",NS1oaa,", #moving strands = ",No1MS];
(*Print["DBoundary1NeighbourPairs = ",MatrixForm[{MatrixForm/@DBoundary1NeighbourPairs}]];*)
If[ArcDiagram2Matchings=={},Print["There is only one glueing surface."],Print["Surface 2 (",TypeS2,"): #occupied \[Alpha]-arcs = ",NS2oaa,", #moving strands = ",No2MS]];
(*If[S2Boundary=!={},Print["DBoundary2NeighbourPairs = ",MatrixForm[{MatrixForm/@DBoundary2NeighbourPairs}]]]*)
];


Step1x1Algebra:=Module[{},
(* Definition of strands algebra.
(We define the glueing algebra by implementing the bigger strands algebra first and then constructing a basis for the glueing algebra from this. )

A single moving strands algebra element is a vector:

			v={Subscript[v, 1],...,Subscript[v, n],n+1,inv}.

Here, n is the number of levels in the arc diagram,  Subscript[v, i] is the endpoint of an active (=moving, but possibly constant) strand starting at i or (n+1) if no active strand starts at i, and inv is the number of inversions in this element. So the last two entries are actually superfluous, but they simplify the definition of composition  for this strands algebra.
*)
S1Reebpre1[z__]:=ReplacePart[ConstantArray[N1E1,N1E1],Table[{z}[[Floor[2i-1]]]->{z}[[2i]],{i,1,Length[{z}]/2}]];
S2Reebpre1[z__]:=ReplacePart[ConstantArray[N2E1,N2E1],Table[{z}[[Floor[2i-1]]]->{z}[[2i]],{i,1,Length[{z}]/2}]];
(* Select possible ends for completions *)
S1Reebpre2[z__]:=DeleteCases[ArcDiagram1Matchings,v_/;Or@@Table[MemberQ[v,{z}[[i]]],{i,1,Length[{z}]}]];
S2Reebpre2[z__]:=DeleteCases[ArcDiagram2Matchings,v_/;Or@@Table[MemberQ[v,{z}[[i]]],{i,1,Length[{z}]}]];
S1Reebpre3[v_,z__]:=Flatten[Map[Tuples,Subsets[v,{No1MS-Length[{z}]/2}],1],1];
S2Reebpre3[v_,z__]:=Flatten[Map[Tuples,Subsets[v,{No2MS-Length[{z}]/2}],1],1];
(* Do all completions *)
S1Reebpre4[z__]:=ReplacePart[S1Reebpre1[z],#]&/@(Thread[#->#]&/@S1Reebpre3[S1Reebpre2[z],z]);
S2Reebpre4[z__]:=ReplacePart[S2Reebpre1[z],#]&/@(Thread[#->#]&/@S2Reebpre3[S2Reebpre2[z],z]);
(* inversions *)
S1Inv[v_]:=Cases[Subsets[DeleteCases[Drop[v,-2],elem_/;elem==N1E1],{2}],elem_/;elem[[1]]>elem[[2]]];
S2Inv[v_]:=Cases[Subsets[DeleteCases[Drop[v,-2],elem_/;elem==N2E1],{2}],elem_/;elem[[1]]>elem[[2]]];
(* number of inversions *) 
S1NInv[v_]:=Count[Subsets[DeleteCases[Drop[v,-2],elem_/;elem==N1E1],{2}],elem_/;elem[[1]]>elem[[2]]];
S2NInv[v_]:=Count[Subsets[DeleteCases[Drop[v,-2],elem_/;elem==N2E1],{2}],elem_/;elem[[1]]>elem[[2]]];
(* Add last coordinate that records the number of inversions *)
S1Reeb[z__]:=Table[Join[S1Reebpre4[z][[i]],{S1NInv[S1Reebpre4[z][[i]]]}],{i,1,Length[S1Reebpre3[S1Reebpre2[z],z]]}];
S2Reeb[z__]:=Table[Join[S2Reebpre4[z][[i]],{S2NInv[S2Reebpre4[z][[i]]]}],{i,1,Length[S2Reebpre3[S2Reebpre2[z],z]]}];
(* Turn a single moving strands algebra element into list of active strands *)
S1ActiveStrands[v_]:=DeleteCases[Transpose[{Range[1,N1E],v[[1;;N1E]]}],elem_/;elem[[2]]==N1E1];
S2ActiveStrands[v_]:=DeleteCases[Transpose[{Range[1,N2E],v[[1;;N2E]]}],elem_/;elem[[2]]==N2E1];
(* Turn a single moving strands algebra element into a human readable form *)
MorH1Single[v_]:=If[v===1,Grid[{{1}}],Grid[Reverse[Table[If[S1ActiveStrands[v][[i,1]]==S1ActiveStrands[v][[i,2]],S1ActiveStrands[v][[i]],Map[Style[#,{Bold,Background->Yellow}]&,S1ActiveStrands[v][[i]]]],{i,1,No1MS}]],Frame->True]];
MorH2Single[v_]:=If[v===1,Grid[{{1}}],Grid[Reverse[Table[If[S2ActiveStrands[v][[i,1]]==S2ActiveStrands[v][[i,2]],S2ActiveStrands[v][[i]],Map[Style[#,{Bold,Background->Green}]&,S2ActiveStrands[v][[i]]]],{i,1,No2MS}]],Frame->True]];
(* Turn a moving strands algebra element into a human readable form *)
MorH1[v_]:=If[v==={},0,Total[MorH1Single/@v]];
MorH2[v_]:=If[v==={},0,Total[MorH2Single/@v]];
(* List of idempotents, as lists of occupied alpha arcs *)
S1IdempotentsAArcs=Sort[Join[S1AlwaysMovingArcs,#]]&/@Subsets[S1PotMovingArcs,{No1MS-Length[S1AlwaysMovingArcs]}];
S2IdempotentsAArcs=Sort[Join[S2AlwaysMovingArcs,#]]&/@Subsets[S2PotMovingArcs,{No2MS-Length[S2AlwaysMovingArcs]}];
(* numbers of idempotents for S12 and S2 separately *)
NS1Idem=Length[S1IdempotentsAArcs];
NS2Idem=Length[S2IdempotentsAArcs];
(* List of idempotents, as moving strands *)
S1IdempotentsAlg=Sort/@Map[Function[{d},Flatten[S1Reeb@@(Function[{c},Flatten[Transpose[{c,c}]]]@d)]],Tuples/@(ArcDiagram1Matchings[[#]]&/@S1IdempotentsAArcs),{2}];
S2IdempotentsAlg=If[S2Boundary==={},{{1}},Sort/@Map[Function[{d},Flatten[S2Reeb@@(Function[{c},Flatten[Transpose[{c,c}]]]@d)]],Tuples/@(ArcDiagram2Matchings[[#]]&/@(S2IdempotentsAArcs-N1E/2)),{2}]];
(* Find starting idempotent of a moving strands algebra element *)
S1StartingIdem[v_]:=If[MemberQ[S1IdempotentsAArcs,#],Position[S1IdempotentsAArcs,#][[1,1]],0]&@Sort[S1LevelToAlphaArcIndex/@S1ActiveStrands[v][[All,1]]];
S2StartingIdem[v_]:=If[MemberQ[S2IdempotentsAArcs,#],Position[S2IdempotentsAArcs,#][[1,1]],0]&@Sort[S2LevelToAlphaArcIndex/@S2ActiveStrands[v][[All,1]]];
(* Find ending idempotent of a moving strands algebra element *)
S1EndingIdem[v_]:=If[MemberQ[S1IdempotentsAArcs,#],Position[S1IdempotentsAArcs,#][[1,1]],0]&@Sort[S1LevelToAlphaArcIndex/@S1ActiveStrands[v][[All,2]]];
S2EndingIdem[v_]:=If[MemberQ[S2IdempotentsAArcs,#],Position[S2IdempotentsAArcs,#][[1,1]],0]&@Sort[S2LevelToAlphaArcIndex/@S2ActiveStrands[v][[All,2]]];
(* feedback *)
Print["Idempotents for S1 (occupied arcs in strands diagram):\n ",MatrixForm[Transpose[Table[{i,S1IdempotentsAArcs[[i]]},{i,1,Length[S1IdempotentsAArcs]}]]]];
Print["Idempotents for S2 (occupied arcs in strands diagram):\n ",MatrixForm[Transpose[Table[{i,S2IdempotentsAArcs[[i]]},{i,1,Length[S2IdempotentsAArcs]}]]]];
(* Definition of differential on strands algebra *)
(* Find starting points of a given inversion (inversion=output of S1Inv) *)
PosOfInv[v_,inversion_]:=FirstPosition[v,#][[1]]&/@inversion;
(* resolve a single inversion *)
S1DifferentialSingleInSingleOut[v_,inversion_]:=v+Normal[SparseArray[{#[[1]]->inversion[[2]]-inversion[[1]],#[[2]]->inversion[[1]]-inversion[[2]],N1E2->-1},N1E2,0]]&@PosOfInv[v,inversion];
S2DifferentialSingleInSingleOut[v_,inversion_]:=v+Normal[SparseArray[{#[[1]]->inversion[[2]]-inversion[[1]],#[[2]]->inversion[[1]]-inversion[[2]],N2E2->-1},N2E2,0]]&@PosOfInv[v,inversion];
(* differential on single moving strands algebra element *)
S1differentialSingle[v_]:=S1DifferentialSingleInSingleOut[v,#]&/@S1Inv[v];
S2differentialSingle[v_]:=S2DifferentialSingleInSingleOut[v,#]&/@S2Inv[v];
(* differential on strands algebra *)
S1differential[v_]:=Join@@(S1differentialSingle/@v);
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{"S2differential", "[", "v_", "]"}], ":=", 
RowBox[{"{", "}"}]}], 
RowBox[{"S2Boundary", "===", 
RowBox[{"{", "}"}]}]},
{
RowBox[{
RowBox[{"S2differential", "[", "v_", "]"}], ":=", 
RowBox[{"Join", "@@", 
RowBox[{"(", 
RowBox[{"S2differentialSingle", "/@", "v"}], ")"}]}]}], "\[Placeholder]"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Feedback function to calculate differential of moving strands *)
MorH1DifferentialSingle[v_]:=Print["d(",MorH1Single[v],") = ",MorH1[S1differentialSingle[v]]];
MorH2DifferentialSingle[v_]:=Print["d(",MorH2Single[v],") = ",MorH2[S2differentialSingle[v]]];
(* Definition of composition in strands algebra *)
S1AlgCompSinglePre[v1_,v2_]:=Join[Table[v2[[v1[[i]]]],{i,1,N1E}],{N1E1,v1[[N1E2]]+v2[[N1E2]]}];
S2AlgCompSinglePre[v1_,v2_]:=Join[Table[v2[[v1[[i]]]],{i,1,N2E}],{N2E1,v1[[N2E2]]+v2[[N2E2]]}];
S1Clean1[v_]:=If[S1NInv[v]==v[[N1E2]],v,0];
S2Clean1[v_]:=If[S2NInv[v]==v[[N2E2]],v,0];
S1Clean2[v_]:=If[Count[Drop[v,-2],N1E1]==N1NE1,S1Clean1[v],0];
S2Clean2[v_]:=If[Count[Drop[v,-2],N2E1]==N2NE1,S2Clean1[v],0];
(* Composition for single moving strands algebra elements *)
S1AlgCompSingle[v1_,v2_]:=If[v1===1,v2,If[v2===1, v1,S1Clean2[S1AlgCompSinglePre[v1,v2]]]];
S2AlgCompSingle[v1_,v2_]:=If[v1===1,v2,If[v2===1, v1,S2Clean2[S2AlgCompSinglePre[v1,v2]]]];
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{"S2AlgCompSingle", "[", 
RowBox[{"v1_", ",", "v2_"}], "]"}], ":=", "1"}], 
RowBox[{"S2Boundary", "===", 
RowBox[{"{", "}"}]}]},
{
RowBox[{
RowBox[{"S2AlgCompSingle", "[", 
RowBox[{"v1_", ",", "v2_"}], "]"}], ":=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{"v1", "===", "1"}], ",", "v2", ",", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{"v2", "===", "1"}], ",", " ", "v1", ",", 
RowBox[{"S2Clean2", "[", 
RowBox[{"S2AlgCompSinglePre", "[", 
RowBox[{"v1", ",", "v2"}], "]"}], "]"}]}], "]"}]}], "]"}]}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Composition for each algebra separately *)
S1AlgComp[b1_,b2_]:=If[b1==={},{},If[b2==={},{},DeleteCases[S1AlgCompSingle @@@Tuples[{b1,b2}],v_/;v===0]]];
S2AlgComp[b1_,b2_]:=If[b1==={},{},If[b2==={},{},DeleteCases[S2AlgCompSingle @@@Tuples[{b1,b2}],v_/;v===0]]];
(* Feedback function for algebra composition *)
MorH1AlgComp[b1_,b2_]:=Print["\!\(\*SubscriptBox[\(\[Mu]\), \(2\)]\)(",MorH1[b1]," ,\n",MorH1[b2],") =\n",MorH1[S1AlgComp[b1,b2]]];
MorH2AlgComp[b1_,b2_]:=Print["\!\(\*SubscriptBox[\(\[Mu]\), \(2\)]\)(",MorH2[b1]," ,\n",MorH2[b2],") =\n",MorH2[S2AlgComp[b1,b2]]];
];


Step1x2Algebra:=Module[{},
Print["S1: Idempotents (occupied \[Alpha]-arcs) and their names (from 'S1IdemNames'):\n ",{S1IdempotentsAArcs,S1IdemNames}//MatrixForm];
Print["S2: Idempotents (occupied \[Alpha]-arcs) and their names (from 'S2IdemNames'):\n ",{S2IdempotentsAArcs,S2IdemNames}//MatrixForm];
(* Construction of basis for each algebra separately. 

The basic idea is the following.
- Consider all possible single moving strands.
- Take all possible combinations of these.
- Form completions of these (i.e. apply S1Reeb).
- multiply these by idempotents and remove all zero algebra elements.
- Take union with idempotents; this is the basis.

*)
(* algebra elements corresponding to boundary regions *)
(* Note the instructions for DBoundary *)
S1ArcNumbering[i_,ii_]:=ii+i-1+Sum[Length[S1Boundary[[iii]]],{iii,1,i-1}];
S2ArcNumbering[i_,ii_]:=ii+i-1+Sum[Length[S2Boundary[[iii]]],{iii,1,i-1}];
(* Calculate all possible single moving strands. *)
S1BasicArcs=Flatten[Table[Table[Table[{S1ArcNumbering[i,ii],S1ArcNumbering[i,ii]+iii},{iii,1,Length[S1Boundary[[i]]]+1-ii}],{ii,1,Length[S1Boundary[[i]]]}],{i,1,Length[S1Boundary]}],2];
S2BasicArcs=Flatten[Table[Table[Table[{S2ArcNumbering[i,ii],S2ArcNumbering[i,ii]+iii},{iii,1,Length[S2Boundary[[i]]]+1-ii}],{ii,1,Length[S2Boundary[[i]]]}],{i,1,Length[S2Boundary]}],2];
(* Prep for next step *)
S1NotStartingAtSame[list_]:=DuplicateFreeQ[list[[All,1]],S1LevelToAlphaArcIndex[#1]==S1LevelToAlphaArcIndex[#2]&];
S2NotStartingAtSame[list_]:=DuplicateFreeQ[list[[All,1]],S2LevelToAlphaArcIndex[#1]==S2LevelToAlphaArcIndex[#2]&];
S1NotEndingAtSame[list_]:=DuplicateFreeQ[list[[All,2]],S1LevelToAlphaArcIndex[#1]==S1LevelToAlphaArcIndex[#2]&];
S2NotEndingAtSame[list_]:=DuplicateFreeQ[list[[All,2]],S2LevelToAlphaArcIndex[#1]==S2LevelToAlphaArcIndex[#2]&];
S1RinseMovingStrands[list_,element_]:=Join[list,Select[Map[Append[# ,element]&,list],(S1NotStartingAtSame[#]&&S1NotEndingAtSame[#])&]];
S2RinseMovingStrands[list_,element_]:=Join[list,Select[Map[Append[# ,element]&,list],(S2NotStartingAtSame[#]&&S2NotEndingAtSame[#])&]];
(* Take all possible combinations of single moving strands *)
S1BasicMovingStrands=Select[Flatten/@Drop[Fold[S1RinseMovingStrands,{{}},S1BasicArcs],1],Length[#]<=2*No1MS&];
S2BasicMovingStrands=Select[Flatten/@Drop[Fold[S2RinseMovingStrands,{{}},S2BasicArcs],1],Length[#]<=2*No2MS&];
(* Apply S1Reeb to all basic moving strands, i.e. form all completions  *)
S1BasicMovingStrandsS1ReeregionsInput=Parallelize[Apply[S1Reeb,S1BasicMovingStrands,2]];
S2BasicMovingStrandsS2ReeregionsInput=Parallelize[Apply[S2Reeb,S2BasicMovingStrands,2]];
(* delete those basic algebra moving strands that belong to wrong idempotents *)
S1BasicMovingStrandsS1ReebPre1=DeleteCases[Select[#,Function[{c},S1StartingIdem[c]!=0&&S1EndingIdem[c]!=0]]&/@S1BasicMovingStrandsS1ReeregionsInput,v_/;v==={}];
S2BasicMovingStrandsS2ReebPre1=DeleteCases[Select[#,Function[{c},S2StartingIdem[c]!=0&&S2EndingIdem[c]!=0]]&/@S2BasicMovingStrandsS2ReeregionsInput,v_/;v==={}];
(* Auxilliary functions for next step *)
S1PreAndPostMultiplyWithIdempotents[start_,alg_,end_]:=S1AlgComp[S1AlgComp[S1IdempotentsAlg[[start]],alg],S1IdempotentsAlg[[end]]];
S2PreAndPostMultiplyWithIdempotents[start_,alg_,end_]:=S2AlgComp[S2AlgComp[S2IdempotentsAlg[[start]],alg],S2IdempotentsAlg[[end]]];
(* S1AlgebraTable[[end,start]] gives list of algebra elements going from idempotent with index start to idempotent with index end *)
S1AlgebraTablePre=Parallelize[Table[Table[Sort[Sort/@DeleteCases[S1PreAndPostMultiplyWithIdempotents[startingIdempotent,#,endingIdempotent]&/@S1BasicMovingStrandsS1ReebPre1,v_/;v==={}]],{startingIdempotent, 1,Length[S1IdempotentsAArcs]}],{endingIdempotent, 1,Length[S1IdempotentsAArcs]}]];
(* same as above, but only for idempotents *)
S1IdemTable=Map[If[#==0,{},{S1IdempotentsAlg[[#]]}]&,DiagonalMatrix[Range[1,Length[S1IdempotentsAlg]]],{2}];
(* S1AlgebraTablePre and S1IdemTable combined *)
S1AlgebraTable=Join[S1IdemTable,S1AlgebraTablePre,3];
(* Feedback *)
Print["S1: number of algebra elements for each pair of idempotents = ",MatrixForm[Map[Length,S1AlgebraTable,{2}]]];
(* S2AlgebraTable[[end,start]] gives list of algebra elements going from idempotent with index start to idempotent with index end *)
S2AlgebraTablePre=Parallelize[Table[Table[Sort[Sort/@DeleteCases[S2PreAndPostMultiplyWithIdempotents[startingIdempotent,#,endingIdempotent]&/@S2BasicMovingStrandsS2ReebPre1,v_/;v==={}]],{startingIdempotent, 1,Length[S2IdempotentsAArcs]}],{endingIdempotent, 1,Length[S2IdempotentsAArcs]}]];
(* same as above, but only for idempotents *)
S2IdemTable=Map[If[#==0,{},{S2IdempotentsAlg[[#]]}]&,DiagonalMatrix[Range[1,Length[S2IdempotentsAlg]]],{2}];
(* S1AlgebraTablePre and S1IdemTable combined *)
S2AlgebraTable=Join[S2IdemTable,S2AlgebraTablePre,3];
(* Feedback *)
Print["S2: number of algebra elements for each pair of idempotents = ",MatrixForm[Map[Length,S2AlgebraTable,{2}]]];
(* Bases for glueing algebras *)
S1AlgebraBasis=Join[S1IdempotentsAlg,Flatten[S1AlgebraTablePre,2]];
S2AlgebraBasis=Join[S2IdempotentsAlg,Flatten[S2AlgebraTablePre,2]];
(* feedback *)
Print["Sanity checks for algebra basis: ",If[DuplicateFreeQ[S1AlgebraBasis],"S1 ok","S1 ERROR!"
],", ",If[DuplicateFreeQ[S2AlgebraBasis],"S2 ok","S2 ERROR!"
]];
(* same as S1AlgebraTable, but now, the algebra elements have been replaced by their indices in S1AlgebraBasis *)
S1AlgebraTableInd=Map[Position[S1AlgebraBasis,#][[1,1]]&,S1AlgebraTable,{3}];
S2AlgebraTableInd=Map[Position[S2AlgebraBasis,#][[1,1]]&,S2AlgebraTable,{3}];
S1IdemForAlgIndStart[v_]:=Position[S1AlgebraTableInd,If[IntegerQ[v],v,First[v]]][[1,2]];
S1IdemForAlgIndEnd[v_]:=Position[S1AlgebraTableInd,If[IntegerQ[v],v,Last[v]]][[1,1]];
S2IdemForAlgIndStart[v_]:=Position[S2AlgebraTableInd,If[IntegerQ[v],v,First[v]]][[1,2]];
S2IdemForAlgIndEnd[v_]:=Position[S2AlgebraTableInd,If[IntegerQ[v],v,Last[v]]][[1,1]];
S1IdemForAlgIndStart::usage="S1IdemForAlgIndStart[v] gives the starting idempotent of a basic algebra element in S1 or of a sequence of such elements.";
S1IdemForAlgIndEnd::usage="S1IdemForAlgIndStart[v] gives the ending idempotent of a basic algebra element in S1 or of a sequence of such elements.";
S2IdemForAlgIndStart::usage="S1IdemForAlgIndStart[v] gives the starting idempotent of a basic algebra element in S2 or of a sequence of such elements.";
S2IdemForAlgIndEnd::usage="S1IdemForAlgIndStart[v] gives the ending idempotent of a basic algebra element in S2 or of a sequence of such elements.";
(* Feedback functions *)
S1H[algebraindex_]:=Print[MorH1[S1AlgebraBasis[[algebraindex]]]];
S2H[algebraindex_]:=Print[MorH2[S2AlgebraBasis[[algebraindex]]]];
(* feedback *)
(*Print["S1AlgebraTableInd=",S1AlgebraTableInd//MatrixForm];
Print["S2AlgebraTableInd=",S2AlgebraTableInd//MatrixForm];*)
(* Multiplication table for S1 glueing algebra *)
S1MultTable=Parallelize[Table[Table[If[#=!={},Position[S1AlgebraBasis,Sort[#]][[1,1]],0]&@S1AlgComp[S1AlgebraBasis[[start]],S1AlgebraBasis[[end]]],{start,1,Length[S1AlgebraBasis]}],{end,1,Length[S1AlgebraBasis]}]];
(* Feedback *)
Print["There",If[#==1," is one basis element "," are "<>ToString[#]<>" basis elements "]&@Length[S1AlgebraBasis] ,"in the algebra for S1. The multiplication table looks like this: ",ArrayPlot[S1MultTable,ColorFunction->"Monochrome"]];

(* Multiplication table for S2 glueing algebra *)
S2MultTable=Parallelize[Table[Table[If[#=!={},Position[S2AlgebraBasis,Sort[#]][[1,1]],0]&@S2AlgComp[S2AlgebraBasis[[start]],S2AlgebraBasis[[end]]],{start,1,Length[S2AlgebraBasis]}],{end,1,Length[S2AlgebraBasis]}]];
(* Feedback *)
Print["There",If[#==1," is one basis element "," are "<>ToString[#]<>" basis elements "]&@Length[S2AlgebraBasis] ,"in the algebra for S2. The multiplication table looks like this: ",ArrayPlot[S2MultTable,ColorFunction->"Monochrome"]];
(* S1DiffList[[i]]= list of basis elements whose sum is equal to the differential of the ith algebra basis element; an empty list means that the differential is 0. *)
S1DiffList=Parallelize[Table[Select[If[Intersection[S1differential[S1AlgebraBasis[[i]]],S1AlgebraBasis[[#]]]=!={},#,0]&/@Range[1,Length[S1AlgebraBasis]],#!=0&],{i,1,Length[S1AlgebraBasis]}]];
(* Doublecheck if computation was correct *)
If[AllTrue[Range[1,Length[S1DiffList]],Sort[Join@@S1AlgebraBasis[[S1DiffList[[#]]]]]===Sort[S1differential[S1AlgebraBasis[[#]]]]&],Print["Sanity check for the differential on S1 algebra: ok."],Print["ERROR in computation of the differential on S1!"]];
(* Feedback *)
Print["There",If[#==1," is one basis element "," are "<>ToString[#]<>" basis elements "]&@Count[S1DiffList,v_/;v=!={}] ,"in the algebra for S1 whose differential is non-vanishing."];
(* S2DiffList[[i]]= list of basis elements whose sum is equal to the differential of the ith algebra basis element; an empty list means that the differential is 0. *)
S2DiffList=Parallelize[Table[Select[If[Intersection[S2differential[S2AlgebraBasis[[i]]],S2AlgebraBasis[[#]]]=!={},#,0]&/@Range[1,Length[S2AlgebraBasis]],#!=0&],{i,1,Length[S2AlgebraBasis]}]];
(* Doublecheck if computation was correct *)
If[AllTrue[Range[1,Length[S2DiffList]],Sort[Join@@S2AlgebraBasis[[S2DiffList[[#]]]]]===Sort[S2differential[S2AlgebraBasis[[#]]]]&],Print["Sanity check for the differential on S2 algebra: ok."],Print["ERROR in computation of the differential on S2!"]];
(* Feedback *)
Print["There",If[#==1," is one basis element "," are "<>ToString[#]<>" basis elements "]&@Count[S2DiffList,v_/;v=!={}] ,"in the algebra for S2 whose differential is non-vanishing."];
];


S1SubalgebraGensInput::usage="Matrix with each row corresponding to a generator of the subalgebra. Note that idempotents need not be included here, they are automatically added later. The columns correspond to the following
1) the Mathematica-nice name, which is a string, 
2) the TeX nice name, which is a string, 
3) starting idempotent, 
4) the moving strands of the algebra element and
5) the ending idempotent.
Note that the moving strand should use the same convention as in the input of the arc diagram. For type A computations, this is automatically reversed for S1SubalgebraGens. This is opposite to Zarev's conventions.";
S2SubalgebraGensInput::usage="Matrix with each row corresponding to a generator of the subalgebra. Note that idempotents need not be included here, they are automatically added later. The columns correspond to the following:
1) the Mathematica-nice name, which is a string, 
2) the TeX nice name, which is a string, 
3) starting idempotent, 
4) the moving strands of the algebra element and
5) the ending idempotent.
Note that the moving strand should use the same convention as in the input of the arc diagram. For type A computations, this is automatically reversed for S2SubalgebraGens. This is opposite to Zarev's conventions. Set this equal to {} if there is no S2.";
Step1x3Algebra:=Module[{},
Print["Note: If this step produces an error message, double-check 'S1SubalgebraGensInput' and 'S2SubalgebraGensInput'.\n If some of the entries of the lists below are empty, you have probably only mixed up the starting/ending idempotents."];
S1SubalgebraGens=If[TypeS1===TypeD,S1SubalgebraGensInput,MapAt[2*Length[ArcDiagram1Matchings]+1-Reverse[#]&, S1SubalgebraGensInput,{{All,4}}]];
S2SubalgebraGens=If[S2SubalgebraGensInput=={},{},If[TypeS2===TypeD,S2SubalgebraGensInput,MapAt[2*Length[ArcDiagram2Matchings]+1-Reverse[#]&, S2SubalgebraGensInput,{{All,4}}]]];
(* find indices of algebra elements in algebra basis *)
S1SubalgebraGensIndices= Join[Range[1,Length[S1IdempotentsAArcs]],Apply[First[First[Position[S1AlgebraBasis,Sort[S1PreAndPostMultiplyWithIdempotents[#1,S1Reeb@@#2,#3]]]]]&,S1SubalgebraGens[[All,3;;5]],1]];
(* feedback *)
Print["S1 algebra indices of the subalgebra generators = ",S1SubalgebraGensIndices];
(* Same for S2 *)
S2SubalgebraGensIndices= Join[Range[1,Length[S2IdempotentsAArcs]],Apply[First[First[Position[S2AlgebraBasis,Sort[S2PreAndPostMultiplyWithIdempotents[#1,S2Reeb@@#2,#3]]]]]&,S2SubalgebraGens[[All,3;;5]],1]];
(* feedback *)
Print["S2 algebra indices of the subalgebra generators = ",S2SubalgebraGensIndices];
];


Step1x4Algebra:=Module[{},
(* S1firstGensubset is a list of pairs {algindex i, Subscript[v, i]}, where Subscript[v, i] is a list of indices of rows in S1SubalgebraGens, namely the list of subalgebra generators that  *)
S1firstGensubset={S1SubalgebraGensIndices,{#}&/@Join[ConstantArray[0,Length[S1IdempotentsAArcs]],Range[1,Length[S1SubalgebraGens]]]};
S2firstGensubset={S2SubalgebraGensIndices,{#}&/@Join[ConstantArray[0,Length[S2IdempotentsAArcs]],Range[1,Length[S2SubalgebraGens]]]};
(* find all algebra elements that can be constructed by multiplying two algebra elements in gensubset  *)
TakeConvexHull[algebraMultTable_,gensubset_]:=Transpose[DeleteCases[DeleteDuplicatesBy[Flatten[Join[Map[{#}&,algebraMultTable[[gensubset[[1]],gensubset[[1]]]],{2}],Map[{#}&,Apply[Join,Tuples[{gensubset[[2]],{#}}]&/@gensubset[[2]],{2}],{2}],3],1],First],v_/;First[v]==0]];
(* Function for finding a basis of the subalgebra *)
SubalgebraBasis[algebraMultTable_,gensubset_]:=Module[{
gensubsetit=gensubset,
gensubsetprevit={{1},{{0}}}(* the first idempotent is always in the subalgebra  *)
},
While[Sort[First[gensubsetit]]!=Sort[First[gensubsetprevit]],
gensubsetprevit=gensubsetit;
gensubsetit=TakeConvexHull[algebraMultTable,gensubsetit]
];
Transpose[SortBy[Transpose[gensubsetit],First]]
];
(* Now find thise subbasis for each glueing algebra *)
S1SubalgebraBasisPre=SubalgebraBasis[S1MultTable,S1firstGensubset];
S2SubalgebraBasisPre=SubalgebraBasis[S2MultTable,S2firstGensubset];
S1SubalgebraBasis=S1SubalgebraBasisPre[[1]];
S2SubalgebraBasis=S2SubalgebraBasisPre[[1]];
(* Replacement rules for turning indices (in algebra basis) of subalgebra elements into Mathematica-nice names *)
S1RepRulesSubAlgebraNamesMathematica=Join[{0->""},(#->S1SubalgebraGens[[#,1]] )&/@Range[1,Length[S1SubalgebraGens]]];
S2RepRulesSubAlgebraNamesMathematica=Join[{0->""},(#->S2SubalgebraGens[[#,1]] )&/@Range[1,Length[S2SubalgebraGens]]];
(* Replacement rules for turning indices (in algebra basis) of subalgebra elements into TeX-nice names *)
S1RepRulesSubAlgebraNamesTeX=Join[{0->""},(#->S1SubalgebraGens[[#,2]] )&/@Range[1,Length[S1SubalgebraGens]]];
S2RepRulesSubAlgebraNamesTeX=Join[{0->""},(#->S2SubalgebraGens[[#,2]] )&/@Range[1,Length[S2SubalgebraGens]]];
(* lists of Mathematica nice names of subalgebra basis elements, ordered as in S1SubalgebraBasis *)
S1SubalgebraMathematicaNames=Join[Subscript[\[Iota], #]&/@Range[1,Length[S1IdempotentsAArcs]],Drop[StringJoin/@(S1SubalgebraBasisPre[[2]]/.S1RepRulesSubAlgebraNamesMathematica),Length[S1IdempotentsAArcs]]];
S2SubalgebraMathematicaNames=Join[Subscript[\[Iota], #]&/@Range[1,Length[S2IdempotentsAArcs]],Drop[StringJoin/@(S2SubalgebraBasisPre[[2]]/.S2RepRulesSubAlgebraNamesMathematica),Length[S2IdempotentsAArcs]]];
(* lists of Mathematica nice names of subalgebra basis elements, ordered as in S1SubalgebraBasis *)
S1SubalgebraTeXNames=Join[("\\iota_{"<>ToString[#]<>"}")&/@Range[1,Length[S1IdempotentsAArcs]],Drop[StringJoin/@(S1SubalgebraBasisPre[[2]]/.S1RepRulesSubAlgebraNamesTeX),Length[S1IdempotentsAArcs]]];
S2SubalgebraTeXNames=Join[("\\iota_{"<>ToString[#]<>"}")&/@Range[1,Length[S2IdempotentsAArcs]],Drop[StringJoin/@(S2SubalgebraBasisPre[[2]]/.S2RepRulesSubAlgebraNamesTeX),Length[S2IdempotentsAArcs]]];
(* Replacement rules for labelling of arrows in Mathematica-nice names for subalgebra basis elements *)
S1RepRuleLabels=Apply[#1->#2&,Transpose[{S1SubalgebraBasis,S1SubalgebraMathematicaNames}],{1}];
S2RepRuleLabels=Apply[#1->#2&,Transpose[{S2SubalgebraBasis,S2SubalgebraMathematicaNames}],{1}];
(* Convert an algebra element into human readable form by replacing those algebra basis elements in the subalgebra by letters *)
AlgebraToLabels[alg_]:=Apply[{#1/.S1RepRuleLabels,#2/.S2RepRuleLabels}&,alg,{1}];
(* List of all basic elements in subalgebra and their indices in S1AlgebraBasis *)
S1SubalgebraExplained:=Print["S1Subalgebra=",Map[MatrixForm,Map[{##}&,ReleaseHold[ArrayReshape[HoldForm/@Transpose[{S1SubalgebraBasis,S1SubalgebraBasis/.S1RepRuleLabels}],{Ceiling[Length[S1SubalgebraBasis]/10],Min[10,Length[S1SubalgebraBasis]]},{"",""}]],{3}],{2}]//MatrixForm];
S1SubalgebraExplained::usage="List of all basic elements in subalgebra and their indices in S1AlgebraBasis.";
S2SubalgebraExplained:=Print["S2Subalgebra=",Map[MatrixForm,Map[{##}&,ReleaseHold[ArrayReshape[HoldForm/@Transpose[{S2SubalgebraBasis,S2SubalgebraBasis/.S2RepRuleLabels}],{Ceiling[Length[S2SubalgebraBasis]/10],Min[10,Length[S2SubalgebraBasis]]},{"",""}]],{3}],{2}]//MatrixForm];
S2SubalgebraExplained::usage="List of all basic elements in subalgebra and their indices in S2AlgebraBasis.";
S1SubalgebraExplained
S2SubalgebraExplained
];


Step1x5Algebra:=Module[{},
(* For each type D domain, associate single moving strand.
Since type A domains are constructed by adding type D domains, each domains can be given a union of  moving strands. Then take completion, i.e. take S1Reeb of the moving strands. Then multiply with idempotents and find the resulting algebra element in the basis of all algebra elements. Associate to a domain the index of the algebra element 

An element in the glueing algebra is simply a list of indices of those basis elements whose sum is equal to the element. 
A label of an arrow from one generator to another is a list of elements of the form {a,b}, where a and b are basis elements of either the algebra or some tensor power of the algebra. In the former, it is a single index, in the latter, it is a list of indices. For example, {12,{5,19,2}} would be a typical value for {a,b} in a type DA structure. 

Note;
- An empty list as a label means that there is no arrow between the generators;
- "Identity morphisms" for type D structures are in fact idempotents, and we record them as such; Also, the Subscript[m, 1]-action for a type A side is recorded by idempotents. E.g. if 2 is an idempotent, an entry {12,2} in a labelling of an arrow in a type DA structure means that we pick up an algebra element 12 on the type D side if we go along that arrow.
*)

(* We want to compute modulo 2, so we need to make sure we acually do *)
Mod2[v_]:=Part[Select[Tally@v,OddQ[Part[#,2]]&],All,1];
(* Basic arrow compositions for type D and type A *)
S1IndexComp[a_,b_]:=S1MultTable[[b,a]];
S2IndexComp[a_,b_]:=S2MultTable[[b,a]];
TypeAComp[a_,b_]:=If[IntegerQ[a],b,If[IntegerQ[b], a,Join[a,b]]];
(* Composition of arrow labels *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{
RowBox[{"ArrowComp", "[", 
RowBox[{"a1_", ",", "a2_"}], "]"}], ":=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{
RowBox[{"a1", "===", 
RowBox[{"{", "}"}]}], "||", 
RowBox[{"a2", "===", 
RowBox[{"{", "}"}]}]}], ",", 
RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
RowBox[{"DeleteCases", "[", 
RowBox[{
RowBox[{"Mod2", "[", 
RowBox[{"Apply", "[", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"S1IndexComp", "@@", "#1"}], ",", 
RowBox[{"S2IndexComp", "@@", "#2"}]}], "}"}], "&"}], ",", "\[IndentingNewLine]", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{"a1", ",", "a2"}], "}"}], "]"}]}], ",", "1"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
RowBox[{"v_", "/;", 
RowBox[{
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "1", "]"}], "]"}], "==", "0"}], "||", 
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}], "==", "0"}]}]}]}], "]"}]}], "]"}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"ArrowComp", "[", 
RowBox[{"a1_", ",", "a2_"}], "]"}], ":=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{
RowBox[{"a1", "===", 
RowBox[{"{", "}"}]}], "||", 
RowBox[{"a2", "===", 
RowBox[{"{", "}"}]}]}], ",", 
RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
RowBox[{"DeleteCases", "[", 
RowBox[{
RowBox[{"Mod2", "[", 
RowBox[{"Apply", "[", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"S1IndexComp", "@@", "#1"}], ",", 
RowBox[{"TypeAComp", "@@", "#2"}]}], "}"}], "&"}], ",", "\[IndentingNewLine]", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{"a1", ",", "a2"}], "}"}], "]"}]}], ",", "1"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
RowBox[{"v_", "/;", 
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "1", "]"}], "]"}], "==", "0"}]}]}], "]"}]}], "]"}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"ArrowComp", "[", 
RowBox[{"a1_", ",", "a2_"}], "]"}], ":=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{
RowBox[{"a1", "===", 
RowBox[{"{", "}"}]}], "||", 
RowBox[{"a2", "===", 
RowBox[{"{", "}"}]}]}], ",", 
RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
RowBox[{"DeleteCases", "[", 
RowBox[{
RowBox[{"Mod2", "[", 
RowBox[{"Apply", "[", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"TypeAComp", "@@", "#1"}], ",", 
RowBox[{"S2IndexComp", "@@", "#2"}]}], "}"}], "&"}], ",", "\[IndentingNewLine]", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{"a1", ",", "a2"}], "}"}], "]"}]}], ",", "1"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
RowBox[{"v_", "/;", 
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}], "==", "0"}]}]}], "]"}]}], "]"}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"ArrowComp", "[", 
RowBox[{"a1_", ",", "a2_"}], "]"}], ":=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{
RowBox[{"a1", "===", 
RowBox[{"{", "}"}]}], "||", 
RowBox[{"a2", "===", 
RowBox[{"{", "}"}]}]}], ",", 
RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
RowBox[{"Mod2", "[", 
RowBox[{"Apply", "[", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"TypeAComp", "@@", "#1"}], ",", 
RowBox[{"TypeAComp", "@@", "#2"}]}], "}"}], "&"}], ",", "\[IndentingNewLine]", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{"a1", ",", "a2"}], "}"}], "]"}]}], ",", "1"}], "]"}], "]"}]}], "]"}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Addition of arrow labels addition *)
AlgAA[z__]:=Mod2[Join[z]];

(* Same code, but for computations with sparse matrixes, so entries must not be lists (so we turn them into strings). This works surprisingly well. *)
(* Composition of arrow labels *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{
RowBox[{"ArrowCompString", "[", 
RowBox[{"a1_", ",", "a2_"}], "]"}], ":=", 
RowBox[{
RowBox[{"ArrowCompString", "[", 
RowBox[{"a1", ",", "a2"}], "]"}], "=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{
RowBox[{"StringQ", "[", "a1", "]"}], "&&", 
RowBox[{"StringQ", "[", "a2", "]"}]}], ",", "\[IndentingNewLine]", 
RowBox[{
RowBox[{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"#", "===", 
RowBox[{"{", "}"}]}], ",", "0", ",", 
RowBox[{"ToString", "[", 
RowBox[{"#", ",", 
RowBox[{"FormatType", "->", "StandardForm"}]}], "]"}]}], "]"}], "&"}], "@", 
RowBox[{"DeleteCases", "[", 
RowBox[{
RowBox[{"Mod2", "[", 
RowBox[{"Apply", "[", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"S1IndexComp", "@@", "#1"}], ",", 
RowBox[{"S2IndexComp", "@@", "#2"}]}], "}"}], "&"}], ",", "\[IndentingNewLine]", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{
RowBox[{"ToExpression", "[", "a1", "]"}], ",", 
RowBox[{"ToExpression", "[", "a2", "]"}]}], "}"}], "]"}]}], ",", "1"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
RowBox[{"v_", "/;", 
RowBox[{
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "1", "]"}], "]"}], "==", "0"}], "||", 
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}], "==", "0"}]}]}]}], "]"}]}], ",", "0"}], "]"}]}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"ArrowCompString", "[", 
RowBox[{"a1_", ",", "a2_"}], "]"}], ":=", 
RowBox[{
RowBox[{"ArrowCompString", "[", 
RowBox[{"a1", ",", "a2"}], "]"}], "=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{
RowBox[{"StringQ", "[", "a1", "]"}], "&&", 
RowBox[{"StringQ", "[", "a2", "]"}]}], ",", "\[IndentingNewLine]", 
RowBox[{
RowBox[{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"#", "===", 
RowBox[{"{", "}"}]}], ",", "0", ",", 
RowBox[{"ToString", "[", 
RowBox[{"#", ",", 
RowBox[{"FormatType", "->", "StandardForm"}]}], "]"}]}], "]"}], "&"}], "@", 
RowBox[{"DeleteCases", "[", 
RowBox[{
RowBox[{"Mod2", "[", 
RowBox[{"Apply", "[", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"S1IndexComp", "@@", "#1"}], ",", 
RowBox[{"TypeAComp", "@@", "#2"}]}], "}"}], "&"}], ",", "\[IndentingNewLine]", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{
RowBox[{"ToExpression", "[", "a1", "]"}], ",", 
RowBox[{"ToExpression", "[", "a2", "]"}]}], "}"}], "]"}]}], ",", "1"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
RowBox[{"v_", "/;", 
RowBox[{
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "1", "]"}], "]"}], "==", "0"}], "||", 
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}], "==", "0"}]}]}]}], "]"}]}], ",", "0"}], "]"}]}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"ArrowCompString", "[", 
RowBox[{"a1_", ",", "a2_"}], "]"}], ":=", 
RowBox[{
RowBox[{"ArrowCompString", "[", 
RowBox[{"a1", ",", "a2"}], "]"}], "=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{
RowBox[{"StringQ", "[", "a1", "]"}], "&&", 
RowBox[{"StringQ", "[", "a2", "]"}]}], ",", "\[IndentingNewLine]", 
RowBox[{
RowBox[{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"#", "===", 
RowBox[{"{", "}"}]}], ",", "0", ",", 
RowBox[{"ToString", "[", 
RowBox[{"#", ",", 
RowBox[{"FormatType", "->", "StandardForm"}]}], "]"}]}], "]"}], "&"}], "@", 
RowBox[{"DeleteCases", "[", 
RowBox[{
RowBox[{"Mod2", "[", 
RowBox[{"Apply", "[", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"TypeAComp", "@@", "#1"}], ",", 
RowBox[{"S2IndexComp", "@@", "#2"}]}], "}"}], "&"}], ",", "\[IndentingNewLine]", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{
RowBox[{"ToExpression", "[", "a1", "]"}], ",", 
RowBox[{"ToExpression", "[", "a2", "]"}]}], "}"}], "]"}]}], ",", "1"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
RowBox[{"v_", "/;", 
RowBox[{
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "1", "]"}], "]"}], "==", "0"}], "||", 
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}], "==", "0"}]}]}]}], "]"}]}], ",", "0"}], "]"}]}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"ArrowCompString", "[", 
RowBox[{"a1_", ",", "a2_"}], "]"}], ":=", 
RowBox[{
RowBox[{"ArrowCompString", "[", 
RowBox[{"a1", ",", "a2"}], "]"}], "=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{
RowBox[{"StringQ", "[", "a1", "]"}], "&&", 
RowBox[{"StringQ", "[", "a2", "]"}]}], ",", "\[IndentingNewLine]", 
RowBox[{
RowBox[{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"#", "===", 
RowBox[{"{", "}"}]}], ",", "0", ",", 
RowBox[{"ToString", "[", 
RowBox[{"#", ",", 
RowBox[{"FormatType", "->", "StandardForm"}]}], "]"}]}], "]"}], "&"}], "@", 
RowBox[{"Mod2", "[", 
RowBox[{"Apply", "[", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"TypeAComp", "@@", "#1"}], ",", 
RowBox[{"TypeAComp", "@@", "#2"}]}], "}"}], "&"}], ",", "\[IndentingNewLine]", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{
RowBox[{"ToExpression", "[", "a1", "]"}], ",", 
RowBox[{"ToExpression", "[", "a2", "]"}]}], "}"}], "]"}]}], ",", "1"}], "]"}], "]"}]}], ",", "0"}], "]"}]}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* preparation for addition of arrow labels addition *)
AlgAAString[z1_,z2_]:=AlgAAString[z1,z2]=If[IntegerQ[z1],z2,If[IntegerQ[z2],z1, If[#==={},0,ToString[#,FormatType->StandardForm]]&@Mod2[Join[ToExpression[z1],ToExpression[z2]]]]];
(* addition of arrow labels addition *)
AlgAASparse[v_]:=AlgAASparse[v]=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{"v", 
RowBox[{
RowBox[{"Head", "[", "v", "]"}], "===", "Integer"}]},
{"v", 
RowBox[{
RowBox[{"Head", "[", "v", "]"}], "===", "String"}]},
{"0", 
RowBox[{
RowBox[{"Head", "[", "v", "]"}], "===", "Times"}]},
{
RowBox[{"AlgAAString", "[", 
RowBox[{
RowBox[{"v", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", 
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "]"}], 
RowBox[{
RowBox[{"Head", "[", "v", "]"}], "===", "Plus"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
AlgAASparse[0]=0;
(* Find list of alpha arcs that a generator with index genindex occupies *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{"S1WhichAlphaQ0", "[", "gen_", "]"}], ":=", 
RowBox[{"Sort", "[", 
RowBox[{"Select", "[", 
RowBox[{
RowBox[{"WhichAlphaQ", " ", "/@", " ", "gen"}], ",", " ", 
RowBox[{
RowBox[{"#", " ", "<=", " ", 
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}]}], " ", "&"}]}], "]"}], "]"}]}], 
RowBox[{"TypeS1", "===", "TypeA"}]},
{
RowBox[{
RowBox[{"S1WhichAlphaQ0", "[", "gen_", "]"}], ":=", 
RowBox[{"Sort", "[", 
RowBox[{"Complement", "[", 
RowBox[{
RowBox[{"Range", "[", 
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}], "]"}], ",", 
RowBox[{"Select", "[", 
RowBox[{
RowBox[{"WhichAlphaQ", " ", "/@", " ", "gen"}], ",", " ", 
RowBox[{
RowBox[{"#", " ", "<=", " ", 
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}]}], " ", "&"}]}], "]"}]}], "]"}], "]"}]}], 
RowBox[{"TypeS1", "===", "TypeD"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{"S2WhichAlphaQ0", "[", "gen_", "]"}], ":=", 
RowBox[{"Sort", "[", 
RowBox[{"Select", "[", 
RowBox[{
RowBox[{"WhichAlphaQ", " ", "/@", " ", "gen"}], ",", " ", 
RowBox[{
RowBox[{"(", 
RowBox[{
RowBox[{"#", " ", ">", " ", 
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}]}], " ", "&&", " ", 
RowBox[{"#", " ", "<=", " ", "\n", "      ", 
RowBox[{
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}], " ", "+", " ", 
RowBox[{"Length", "[", "ArcDiagram2Matchings", "]"}]}]}]}], ")"}], " ", "&"}]}], "]"}], "]"}]}], 
RowBox[{"TypeS2", "===", "TypeA"}]},
{
RowBox[{
RowBox[{"S2WhichAlphaQ0", "[", "gen_", "]"}], ":=", 
RowBox[{"Sort", "[", 
RowBox[{"Complement", "[", 
RowBox[{
RowBox[{"Range", "[", 
RowBox[{
RowBox[{
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}], "+", "1"}], ",", 
RowBox[{
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}], " ", "+", " ", 
RowBox[{"Length", "[", "ArcDiagram2Matchings", "]"}]}]}], "]"}], ",", 
RowBox[{"Select", "[", 
RowBox[{
RowBox[{"WhichAlphaQ", " ", "/@", " ", "gen"}], ",", " ", 
RowBox[{
RowBox[{"(", 
RowBox[{
RowBox[{"#", " ", ">", " ", 
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}]}], " ", "&&", " ", 
RowBox[{"#", " ", "<=", " ", "\n", "      ", 
RowBox[{
RowBox[{"Length", "[", "ArcDiagram1Matchings", "]"}], " ", "+", " ", 
RowBox[{"Length", "[", "ArcDiagram2Matchings", "]"}]}]}]}], ")"}], " ", "&"}]}], "]"}]}], "]"}], "]"}]}], 
RowBox[{"TypeS2", "===", "TypeD"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
S1WhichAlphaQ[gen_]:=If[IntegerQ[gen],S1WhichAlphaQ0[generators[[gen]]],S1WhichAlphaQ0[gen]];
S2WhichAlphaQ[gen_]:=If[IntegerQ[gen],S2WhichAlphaQ0[generators[[gen]]],S2WhichAlphaQ0[gen]];
(* Find index of idempotent (=occupied alpha arcs) for a generator with index genindex *)
S1IdemForGenQ[gen_]:=If[MemberQ[##],Position[##][[1,1]],0]&@@{S1IdempotentsAArcs,S1WhichAlphaQ[gen]};
S2IdemForGenQ[gen_]:=If[MemberQ[##],Position[##][[1,1]],0]&@@{S2IdempotentsAArcs,S2WhichAlphaQ[gen]};
S1IdemForGenQ::usage="S1IdemForGenQ[gen_] computes the idempotent of a generator. Input is either the index of the generator in 'generators' or a list of intersection points defining the generator. ";
S2IdemForGenQ::usage="S2IdemForGenQ[gen_] computes the idempotent of a generator. Input is either the index of the generator in 'generators' or a list of intersection points defining the generator. ";
];


Step2Generators:=Module[{},
(* memory efficient generation of generators *)
Gpre0={#}&/@betacurves[[1]]; (*  Length[betacurves] *)
Monitor[ibeta=2;
For[ibeta=2,ibeta<=Length[betacurves],ibeta++,
Gpre0=Flatten[Parallelize[Outer[(Function[{c},If[Function[{d},DuplicateFreeQ[d]&&RelevantGeneratorGenerationQ[d]]@(WhichAlphaQ/@c),{c},{}]]@Append[#1,#2])&,
Gpre0,betacurves[[ibeta]],1,1]],2];
],"Looking at \[Beta]-curve "<>ToString[ibeta]<>"/"<>ToString[Length[betacurves]]<>". Current length of Gpre0: "<>ToString[Length[Gpre0]]<>"."];
generatorsUnordd=Select[Gpre0,NumberOfAlphaCurvesCorrect];
(* Feedback generating generators *)
Print["Number of prefiltered (RelevantGeneratorGenerationQ) generators = ",Length[generatorsUnordd]];
(* Ordering the generators *)
CSortListX=Join[Table[{___,CancellationSortListInput[[i,1]],___}->i,{i,1,Length[CancellationSortListInput]}],{Normal[ConstantArray[__,NB]]->Length[CancellationSortListInput]+1}];
CSortListY=Join[Table[{___,CancellationSortListInput[[i,1]],___}->i,{i,1,Length[CancellationSortListInput]}],{Normal[ConstantArray[__,NB]]->Length[CancellationSortListInput]+1}];
MysortX[v_]:=v/.CSortListX;
MysortY[v_]:=v/.CSortListY;
generatorsX=SortBy[generatorsUnordd,MysortX];
generatorsY=SortBy[generatorsUnordd,MysortY];
generatorsYnew=generatorsX/.Table[{a___,CancellationSortListInput[[i,1]],b___}->{a,CancellationSortListInput[[i,2]],b},{i,1,Length[CancellationSortListInput]}];
generatorsH=DeleteDuplicates[Flatten[Transpose[{generatorsYnew,generatorsX}],1]];
(* Select the relevant idempotents *)
generatorsHrel=Cases[generatorsH,v_/;RelevantGeneratorQ[v]];
(* Translate human readable generators into vectors of 0s and 1s *)
GenToVector[v_]:=Normal[SparseArray[v->ConstantArray[1,NB],NI,0]];
VectorToGen[v_]:=SortBy[Flatten[Position[v,1]],WhichBetaQ];
generators=generatorsHrel;
(* Feedback Ordering *)
Print["Final number of generators = ",Length[generators],". The first (up to) 10 generators are = ",TableForm[{MatrixForm/@generators[[1;;Min[10,Length[generators]]]]}]];
CancelledIPs=Flatten[CancellationSortListInput];
SurvivingGeneratorsAfterFirstCancellation=DeleteCases[Range[Length[generators]],v_/;IntersectingQ[generators[[v]],CancelledIPs]];
Print["After the initial cancellation (de-niceification), we expect to have ",Length[SurvivingGeneratorsAfterFirstCancellation]," generators left."];
(* List of idempotents for generators *)
S1IdemForGen=S1IdemForGenQ/@Range[1,Length[generators]];
S2IdemForGen=S2IdemForGenQ/@Range[1,Length[generators]];
S1IdemNameForGenQ[gen_]:=ToString[S1IdemNames[[S1IdemForGenQ[gen]]]];
S2IdemNameForGenQ[gen_]:=ToString[S2IdemNames[[S2IdemForGenQ[gen]]]];
(* Checking if all generators belong to an allowed idempotent *)
If[MemberQ[S1IdemForGen,0],Print["ERROR! The set of idempotents is larger than you thought. Double-check definition of generatorsHrel and algebra input data!"]];
If[MemberQ[S2IdemForGen,0],Print["ERROR! The set of idempotents is larger than you thought. Double-check definition of generatorsHrel and algebra input data!"]];
Print["They will lie in the following idempotents:\n",Join[{{"idempotents","#{generators}"}},Tally[{S1IdemNameForGenQ[#],S2IdemNameForGenQ[#]}&/@SurvivingGeneratorsAfterFirstCancellation]]//MatrixForm];
];


Step3x1Domains:=Module[{},
(* BEGIN Domains *)
(* Strategy:
Find all domains containing a single region r at a time and then ignore this region r in the following iterations. Start with bigon regions and then the rest. In both cases, we generate all possible (1xn)-rectangles R containing region r, connected via alpha-edges. In the beta-direction, we consider the maximal such rectangle Bmax. Then for each R, we form a matrix build out of R and Bmax that imitates a grid-structure, with horizontal alpha-direction. (For bigon regions, we "double" R to make it symmetric with r in the centre.) Then we fill up the empty (0) entries of this matrix with adjacent regions (if they exist), otherwise ith -1's. Then we delete all rows containing -1's and take all possible intervals of rows containing r. *)
(* Find AlphaEdges 1) square, 2) Bigon, 3) Boundary Region *)
FindAlphaEdges[region_]:=Sort/@\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"{", 
RowBox[{
RowBox[{"regionsInput", "[", 
RowBox[{"[", 
RowBox[{"region", ",", 
RowBox[{"{", 
RowBox[{"2", ",", "3"}], "}"}]}], "]"}], "]"}], ",", 
RowBox[{"regionsInput", "[", 
RowBox[{"[", 
RowBox[{"region", ",", 
RowBox[{"{", 
RowBox[{"4", ",", "5"}], "}"}]}], "]"}], "]"}]}], "}"}], 
RowBox[{
RowBox[{"MemberQ", "[", 
RowBox[{
RowBox[{"regionsInput", "[", 
RowBox[{"[", "region", "]"}], "]"}], ",", "\[Placeholder]"}], "]"}], "==", "False"}]},
{
RowBox[{"{", 
RowBox[{
RowBox[{"{", 
RowBox[{"0", ",", 
RowBox[{"regionsInput", "[", 
RowBox[{"[", 
RowBox[{"region", ",", "2"}], "]"}], "]"}]}], "}"}], ",", 
RowBox[{"{", 
RowBox[{"0", ",", 
RowBox[{"regionsInput", "[", 
RowBox[{"[", 
RowBox[{"region", ",", "3"}], "]"}], "]"}]}], "}"}]}], "}"}], 
RowBox[{
RowBox[{"MemberQ", "[", 
RowBox[{
RowBox[{"regionsInput", "[", 
RowBox[{"[", "region", "]"}], "]"}], ",", "\[Placeholder]"}], "]"}], "&&", 
RowBox[{"MemberQ", "[", 
RowBox[{"DBoundaryRegions", ",", "region"}], "]"}]}]},
{
RowBox[{"{", 
RowBox[{"regionsInput", "[", 
RowBox[{"[", 
RowBox[{"region", ",", 
RowBox[{"{", 
RowBox[{"2", ",", "3"}], "}"}]}], "]"}], "]"}], "}"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Find BetaEdges 1) square, 2) Bigon and Boundary Region *)
FindBetaEdges[region_]:=Sort/@\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"{", 
RowBox[{
RowBox[{"regionsInput", "[", 
RowBox[{"[", 
RowBox[{"region", ",", 
RowBox[{"{", 
RowBox[{"3", ",", "4"}], "}"}]}], "]"}], "]"}], ",", 
RowBox[{"regionsInput", "[", 
RowBox[{"[", 
RowBox[{"region", ",", 
RowBox[{"{", 
RowBox[{"5", ",", "2"}], "}"}]}], "]"}], "]"}]}], "}"}], 
RowBox[{
RowBox[{"MemberQ", "[", 
RowBox[{
RowBox[{"regionsInput", "[", 
RowBox[{"[", "region", "]"}], "]"}], ",", "\[Placeholder]"}], "]"}], "==", "False"}]},
{
RowBox[{"{", 
RowBox[{"regionsInput", "[", 
RowBox[{"[", 
RowBox[{"region", ",", 
RowBox[{"{", 
RowBox[{"2", ",", "3"}], "}"}]}], "]"}], "]"}], "}"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Generate List of AlphaEdges *)
AlphaEdges=Sort@DeleteDuplicates[Flatten[FindAlphaEdges/@Range[1,NR],1]];
(* Generate List of BetaEdges *)
BetaEdges=Sort@DeleteDuplicates[Flatten[FindBetaEdges/@Range[1,NR],1]];
(* Lists of Alpha/Beta-edges for each region, ordered by regions *)
AlphaEdgesOfRegions=Map[Position[AlphaEdges,#][[1,1]]&,FindAlphaEdges/@Range[1,NR],{2}];
BetaEdgesOfRegions=Map[Position[BetaEdges,#][[1,1]]&,FindBetaEdges/@Range[1,NR],{2}];
(* Test if two regins share an edge *)
AlphaAdjacentRegionsQ[region1_,region2_]:=
If[region1==region2,0,If[DuplicateFreeQ[Flatten[AlphaEdgesOfRegions[[{region1,region2}]]]],0,1]];
BetaAdjacentRegionsQ[region1_,region2_]:=
If[region1==region2,0,If[DuplicateFreeQ[Flatten[BetaEdgesOfRegions[[{region1,region2}]]]],0,1]];
(* Create Matrix showing which regions have common alpha/beta-edges; 1 means edge, 0 means no edge *)
AlphaAdjacentRegionsMatrix=Table[Table[AlphaAdjacentRegionsQ[i,ii],{ii,1,NR}],{i,1,NR}];
BetaAdjacentRegionsMatrix=Table[Table[BetaAdjacentRegionsQ[i,ii],{ii,1,NR}],{i,1,NR}];
AlphaAdjacentRegionsMatrix//MatrixForm;
BetaAdjacentRegionsMatrix//MatrixForm;
FindAlphaAdjacentRegions[region_,butnotoneofthese_]:=Complement[If[#=={},{},Flatten[#]]&@Position[AlphaAdjacentRegionsMatrix[[region]],1],butnotoneofthese];
FindBetaAdjacentRegions[region_,butnotoneofthese_]:=Complement[If[#=={},{},Flatten[#]]&@Position[BetaAdjacentRegionsMatrix[[region]],1],butnotoneofthese];
FindNextAlphaAdjacentRegions[region_,butnotthisregion_,butnotoneofthese_]:=If[{}==#,{},#]&@Complement[FindAlphaAdjacentRegions[region,butnotoneofthese],{butnotthisregion}];
FindNextBetaAdjacentRegions[region_,butnotthisregion_,butnotoneofthese_]:=If[{}==#,{},#]&@Complement[FindBetaAdjacentRegions[region,butnotoneofthese],{butnotthisregion}];
(* find all (1xn)-rectangles that involve the input region; we expand in alpha-edge direction *)
(* The region r is inserted at the second position, so for bigons, r is always at the last position of MaxRectangle1xAlpha. *)
MaxRectangle1xAlpha[region_,butnotoneofthese_]:=Module[{r=region,l=FindAlphaAdjacentRegions[region,butnotoneofthese],counter=0},
If[l=={},{r},
l=Insert[l,r,2];
counter=Length[l];
While[Length[l]==counter,
l=Join[FindNextAlphaAdjacentRegions[l[[1]],l[[2]],butnotoneofthese],l];
counter=counter+1];
counter=Length[l];
While[Length[l]==counter,
l=Join[l,FindNextAlphaAdjacentRegions[l[[-1]],l[[-2]],butnotoneofthese]];
counter=counter+1];
l
]];
MaxRectangle1xBeta[region_,butnotoneofthese_]:=Module[{r=region,l=FindBetaAdjacentRegions[region,butnotoneofthese],counter=0},
If[l=={},{r},
l=Insert[l,r,2];
counter=Length[l];
While[Length[l]==counter,
l=Join[FindNextBetaAdjacentRegions[l[[1]],l[[2]],butnotoneofthese],l];
counter=counter+1];
counter=Length[l];
While[Length[l]==counter,
l=Join[l,FindNextBetaAdjacentRegions[l[[-1]],l[[-2]],butnotoneofthese]];
counter=counter+1];
l
]];
(* Every entry in here (=rectangle) becomes a row in its gridmatrix for rectangles *)
Rectangles1xAlpha[region_,butnotoneofthese_]:=SequenceCases[#,{___,region,___},Overlaps->All]&@MaxRectangle1xAlpha[region,butnotoneofthese];
(* Every entry in here (=rectangle) becomes a column in its gridmatrix for rectangles and bigons *)
Rectangles1xBeta[region_,butnotoneofthese_]:=SequenceCases[#,{___,region,___},Overlaps->All]&@MaxRectangle1xBeta[region,butnotoneofthese];
(* Every entry in here (=rectangle) becomes the last row in its gridmatrix for bigons; we make it symmetric, so we need to delete duplicate regions at the end. *)
BigonsAlphaxRxAlpha[region_,butnotoneofthese_]:=Join[Drop[#,-1],Reverse[#]]&/@SequenceCases[MaxRectangle1xAlpha[region,butnotoneofthese],{___,region},Overlaps->All];

(* Preparation for FullGridMatrix *)
(*vector with 1s at positions in v of length iii*)
ToVector[v_,iii_]:=ConstantArray[0,iii]+Sum[Normal[SparseArray[{v[[i]]->1},iii,0]],{i,1,Length[v]}];
(* Find a common vertex of three regions *)
CommonVertex[r1_,r2_,r3_]:=Position[Normal[ToVector[{r1,r2,r3},NR].AbsB],3][[1,1]];
(* Each time we expand a bigon in beta-direction, there are two common vertices, and we must not choose the same one on either side *)
SecondCommonVertex[r1_,r2_,r3_]:=Last[Position[Normal[ToVector[{r1,r2,r3},NR].AbsB],3]][[1]];
(* alpha direction is horizontal;
(\[NoBreak]r1	?
r2	r3

\[NoBreak])
*)
(* Find a fourth region adjacent to the common vertex of three regions (if it exists) *)
Find4thRegion[r1_,r2_,r3_,butnotoneofthese_]:=If[MemberQ[{r1,r2,r3},-1],-1,
If[#=={},-1,If[AlphaAdjacentRegionsMatrix[[#[[1]],r1]]==1&&BetaAdjacentRegionsMatrix[[#[[1]],r3]]==1&&MemberQ[butnotoneofthese,#[[1]]]==False,#[[1]],-1]]&@
Flatten[Position[AbsB[[All,CommonVertex[r1,r2,r3]]]-ToVector[{r1,r2,r3},NR],1]]];
(* Each time we expand a bigon in beta-direction, there are two common vertices, and we must not choose the same one on either side *)
Find4thRegionBigonAlt[r1_,r2_,r3_,butnotoneofthese_]:=If[MemberQ[{r1,r2,r3},-1],-1,
If[#=={},-1,If[AlphaAdjacentRegionsMatrix[[#[[1]],r1]]==1&&BetaAdjacentRegionsMatrix[[#[[1]],r3]]==1&&MemberQ[butnotoneofthese,#[[1]]]==False,#[[1]],-1]]&@
Flatten[Position[AbsB[[All,SecondCommonVertex[r1,r2,r3]]]-ToVector[{r1,r2,r3},NR],1]]];
(* computes fullgridmatrix *)
(* rectangle takes form of a matrix alpha direction is horizontal=rows=1st level *)
FindAllBetaBigonExtensions[rectangle_,region_,butnotoneofthese_]:=Module[
{maxbeta=MaxRectangle1xBeta[region,butnotoneofthese],
crow,
ccol,
nrows,
ncols,
gridmatrix,
i,
ii,
maxgridmatrix,
output
},
crow=Position[maxbeta,region][[1,1]];
ccol=Position[rectangle[[1]],region][[1,1]];
nrows=Length[maxbeta];
ncols=Dimensions[rectangle][[2]];
gridmatrix=MapAt[Function[c,c/2],
PadRight[PadLeft[rectangle,{crow,ncols}],{nrows,ncols}]+PadRight[PadLeft[Transpose[{maxbeta}],{nrows,ccol}],{nrows,ncols}],
{crow,ccol}];
(* We want to fill up this matrix *)
(* upper left quadrant *)
For[i=crow-1,i>=1, i--,
For[ii=ccol-1,ii>=1, ii--,
(* alpha direction is horizontal;
(\[NoBreak]?	r1
r3	r2

\[NoBreak])
*)
gridmatrix[[i,ii]]=Find4thRegion[
gridmatrix[[i,ii+1]],
gridmatrix[[i+1,ii+1]],
gridmatrix[[i+1,ii]],
butnotoneofthese];
];
];
(* upper right quadrant *)
For[i=crow-1,i>=1, i--,
For[ii=ccol+1,ii<=ncols, ii++,
(* alpha direction is horizontal;
(\[NoBreak]r1	?
r2	r3

\[NoBreak])
*)
gridmatrix[[i,ii]]=Find4thRegionBigonAlt[
gridmatrix[[i,ii-1]],
gridmatrix[[i+1,ii-1]],
gridmatrix[[i+1,ii]],
butnotoneofthese];
];
];
(* gridmatrix is filled up; Now cut off all -1's *)
maxgridmatrix=DeleteCases[gridmatrix,v_/;MemberQ[v,-1]];
output=DeleteDuplicates/@(Flatten/@(Part[maxgridmatrix,#]&/@SequenceCases[Range[1,Length[maxgridmatrix]],{___,Position[maxgridmatrix,region][[1,1]],___},Overlaps->All]));
output
];
FindAllBetaRectangleExtensions[rectangle_,region_,butnotoneofthese_]:=Module[
{maxbeta=MaxRectangle1xBeta[region,butnotoneofthese],
crow,
ccol,
nrows,
ncols,
gridmatrix,
i,
ii,
maxgridmatrix,
output
},
crow=Position[maxbeta,region][[1,1]];
ccol=Position[rectangle[[1]],region][[1,1]];
nrows=Length[maxbeta];
ncols=Dimensions[rectangle][[2]];
gridmatrix=MapAt[Function[c,c/2],
PadRight[PadLeft[rectangle,{crow,ncols}],{nrows,ncols}]+PadRight[PadLeft[Transpose[{maxbeta}],{nrows,ccol}],{nrows,ncols}],
{crow,ccol}];
(* We want to fill up this matrix *)
(* lower right quadrant *)
For[i=crow+1,i<=nrows, i++,
For[ii=ccol+1,ii<=ncols, ii++,
(* alpha direction is horizontal;
(\[NoBreak]r2	r3
r1	?

\[NoBreak])
*)
gridmatrix[[i,ii]]=Find4thRegion[
gridmatrix[[i,ii-1]],
gridmatrix[[i-1,ii-1]],
gridmatrix[[i-1,ii]],
butnotoneofthese];
];
];
(* lower left quadrant *)
For[i=crow+1,i<=nrows, i++,
For[ii=ccol-1,ii>=1, ii--,
(* alpha direction is horizontal;
(\[NoBreak]r3	r2
?	r1

\[NoBreak])
*)
gridmatrix[[i,ii]]=Find4thRegion[
gridmatrix[[i,ii+1]],
gridmatrix[[i-1,ii+1]],
gridmatrix[[i-1,ii]],
butnotoneofthese];
];
];
(* upper left quadrant *)
For[i=crow-1,i>=1, i--,
For[ii=ccol-1,ii>=1, ii--,
(* alpha direction is horizontal;
(\[NoBreak]?	r1
r3	r2

\[NoBreak])
*)
gridmatrix[[i,ii]]=Find4thRegion[
gridmatrix[[i,ii+1]],
gridmatrix[[i+1,ii+1]],
gridmatrix[[i+1,ii]],
butnotoneofthese];
];
];
(* upper right quadrant *)
For[i=crow-1,i>=1, i--,
For[ii=ccol+1,ii<=ncols, ii++,
(* alpha direction is horizontal;
(\[NoBreak]r1	?
r2	r3

\[NoBreak])
*)
gridmatrix[[i,ii]]=Find4thRegion[
gridmatrix[[i,ii-1]],
gridmatrix[[i+1,ii-1]],
gridmatrix[[i+1,ii]],
butnotoneofthese];
];
];
(* gridmatrix is filled up; Now cut off all -1's *)
maxgridmatrix=DeleteCases[gridmatrix,v_/;MemberQ[v,-1]];
output=Flatten/@(Part[maxgridmatrix,#]&/@SequenceCases[Range[1,Length[maxgridmatrix]],{___,Position[maxgridmatrix,region][[1,1]],___},Overlaps->All]);
output
];
FindAllDomainsUsingThisBigonRegion[region_,butnotoneofthese_]:=FindAllBetaBigonExtensions[{#},region,butnotoneofthese]&/@BigonsAlphaxRxAlpha[region,butnotoneofthese];
FindAllDomainsUsingThisSquareRegion[region_,butnotoneofthese_]:=FindAllBetaRectangleExtensions[{#},region,butnotoneofthese]&/@Rectangles1xAlpha[region,butnotoneofthese];
RegionsSquares=(DeleteCases[#,v_/;MemberQ[v,\[Placeholder]]]&@regionsInput)[[All,1]];
RegionsAllButBigons=Sort[Join[RegionsSquares,DBoundaryRegions]];
RegionsBigons=Sort[Complement[Range[1,NR],RegionsAllButBigons]];
DomainsForTypeD=Module[
{
i,
region,
butnotoneofthese={},
domains={}
},
Monitor[
For[i=1,i<=Length[RegionsBigons],i++,
Pause[0.1];
region=RegionsBigons[[i]];
domains=Join[domains,FindAllDomainsUsingThisBigonRegion[region,butnotoneofthese]];
butnotoneofthese=Append[butnotoneofthese,region];
];
,
{{"Iteration counter for bigons (total="<>ToString[Length[RegionsBigons]]<>")" },{i}}//MatrixForm
];
Monitor[
For[i=1,i<=Length[RegionsAllButBigons],i++,
Pause[0.1];
region=RegionsAllButBigons[[i]];
domains=Join[domains,FindAllDomainsUsingThisSquareRegion[region,butnotoneofthese]];
butnotoneofthese=Append[butnotoneofthese,region];
];
,
{{"Iteration counter for squares (total="<>ToString[Length[RegionsAllButBigons]]<>")" },{i}}//MatrixForm
];
Flatten[domains,1]
];
(* lists of domains with a single boundary puncture *)
DpreS1typeD=Sort[Select[DomainsForTypeD,DisjointQ[DBoundaryRegionsS1,#]==False&]];
DpreS2typeD=Sort[Select[DomainsForTypeD,DisjointQ[DBoundaryRegionsS2,#]==False&]];
(* List of single boundary rectangles without the non-boundary regions, with region indices *)
Dpre3preS1Dint=Cases[#,v_/;MemberQ[DBoundaryRegionsS1,v]]&/@DpreS1typeD;
Dpre3preS2Dint=Cases[#,v_/;MemberQ[DBoundaryRegionsS2,v]]&/@DpreS2typeD;
(* domains without boundary *)
domainsOfTrueDifferentials=Complement[DomainsForTypeD,DpreS1typeD,DpreS2typeD];
Print["There are ", Length[DomainsForTypeD]," rectangles, bigons or boundary rectangles. Here is a random selection of up to 10 of these:",MatrixForm[Transpose[{#,DomainsForTypeD[[#]]}&/@RandomSample[Range[Length[DomainsForTypeD]],UpTo[10]]]]];
(* The following code is for type AA structures *)
(* Above, we have generated all bigons, squares, and bigons with a single puncture. For type A structures, what is missing is all domains that are unions of bigons with a single puncture. We generate these essentially by taking tuples of the bigons generated above *)

(* Replacement Rules: associateing a boundary region to its adjacent arc levels *)
S1ArcNumberingRepRule=Flatten[Table[Table[S1Boundary[[i,ii]]->{S1ArcNumbering[i,ii],S1ArcNumbering[i,ii]+1},{ii,1,Length[S1Boundary[[i]]]}],{i,1,Length[S1Boundary]}]];
S2ArcNumberingRepRule=Flatten[Table[Table[S2Boundary[[i,ii]]->{S2ArcNumbering[i,ii],S2ArcNumbering[i,ii]+1},{ii,1,Length[S2Boundary[[i]]]}],{i,1,Length[S2Boundary]}]];
(* list of start and end points of algebra element corresponding to single boundary rectangles *)
DpreS1typeDalg=Sort/@(DeleteCases[Tally[Flatten[#]],v_/;v[[2]]=!=1][[All,1]]&/@(Dpre3preS1Dint/.S1ArcNumberingRepRule));
DpreS2typeDalg=Sort/@(DeleteCases[Tally[Flatten[#]],v_/;v[[2]]=!=1][[All,1]]&/@(Dpre3preS2Dint/.S2ArcNumberingRepRule));
(* Identify those ends matched up by arc diagram, i.e. that belong to same alpha arc *)
AlgebraZeroS1aux=DpreS1typeDalg/.Flatten[{#[[1]]->#[[2]]}&/@ArcDiagram1Matchings];
AlgebraZeroS2aux=DpreS2typeDalg/.Flatten[{#[[1]]->#[[2]]}&/@ArcDiagram2Matchings];
(* This is True iff algebra elements of the regions specified (by their indices) start and end at different alpha-arcs *)
AlgebraZeroS1[listofDS1domainindices_]:=DuplicateFreeQ[Flatten[AlgebraZeroS1aux[[listofDS1domainindices]][[All,1]]]]&&DuplicateFreeQ[Flatten[AlgebraZeroS1aux[[listofDS1domainindices]][[All,2]]]];
AlgebraZeroS2[listofDS1domainindices_]:=DuplicateFreeQ[Flatten[AlgebraZeroS2aux[[listofDS1domainindices]][[All,1]]]]&&DuplicateFreeQ[Flatten[AlgebraZeroS2aux[[listofDS1domainindices]][[All,2]]]];
(* This is true if none of the boundary domains contain one another *)
NotContainedS1[listofDS1domainindices_]:=AnyTrue[DeleteCases[Tuples[DpreS1typeD[[listofDS1domainindices]],2],v_/;v[[1]]==v[[2]]],SubsetQ@@#&]==False;
NotContainedS2[listofDS1domainindices_]:=AnyTrue[DeleteCases[Tuples[DpreS2typeD[[listofDS1domainindices]],2],v_/;v[[1]]==v[[2]]],SubsetQ@@#&]==False;
(* List of intersection points that the single boundary rectangles connect *)
Dpre3preS1connectinggens=Flatten[Position[ToVector[#,NR].B,v_/;v!=0]]&/@DpreS1typeD;
Dpre3preS2connectinggens=Flatten[Position[ToVector[#,NR].B,v_/;v!=0]]&/@DpreS2typeD;
(* If two such domains have a common vertex, then there are at least 4 intersection points on one betacurve, so this cannot be part of a domain connecting two generators *)
XYnotthesameS1[listofdomainindices_]:=DuplicateFreeQ[Flatten[Dpre3preS1connectinggens[[listofdomainindices]]]];
XYnotthesameS2[listofdomainindices_]:=DuplicateFreeQ[Flatten[Dpre3preS2connectinggens[[listofdomainindices]]]];
(* all domains obtained by combining boundary domains that do not contain one another or whose algebra element is zero  *)
RinseS1[list_,element_]:=Monitor[Join[list,Select[Map[Append[# ,element]&,list],(NotContainedS1[#]&&AlgebraZeroS1[#]&&XYnotthesameS1[#])&]],"Calculating DpreS1typeA: "<>ToString[element]<>"/"<>ToString[Length[DpreS1typeD]]];
RinseS2[list_,element_]:=Monitor[Join[list,Select[Map[Append[# ,element]&,list],(NotContainedS2[#]&&AlgebraZeroS2[#]&&XYnotthesameS2[#])&]],"Calculating DpreS2typeA: "<>ToString[element]<>"/"<>ToString[Length[DpreS2typeD]]];
If[TypeS1===TypeA,DpreS1typeApre=Drop[Fold[RinseS1,{{}},Range[1,Length[DpreS1typeD]]],1]];
If[TypeS2===TypeA,DpreS2typeApre=Drop[Fold[RinseS2,{{}},Range[1,Length[DpreS2typeD]]],1]];
(* the above, but the domain indices have been replaced by actual lists of regions *)
If[TypeS1===TypeA,DpreS1typeA=Flatten/@Map[DpreS1typeD[[#]]&,DpreS1typeApre,{2}]];
If[TypeS2===TypeA,DpreS2typeA=Flatten/@Map[DpreS2typeD[[#]]&,DpreS2typeApre,{2}]];
Print["There are ",If[TypeS1===TypeA,Length[DpreS1typeA],Length[DpreS1typeD]]," domains with boundary on S1, ",If[TypeS2===TypeA,Length[DpreS2typeA],Length[DpreS2typeD]]," on S2 and ",Length[domainsOfTrueDifferentials]," without any boundary."];
(* backup *)
DumpSave[BackupFilePath<>"_AfterDomains1.mx","Global`"];
Print[" << backup saved at: " ,BackupFilePath<>"_AfterDomains1.mx >>"];
];


Step3x2Domains:=Module[{},
(* prep for next step *)
S1ArcNumberingRepRuleRevStartpre1=Join[Flatten[Table[Table[{S1ArcNumbering[i,ii],S1Boundary[[i,ii]]},{ii,1,Length[S1Boundary[[i]]]}],{i,1,Length[S1Boundary]}],1],
Flatten[Table[Table[{S1ArcNumbering[i,ii]+1,S1Boundary[[i,ii]]},{ii,1,Length[S1Boundary[[i]]]}],{i,1,Length[S1Boundary]}],1]];S2ArcNumberingRepRuleRevStartpre1=Join[Flatten[Table[Table[{S2ArcNumbering[i,ii],S2Boundary[[i,ii]]},{ii,1,Length[S2Boundary[[i]]]}],{i,1,Length[S2Boundary]}],1],
Flatten[Table[Table[{S2ArcNumbering[i,ii]+1,S2Boundary[[i,ii]]},{ii,1,Length[S2Boundary[[i]]]}],{i,1,Length[S2Boundary]}],1]];
(* list[[i]]= list of boundary region indices adjacent to level i of arc diagram *)
S1ArcNumberingRepRuleRevStart=Fold[Function[{table,v},Module[{t=table},t[[v[[1]]]]=Append[t[[v[[1]]]],v[[2]]];t]],ConstantArray[{},2*Length[ArcDiagram1Matchings]],S1ArcNumberingRepRuleRevStartpre1];
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"S2ArcNumberingRepRuleRevStart", "=", 
RowBox[{"{", "}"}]}], 
RowBox[{"S2Boundary", "===", 
RowBox[{"{", "}"}]}]},
{
RowBox[{"S2ArcNumberingRepRuleRevStart", "=", 
RowBox[{"Fold", "[", 
RowBox[{
RowBox[{"Function", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{"table", ",", "v"}], "}"}], ",", 
RowBox[{"Module", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{"t", "=", "table"}], "}"}], ",", 
RowBox[{
RowBox[{
RowBox[{"t", "[", 
RowBox[{"[", 
RowBox[{"v", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}], "=", 
RowBox[{"Append", "[", 
RowBox[{
RowBox[{"t", "[", 
RowBox[{"[", 
RowBox[{"v", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}], ",", 
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";", "t"}]}], "]"}]}], "]"}], ",", 
RowBox[{"ConstantArray", "[", 
RowBox[{
RowBox[{"{", "}"}], ",", 
RowBox[{"2", "*", 
RowBox[{"Length", "[", "ArcDiagram2Matchings", "]"}]}]}], "]"}], ",", "S2ArcNumberingRepRuleRevStartpre1"}], "]"}]}], "\[Placeholder]"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Algebra elements for type A *)
If[TypeS1===TypeA,DpreS1typeAalg=Flatten[Sort[#]]&/@Map[DpreS1typeDalg[[#]]&,DpreS1typeApre,{2}]];
If[TypeS2===TypeA,DpreS2typeAalg=Flatten[Sort[#]]&/@Map[DpreS2typeDalg[[#]]&,DpreS2typeApre,{2}]];
DpreTrueDiffsalg=ConstantArray[1,Length[domainsOfTrueDifferentials]];
(* list of starting levels of algebra elements corresponding to single boundary rectangles *)
Dpre3preS1Start=DpreS1typeDalg[[All,1]];
Dpre3preS2Start=DpreS2typeDalg[[All,1]];
(* List of adjacent regions of those starting points (still only for single boundary rectangles) *)
DpreS1typeDAdjRegToStarts=Dpre3preS1Start/.Table[i->S1ArcNumberingRepRuleRevStart[[i]],{i,1,Length[S1ArcNumberingRepRuleRevStart]}];
DpreS2typeDAdjRegToStarts=Dpre3preS2Start/.Table[i->S2ArcNumberingRepRuleRevStart[[i]],{i,1,Length[S2ArcNumberingRepRuleRevStart]}];
(* now do it for all type A domains, list[[i]]=List of adjacent regions of starts of domain with index i *)
If[TypeS1===TypeA,DpreS1typeAAdjRegToStarts=Flatten/@Map[DpreS1typeDAdjRegToStarts[[#]]&,DpreS1typeApre,{2}]];
If[TypeS2===TypeA,DpreS2typeAAdjRegToStarts=Flatten/@Map[DpreS2typeDAdjRegToStarts[[#]]&,DpreS2typeApre,{2}]];
(* generate empty list for true differentials *)
DpreTrueDiffsAdjRegToStarts=ConstantArray[{},Length[domainsOfTrueDifferentials]];
(* Lists for generation of algebra elements; entries are either 1, denoting an honest differential, or a list of moving strands to which S1Reeb can be applied. If a domain touches the boundary (and if so: which) is not recorded; this can be deduced later from the position in the list  *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"AllDomainsAlg", "=", 
RowBox[{"Join", "[", 
RowBox[{"DpreTrueDiffsalg", ",", "DpreS1typeDalg", ",", "DpreS2typeDalg"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeD"}], "&&", 
RowBox[{"TypeS2", "==", "TypeD"}]}]},
{
RowBox[{"AllDomainsAlg", "=", 
RowBox[{"Join", "[", 
RowBox[{"DpreTrueDiffsalg", ",", "DpreS1typeAalg", ",", "DpreS2typeDalg"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeA"}], "&&", 
RowBox[{"TypeS2", "==", "TypeD"}]}]},
{
RowBox[{"AllDomainsAlg", "=", 
RowBox[{"Join", "[", 
RowBox[{"DpreTrueDiffsalg", ",", "DpreS1typeDalg", ",", "DpreS2typeAalg"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeD"}], "&&", 
RowBox[{"TypeS2", "==", "TypeA"}]}]},
{
RowBox[{"AllDomainsAlg", "=", 
RowBox[{"Join", "[", 
RowBox[{"DpreTrueDiffsalg", ",", "DpreS1typeAalg", ",", "DpreS2typeAalg"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeA"}], "&&", 
RowBox[{"TypeS2", "==", "TypeA"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"AllDomainsAdjRegToStarts", "=", 
RowBox[{"Join", "[", 
RowBox[{"DpreTrueDiffsAdjRegToStarts", ",", "DpreS1typeDAdjRegToStarts", ",", "DpreS2typeDAdjRegToStarts"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeD"}], "&&", 
RowBox[{"TypeS2", "==", "TypeD"}]}]},
{
RowBox[{"AllDomainsAdjRegToStarts", "=", 
RowBox[{"Join", "[", 
RowBox[{"DpreTrueDiffsAdjRegToStarts", ",", "DpreS1typeAAdjRegToStarts", ",", "DpreS2typeDAdjRegToStarts"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeA"}], "&&", 
RowBox[{"TypeS2", "==", "TypeD"}]}]},
{
RowBox[{"AllDomainsAdjRegToStarts", "=", 
RowBox[{"Join", "[", 
RowBox[{"DpreTrueDiffsAdjRegToStarts", ",", "DpreS1typeDAdjRegToStarts", ",", "DpreS2typeAAdjRegToStarts"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeD"}], "&&", 
RowBox[{"TypeS2", "==", "TypeA"}]}]},
{
RowBox[{"AllDomainsAdjRegToStarts", "=", 
RowBox[{"Join", "[", 
RowBox[{"DpreTrueDiffsAdjRegToStarts", ",", "DpreS1typeAAdjRegToStarts", ",", "DpreS2typeAAdjRegToStarts"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeA"}], "&&", 
RowBox[{"TypeS2", "==", "TypeA"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"AllDomains", "=", 
RowBox[{"Join", "[", 
RowBox[{"domainsOfTrueDifferentials", ",", "DpreS1typeD", ",", "DpreS2typeD"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeD"}], "&&", 
RowBox[{"TypeS2", "==", "TypeD"}]}]},
{
RowBox[{"AllDomains", "=", 
RowBox[{"Join", "[", 
RowBox[{"domainsOfTrueDifferentials", ",", "DpreS1typeA", ",", "DpreS2typeD"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeA"}], "&&", 
RowBox[{"TypeS2", "==", "TypeD"}]}]},
{
RowBox[{"AllDomains", "=", 
RowBox[{"Join", "[", 
RowBox[{"domainsOfTrueDifferentials", ",", "DpreS1typeD", ",", "DpreS2typeA"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeD"}], "&&", 
RowBox[{"TypeS2", "==", "TypeA"}]}]},
{
RowBox[{"AllDomains", "=", 
RowBox[{"Join", "[", 
RowBox[{"domainsOfTrueDifferentials", ",", "DpreS1typeA", ",", "DpreS2typeA"}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "==", "TypeA"}], "&&", 
RowBox[{"TypeS2", "==", "TypeA"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(*Print["Checking that lengths of AllDomains, AllDomainsAlg and AllDomainsAdjRegToStarts are the same: ",If[Length[AllDomains]\[Equal]Length[AllDomainsAdjRegToStarts]&&Length[AllDomains]\[Equal]Length[AllDomainsAlg],"ok","ERROR! "<>ToString[ Length/@{AllDomains,AllDomainsAdjRegToStarts,AllDomainsAlg}] ]];*)
(* Now each domain has an index and we only refer to the domains by their indices *)
(* List of Euler numbers of all domains *)
EulerNumber4=Parallelize[If[Intersection[RegionsBigons,#]=={},0,2]&/@AllDomains];
(* multiplicities at generator intersection points *)
NxOrYCorners4[domainindex_,generatorindex_]:=ToVector[AllDomains[[domainindex]],NR].AbsB.ToVector[generators[[generatorindex]],NI];
(* length of partitions for S1 and S2; since we do not have any domains withboundary along both S1 and S2, this is 1 if there is any boundary and 0 otherwise *)
NRhos4=Parallelize[If[Intersection[DBoundaryRegions,#]=={},0,4]&/@AllDomains];
(* \[Iota](\[Rho]) *)
IotaRho4=Parallelize[(2*
(ToVector[AllDomainsAdjRegToStarts[[#]],NR].ToVector[AllDomains[[#]],NR])&/@Range[1,Length[AllDomains]])];
EulerNRhosIotaRho4=EulerNumber4+NRhos4-IotaRho4;
(* Add all three components above together *)
Index[domainindex_,xindex_,yindex_]:=NxOrYCorners4[domainindex,xindex]+NxOrYCorners4[domainindex,yindex]+EulerNRhosIotaRho4[[domainindex]];
Print["Computed indices for domains."];
(* Compute starting and ending intersection points of all domains *)
xpreForAllDomains=Parallelize[Flatten[Position[ToVector[AllDomains[[#]],NR].B,1]]&/@Range[1,Length[AllDomains]]];
Print["Computed starting intersection points of domains."];
ypreForAllDomains=Parallelize[Flatten[Position[ToVector[AllDomains[[#]],NR].B,-1]]&/@Range[1,Length[AllDomains]]];
(* Find all starting generators of domains; this can take quite a while *)
Print[Now];
Print["There are "<>ToString[Length[xpreForAllDomains]]<>" domains that I need to look at."];
xForAllDomainsF[c_]:={c[[1]],#}&/@Flatten[Position[generators,_List?(SubsetQ[#,c[[2]]]&),1]];
xForAllDomains=ParallelMap[xForAllDomainsF,Transpose[{Range[Length[xpreForAllDomains]],xpreForAllDomains}]];
Print["Computed starting generators for all domains."];
Print[Now];
(* rearrange xForAllDomains, so that it is now a list with list[[xindex]]=list of domains with starting generator xindex *)
PrintTemporary["Sorting domains by starting generators..."];DomainsForX=ToExpression/@SparseArray[#[[1,2]]->ToString[#[[All,1]]]&/@GatherBy[Flatten[xForAllDomains,1],Last],{Length[generators]},"{}"];
(* Given domain and starting generator, find end generator *)
yFromDomainAndX[domainindex_,xindex_]:=If[#=={},0,#[[1,1]]]&@Position[generators,SortBy[Join[Complement[generators[[xindex]],xpreForAllDomains[[domainindex]]],ypreForAllDomains[[domainindex]]],WhichBetaQ]]
(* Note: there is at most one domain connecting two generators (nice diagrams are totally admissible) 
We build a matrix from DomainsForX, by essentially replacing the list of domains for each xindex by a vector with vector[[yindex]]=domain connecting xindex and yindex with the correct index.
So the columns in StructureMapsDomains corrspond to x's, because we transpose.
*)
];


Step3x3Domains:=Module[{},
(* Compute Structure Maps as a Sparse Array *)
Print[Now];
Print["Calculation of the structure map. There are "<>ToString[Length[DomainsForX]]<>" generators."];
StructureMapsDomainsSparseArrayRules=Flatten[Parallelize@MapIndexed[
Apply[
Function[
{domainindex,yindex},
{yindex,#2[[1]]}-> If[Index[domainindex,#2[[1]],yindex]==4,domainindex,0]
],
Select[
Function[{domainindex},{domainindex,yFromDomainAndX[domainindex,#2[[1]]]}]/@#1,Function[{pair},pair[[2]]!=0]
],
1
]&,
DomainsForX]];
Print[If[DuplicateFreeQ[StructureMapsDomainsSparseArrayRules[[All,1]]],"Good: There are no multiple labels in the structure!","Problem: There are multiple labels on the same arrow! The definition of StructureMapsDomainsSparse in the next line assumes this never happens."]];
StructureMapsDomainsSparse=SparseArray[StructureMapsDomainsSparseArrayRules,{Length[generators],Length[generators]},0];
Print[Now];
];


Step3x4Domains:=Module[{},
(* Find those domains that contribute to structure map *)
AllRelDomainIndices=Complement[Sort[DeleteDuplicates[Flatten[StructureMapsDomainsSparse]]],{0}];
(* Replace domain Indices by new (relevant) domain indices in AllRelDomainIndices *)
DomainIndicesToRelOnes[v_]:=If[v==0,0,Position[AllRelDomainIndices,v][[1,1]]];
StructureMapsRelDomains=Map[DomainIndicesToRelOnes,StructureMapsDomainsSparse,{2}];
(* update various list of (now relevant) domains *)
AllRelDomains=Part[AllDomains,AllRelDomainIndices];
NRelHonestDiffs=Count[AllRelDomainIndices,v_/;v<=Length[DpreTrueDiffsalg]];
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{"NRelHonestAndS1Diffs", "=", 
RowBox[{"Count", "[", 
RowBox[{"AllRelDomainIndices", ",", 
RowBox[{"v_", "/;", 
RowBox[{"v", "<=", 
RowBox[{
RowBox[{"Length", "[", "DpreTrueDiffsalg", "]"}], "+", 
RowBox[{"Length", "[", "DpreS1typeDalg", "]"}]}]}]}]}], "]"}]}], ";"}], 
RowBox[{"TypeS1", "==", "TypeD"}]},
{
RowBox[{
RowBox[{"NRelHonestAndS1Diffs", "=", 
RowBox[{"Count", "[", 
RowBox[{"AllRelDomainIndices", ",", 
RowBox[{"v_", "/;", 
RowBox[{"v", "<=", 
RowBox[{
RowBox[{"Length", "[", "DpreTrueDiffsalg", "]"}], "+", 
RowBox[{"Length", "[", "DpreS1typeAalg", "]"}]}]}]}]}], "]"}]}], ";"}], 
RowBox[{"TypeS1", "==", "TypeA"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
AllRelDomainsAlg=Part[AllDomainsAlg,AllRelDomainIndices];
(* Feedback *)
Print["There are ",Length[AllRelDomainIndices]," domains that contribute to the structure maps."];
Print["There are ",NRelHonestDiffs," domains that contribute an honest differential to the structure maps, e.g. \n",{AllRelDomains[[1;;Min[15,NRelHonestDiffs]]]}//MatrixForm];
Print["There are ",NRelHonestAndS1Diffs-NRelHonestDiffs," domains that contribute an S1 algebra element to the structure maps, e.g. \n",{Join[{"Regions"},AllRelDomains[[NRelHonestDiffs+1;;Min[NRelHonestDiffs+7,NRelHonestAndS1Diffs]]]],Join[{"moving strands"},AllRelDomainsAlg[[NRelHonestDiffs+1;;Min[NRelHonestDiffs+7,NRelHonestAndS1Diffs]]]]}//MatrixForm];
Print["There are ",Length[AllRelDomainIndices]-NRelHonestAndS1Diffs," domains that contribute an S2 algebra element to the structure maps, e.g. \n",{Join[{"Regions"},AllRelDomains[[NRelHonestAndS1Diffs+1;;Min[NRelHonestAndS1Diffs+7,Length[AllRelDomains]]]]],Join[{"moving strands"},AllRelDomainsAlg[[NRelHonestAndS1Diffs+1;;Min[NRelHonestAndS1Diffs+7,Length[AllRelDomains]]]]]}//MatrixForm];
(* Find all relevant moving strands for each algebra *)
S1AllRelMovingStrands=Sort[DeleteDuplicates[AllRelDomainsAlg[[NRelHonestDiffs+1;;NRelHonestAndS1Diffs]]]];
S2AllRelMovingStrands=Sort[DeleteDuplicates[AllRelDomainsAlg[[NRelHonestAndS1Diffs+1;;]]]];
(* For each boundary domain, find the index of the corresponding moving strands *)
S1AllRelDomainsAlg=Parallelize[Position[S1AllRelMovingStrands,#][[1,1]]&/@AllRelDomainsAlg[[NRelHonestDiffs+1;;NRelHonestAndS1Diffs]]];
S2AllRelDomainsAlg=Parallelize[Position[S2AllRelMovingStrands,#][[1,1]]&/@AllRelDomainsAlg[[NRelHonestAndS1Diffs+1;;]]];
(* Same as above, but we have applied S1Reeb to it *)
S1AllRelMovingStrandsReeb=Parallelize[(S1Reeb@@#)&/@S1AllRelMovingStrands];
S2AllRelMovingStrandsReeb=Parallelize[(S2Reeb@@#)&/@S2AllRelMovingStrands];
Print["Note: If this calculation gives an error message 'Part 1 of {} does not exist', there are some algebra elements generated by domains, that do not show up in the glueing algebra. This should not happen. If it does, check your algebra input data."];
(* S1AlgebraTable[[end,start]] gives list of algebra elements going from idempotent with index start to idempotent with index end *)
S1RelAlgebraTablePre=Parallelize[Table[Table[If[#==={},0,Position[S1AlgebraBasis, Sort[#]][[1,1]]]&/@(Function[{c},S1PreAndPostMultiplyWithIdempotents[startingIdempotent,c,endingIdempotent]]/@S1AllRelMovingStrandsReeb),{startingIdempotent, 1,Length[S1IdempotentsAArcs]}],{endingIdempotent, 1,Length[S1IdempotentsAArcs]}]];
(* This table gives at Position [[PostMultIdemIndex,PreMultIdemIndex,i]] the algebra index of the basic algebra element which is the result of multiplying a moving strands algebra element of index i with the idempotents *)
(* Add brackets {...} if it is type A *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"S1RelAlgebraTable", "=", 
RowBox[{"Map", "[", 
RowBox[{
RowBox[{
RowBox[{"{", "#", "}"}], "&"}], ",", "S1RelAlgebraTablePre", ",", 
RowBox[{"{", "3", "}"}]}], "]"}]}], 
RowBox[{"TypeS1", "==", "TypeA"}]},
{
RowBox[{"S1RelAlgebraTable", "=", "S1RelAlgebraTablePre"}], 
RowBox[{"TypeS1", "==", "TypeD"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Feedback *)
Print[If[MemberQ[Total[S1RelAlgebraTablePre,2],0],"Some domains for S1 cannot contribute. Probably, something is WRONG!","OK. All domains for S1 can contribute."]];
(* S2AlgebraTable[[end,start]] gives list of algebra elements going from idempotent with index start to idempotent with index end *)
S2RelAlgebraTablePre=Parallelize[Table[Table[If[#==={},0,Position[S2AlgebraBasis, Sort[#]][[1,1]]]&/@(Function[{c},S2PreAndPostMultiplyWithIdempotents[startingIdempotent,c,endingIdempotent]]/@S2AllRelMovingStrandsReeb),{startingIdempotent, 1,Length[S2IdempotentsAArcs]}],{endingIdempotent, 1,Length[S2IdempotentsAArcs]}]];
(* Add brackets {...} if it is type A *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"S2RelAlgebraTable", "=", 
RowBox[{"Map", "[", 
RowBox[{
RowBox[{
RowBox[{"{", "#", "}"}], "&"}], ",", "S2RelAlgebraTablePre", ",", 
RowBox[{"{", "3", "}"}]}], "]"}]}], 
RowBox[{"TypeS2", "==", "TypeA"}]},
{
RowBox[{"S2RelAlgebraTable", "=", "S2RelAlgebraTablePre"}], 
RowBox[{"TypeS2", "==", "TypeD"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Feedback *)
Print[If[MemberQ[Total[S2RelAlgebraTablePre,2],0],"Some domains for S2 cannot contribute. Probably, something is WRONG!","OK. All domains for S2 can contribute."]];
(* Finally, putting things together:
Replace each entry (=domain index) in StructureMapsDomains by 
\[Piecewise]	{}	no arrow
{idem,idem}	honest diff
{algebra,idem}	diff with boundary on S1
{idem,algebra}	diff with boundary on S2

;
*)
StructureMapsAux1Sparse[startgen_,RelDomainIndex_,endgen_]:=
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{"0", 
RowBox[{"RelDomainIndex", "==", "0"}]},
{
RowBox[{"ToString", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{"{", 
RowBox[{
RowBox[{"S1IdemForGen", "[", 
RowBox[{"[", "startgen", "]"}], "]"}], ",", 
RowBox[{"S2IdemForGen", "[", 
RowBox[{"[", "startgen", "]"}], "]"}]}], "}"}], "}"}], ",", 
RowBox[{"FormatType", "->", "StandardForm"}]}], "]"}], 
RowBox[{"1", "<=", "RelDomainIndex", "<=", "NRelHonestDiffs"}]},
{
RowBox[{
RowBox[{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"#", "===", "0"}], ",", "0", ",", 
RowBox[{"ToString", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"S2IdemForGen", "[", 
RowBox[{"[", "startgen", "]"}], "]"}]}], "}"}], "}"}], ",", 
RowBox[{"FormatType", "->", "StandardForm"}]}], "]"}]}], "]"}], "&"}], "@", 
RowBox[{"S1RelAlgebraTable", "[", 
RowBox[{"[", 
RowBox[{
RowBox[{"S1IdemForGen", "[", 
RowBox[{"[", "endgen", "]"}], "]"}], ",", 
RowBox[{"S1IdemForGen", "[", 
RowBox[{"[", "startgen", "]"}], "]"}], ",", 
RowBox[{"S1AllRelDomainsAlg", "[", 
RowBox[{"[", 
RowBox[{"RelDomainIndex", "-", "NRelHonestDiffs"}], "]"}], "]"}]}], "]"}], "]"}]}], 
RowBox[{
RowBox[{"NRelHonestDiffs", "+", "1"}], "<=", "RelDomainIndex", "<=", "NRelHonestAndS1Diffs"}]},
{
RowBox[{
RowBox[{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"#", "===", "0"}], ",", "0", ",", 
RowBox[{"ToString", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{"{", 
RowBox[{
RowBox[{"S1IdemForGen", "[", 
RowBox[{"[", "startgen", "]"}], "]"}], ",", "#"}], "}"}], "}"}], ",", 
RowBox[{"FormatType", "->", "StandardForm"}]}], "]"}]}], "]"}], "&"}], "@", 
RowBox[{"S2RelAlgebraTable", "[", 
RowBox[{"[", 
RowBox[{
RowBox[{"S2IdemForGen", "[", 
RowBox[{"[", "endgen", "]"}], "]"}], ",", 
RowBox[{"S2IdemForGen", "[", 
RowBox[{"[", "startgen", "]"}], "]"}], ",", 
RowBox[{"S2AllRelDomainsAlg", "[", 
RowBox[{"[", 
RowBox[{"RelDomainIndex", "-", "NRelHonestAndS1Diffs"}], "]"}], "]"}]}], "]"}], "]"}]}], 
RowBox[{
RowBox[{"NRelHonestAndS1Diffs", "+", "1"}], "<=", "RelDomainIndex"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
Print[Now];
Print["Calculation of StructureMapsSparse."];
(* StructureMaps without identity action for type A sides *)
StructureMapsSparseWO1=SparseArray[(#[[1]]->StructureMapsAux1Sparse[#[[1,2]],#[[2]],#[[1,1]]])&/@ArrayRules[StructureMapsRelDomains],{Length[generators],Length[generators]},0];
Print[Now];
GenIndexFromNewIndex[sublist_,index_,target_:generators]:=Position[target,sublist[[index]]][[1,1]];
GenIndexFromNewIndex::usage="GenIndexFromNewIndex[sublist_,index_] finds the index of a generator in 'generators', given an index of a generator in a sublist of generatorsit. An optional parameter specifies the target index list, default is 'generators'.";
(* Add identity action for type A sides; this code is for graphical representation after cancellation *)
S1Action[genlist_]:=Table[Table[If[i==ii,{{{S1IdemForGenQ[genlist[[ii]]]},S2IdemForGenQ[genlist[[ii]]]}},{}],{ii,1,Length[genlist]}],{i,1,Length[genlist]}];
S2Action[genlist_]:=Table[Table[If[i==ii,{{S1IdemForGenQ[genlist[[ii]]],{S2IdemForGenQ[genlist[[ii]]]}}},{}],{ii,1,Length[genlist]}],{i,1,Length[genlist]}];
AddS1S2Action[genlist_]:=Module[
{S1Component,S2Component,l=Length[genlist]},\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"S1Component", "=", 
RowBox[{"S1Action", "[", "genlist", "]"}]}], 
RowBox[{"TypeS1", "===", "TypeA"}]},
{
RowBox[{"S1Component", "=", 
RowBox[{"ConstantArray", "[", 
RowBox[{
RowBox[{"{", "}"}], ",", 
RowBox[{"{", 
RowBox[{"l", ",", "l"}], "}"}]}], "]"}]}], 
RowBox[{"TypeS1", "===", "TypeD"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"S2Component", "=", 
RowBox[{"S2Action", "[", "genlist", "]"}]}], 
RowBox[{
RowBox[{"TypeS2", "===", "TypeA"}], "&&", 
RowBox[{"S2Boundary", "=!=", 
RowBox[{"{", "}"}]}]}]},
{
RowBox[{"S2Component", "=", 
RowBox[{"ConstantArray", "[", 
RowBox[{
RowBox[{"{", "}"}], ",", 
RowBox[{"{", 
RowBox[{"l", ",", "l"}], "}"}]}], "]"}]}], "\[Placeholder]"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
MapThread[AlgAA,{S1Component,S2Component},2]
];
(* Add identity action for type A sides *)
S1ActionList=SparseArray[Band[{1,1}]->Parallelize[ToString[{{{S1IdemForGenQ[#]},S2IdemForGenQ[#]}}]&/@Range[Length[generators]]],{Length[generators],Length[generators]},0];
S2ActionList=SparseArray[Band[{1,1}]->Parallelize[ToString[{{S1IdemForGenQ[#],{S2IdemForGenQ[#]}}}]&/@Range[Length[generators]]],{Length[generators],Length[generators]},0];
AddS1S2ActionSparse=Module[
{S1Component,S2Component,l=Length[generators]},\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"S1Component", "=", "S1ActionList"}], 
RowBox[{"TypeS1", "===", "TypeA"}]},
{
RowBox[{"S1Component", "=", 
RowBox[{"SparseArray", "[", 
RowBox[{
RowBox[{"{", "}"}], ",", 
RowBox[{"{", 
RowBox[{"l", ",", "l"}], "}"}], ",", "0"}], "]"}]}], 
RowBox[{"TypeS1", "===", "TypeD"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"S2Component", "=", "S2ActionList"}], 
RowBox[{
RowBox[{"TypeS2", "===", "TypeA"}], "&&", 
RowBox[{"S2Boundary", "=!=", 
RowBox[{"{", "}"}]}]}]},
{
RowBox[{"S2Component", "=", 
RowBox[{"SparseArray", "[", 
RowBox[{
RowBox[{"{", "}"}], ",", 
RowBox[{"{", 
RowBox[{"l", ",", "l"}], "}"}], ",", "0"}], "]"}]}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
Map[AlgAASparse,S1Component+S2Component,{2}]
];
PrintTemporary["Calculating identity actions..."];
StructureMapsSparse=Map[AlgAASparse,StructureMapsSparseWO1+AddS1S2ActionSparse,{2}];
(* Feedback *)
Print["This is what the upper left corner of 'StructureMapsWith1Action' looks like:\n",MatrixForm[StructureMapsSparse[[1;;Min[10,Length[generators]],1;;Min[10,Length[generators]]]]]];
(* Feedback function, gives list of all arrows starting at given generatorindex *)
ToStringMS[string_]:=StringReplace[ToString[string],{" "->"",","->"|","}"->")","{"->"("}];
SparsePosition[mat_,position_]:=Flatten[Drop[ArrayRules[Part[mat,##]&@@position][[All,1]],-1]];
SparseNiceFindAllArrowsStartingAt[xindex_]:=If[StructureMapsRelDomains[[All,xindex]]==ConstantArray[0,Length[generators]],Print["No arrows starting at generator "<>ToStringMS[xindex]<>"="<>ToStringMS[generators[[xindex]]]<>"."],Print@@Join[{"From "<>ToStringMS[xindex]<>"="<>ToStringMS[generators[[xindex]]]<>"      {regions} = Relevant Domain Index (Old Index)"},"\n To  "<>ToStringMS[#]<>"="<>ToStringMS[generators[[#]]]<>" via "<>ToStringMS[AllDomains[[StructureMapsDomainsSparse[[#,xindex]]]]]<>" = "<>ToStringMS[StructureMapsRelDomains[[#,xindex]]]<>" ("<>ToStringMS[StructureMapsDomainsSparse[[#,xindex]]]<>"), label ="<>ToStringMS[StructureMapsSparseWO1[[#,xindex]]]&/@SparsePosition[StructureMapsRelDomains,{All,xindex}]]];
SparseNiceFindAllArrowsEndingAt[yindex_]:=If[StructureMapsRelDomains[[yindex]]==ConstantArray[0,Length[generators]],Print["No arrows starting at generator "<>ToStringMS[yindex]<>"="<>ToStringMS[generators[[yindex]]]<>"."],Print@@Join[{"From "<>ToStringMS[yindex]<>"="<>ToStringMS[generators[[yindex]]]<>"      {regions} = Relevant Domain Index (Old Index)"},("\n From "<>ToStringMS[#]<>"="<>ToStringMS[generators[[#]]]<>" via "<>ToStringMS[AllDomains[[StructureMapsDomainsSparse[[yindex,#]]]]]<>" = "<>ToStringMS[StructureMapsRelDomains[[yindex,#]]]<>" ("<>ToStringMS[StructureMapsDomainsSparse[[yindex,#]]]<>"), label ="<>ToStringMS[StructureMapsSparseWO1[[yindex,#]]])&/@SparsePosition[StructureMapsRelDomains,{yindex,All}]]];
(* backup *)
DumpSave[BackupFilePath<>"_BeforeCancellation.mx","Global`"];
Print[" << backup saved at: " ,BackupFilePath<>"_BeforeCancellation.mx >>"];
];


Step4CancellationPreparation:=Module[{},
(* Find out if an algebra element (elementary label) represents an idempotent *)
IdentityAlgQ[v_]:=If[IntegerQ[v[[1]]]&&IntegerQ[v[[2]]],v[[1]]<=NS1Idem&&v[[2]]<=NS2Idem,False];
(* Find "identity" labels *)
IdentityQString[v_]:=IdentityQString[v]=If[IntegerQ[v],False,If[Length[#]==1,#[[1,1]]<=NS1Idem&&#[[1,2]]<=NS2Idem,False]&@ToExpression[v]];
(* cancellation function *)
CancellationSparse::usage="Cancellation[StructureMaps,generators] computes the complex obtained from StructureMaps by doing certain cancellations. 'genset' is the set of generators underlying 'StructureMaps'. 
The way cancellations are carried out can be affected by an optional third parameter, which takes one of the following values:
1) 0; this is the default and means that in each iteration, only a cancellation from second generator to the first will be carried out (if possible). 
2) 1, or in fact any other integer; In this case, any identity morphism will be cancelled (if possible).
3) list of generator pairs {endgen,startgen}. Those cancellations will be performed if possible. Cancellation will stop at the first instance the conditions for cancellation are not satisfied. The indices here are those corresponding to 'genset'";
CancellationSparse0[StructureMaps_,genset_,genstocancelInput_]:=Module[{
MMit=StructureMaps,(* MMit[[y,x]], MMit[[All,i]]= column i *)
generatorsit=genset,
genstocancel=genstocancelInput,
diff2=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"{", 
RowBox[{"{", 
RowBox[{"1", ",", "2"}], "}"}], "}"}], 
RowBox[{"genstocancelInput", "==", "0"}]},
{
RowBox[{
RowBox[{"Cases", "[", 
RowBox[{
RowBox[{"ArrayRules", "[", "StructureMaps", "]"}], ",", 
RowBox[{"v_", "/;", 
RowBox[{"IdentityQString", "[", 
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], "]"}], "[", 
RowBox[{"[", 
RowBox[{"All", ",", "1"}], "]"}], "]"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False], 
RowBox[{"IntegerQ", "[", "genstocancelInput", "]"}]},
{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"genstocancelInput", "===", 
RowBox[{"{", "}"}]}], ",", 
RowBox[{"{", "}"}], ",", 
RowBox[{"{", 
RowBox[{"genstocancelInput", "[", 
RowBox[{"[", "1", "]"}], "]"}], "}"}]}], "]"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\),(* diff2: positions of honest differentials in MMtit*)
it=0,(* iterator counting the number of cancellations *)
coorit=1,(* iterator testing all positions *)
coorcond=0,(* flag. 1: generator pair at current coor can be cancelled; 0: we need to look further, so coorit=coorit+1 *)
coorexit=0,(* flag. 0: there are still untested generator pairs; 1: none of the generator pairs can be cancelled *)
cooritmax=0,(* auxiliary variable for flag coorexit *)
vb={},
vc={},
coor={1,2},
RemainingGenRange=Range[3,Length[StructureMaps]],
Output={{},{},{}},
LOGcancelledgenerators={}(* list of all cancelled generatorpairs {{enditerindex,startiterindex},{endgensetindex,startgensetindex}}; iterindex means the index of the generator in the iteration step in which it is cancelled. gensetindex menas its index in 'genset'. *)
},
If[Length[StructureMaps]!=Length[genset],Print["USER ERROR: Generator set cannot belong to the structure maps!"],
If[If[IntegerQ[genstocancelInput],False,(DuplicateFreeQ[Flatten[genstocancelInput]]&&SubsetQ[Range[1,Length[genset]],Flatten[genstocancelInput]])==False],Print["USER ERROR: Check you generator pairs you want cancelled!"],
cooritmax=Length[diff2];
(* test for generator pairs that can be cancelled *)
While[coorit<=cooritmax&&coorcond==0,
coor=diff2[[coorit]];
RemainingGenRange=Complement[Range[1,Length[MMit]],coor];
vc=MMit[[coor[[1]],RemainingGenRange]]; (* image of x *)
vb=MMit[[RemainingGenRange,coor[[2]]]];  (* "preimage" of y *)
If[IdentityQString[MMit[[coor[[1]],coor[[2]]]]]&&MMit[[coor[[1]],coor[[1]]]]==MMit[[coor[[2]],coor[[2]]]],coorcond=1,coorcond=0];
coorit=coorit+1
];
If[coorcond==0,coorexit=1;Print["No Cancellation possible."],
(* feedback *)
Print[TimeObject[Now]];
(* cancellation iteration *)
Monitor[
While[If[genstocancelInput==0,
diff2=!={}&&coorexit==0&&Length[generatorsit]>Length[SurvivingGeneratorsAfterFirstCancellation],
diff2=!={}&&coorexit==0],
(* In case of a prescribed list of cancellation pairs, removing the next pair to be cancelled and shifting the indices *)
If[IntegerQ[genstocancel]==False,genstocancel=Map[GenIndexFromNewIndex[generatorsit,#,generatorsit[[RemainingGenRange]]]&,Drop[genstocancel,1],{2}]];
(* record which generators are cancelled in this iteration *)
LOGcancelledgenerators=Append[LOGcancelledgenerators,{coor,{GenIndexFromNewIndex[generatorsit,coor[[1]],genset],GenIndexFromNewIndex[generatorsit,coor[[2]],genset]}}];
(* Adding contribution from cancelled pair to remaining morphisms *)

MMit=Map[AlgAASparse,MMit[[RemainingGenRange,RemainingGenRange]]+Transpose[Outer[ArrowCompString,vc,vb,1,1]],{2}];
(* cancelling generators *)
generatorsit=generatorsit[[RemainingGenRange]];
(* find new generator pairs *)
diff2=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"{", 
RowBox[{"{", 
RowBox[{"1", ",", "2"}], "}"}], "}"}], 
RowBox[{"genstocancelInput", "==", "0"}]},
{
RowBox[{
RowBox[{"Cases", "[", 
RowBox[{
RowBox[{"ArrayRules", "[", "MMit", "]"}], ",", 
RowBox[{"v_", "/;", 
RowBox[{"IdentityQString", "[", 
RowBox[{"v", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], "]"}], "[", 
RowBox[{"[", 
RowBox[{"All", ",", "1"}], "]"}], "]"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False], 
RowBox[{"IntegerQ", "[", "genstocancelInput", "]"}]},
{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"genstocancel", "==", 
RowBox[{"{", "}"}]}], ",", 
RowBox[{"{", "}"}], ",", 
RowBox[{"{", 
RowBox[{"genstocancel", "[", 
RowBox[{"[", "1", "]"}], "]"}], "}"}]}], "]"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);(* Reset the generator pair iterator *)
coorit=1;
(* Reset flag *)
coorcond=0;
(* Reset auxiliary variable for flag coorexit *)
cooritmax=Length[diff2];
(* test for generator pairs that can be cancelled *)
While[coorit<=cooritmax&&coorcond==0,
coor=diff2[[coorit]];
RemainingGenRange=Complement[Range[1,Length[MMit]],coor];
vc=MMit[[coor[[1]],RemainingGenRange]]; (* image of x *)
vb=MMit[[RemainingGenRange,coor[[2]]]];  (* "preimage" of y *)
If[IdentityQString[MMit[[coor[[1]],coor[[2]]]]]&&MMit[[coor[[1]],coor[[1]]]]==MMit[[coor[[2]],coor[[2]]]],coorcond=1,coorcond=0];
coorit=coorit+1;
];
If[coorcond==0,coorexit=1];
it=it+1;]
,"Iteration: "<>ToString[it]<>"\nNumber of remaining generators: "<>ToString[Length[generatorsit]]
(*
{"...Calculating...","Iteration",it,ArrayPlot[Map[Length,MMit[[1;;Min[200,Length[MMit]],1;;Min[200,Length[MMit]]]],{2}]]}//MatrixForm
*)
]
];
Output=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"{", 
RowBox[{
RowBox[{"ToNonSparse", "[", "MMit", "]"}], ",", "generatorsit"}], "}"}], 
RowBox[{"genstocancel", "===", "0"}]},
{
RowBox[{"{", 
RowBox[{
RowBox[{"ToNonSparse", "[", "MMit", "]"}], ",", "generatorsit", ",", "LOGcancelledgenerators"}], "}"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(*More Feedback*)
Print["Finished calculation after "<>ToString[it]<>" iteration"<>If[it==1,"","s"]<>"."];
(* Print remaining number of generators  *)
Print["There are ",Length[generatorsit]," generators left."];
(* If genstocancelInput=!=0, print a list of all cancelled generator pairs *)
If[genstocancelInput=!=0,Print["The cancelled generators were ",Transpose[{"("<>ToString[#[[1,2]]]<>")",ToString[#[[2,2]]],"\[DownArrow]",ToString[#[[2,1]]],"("<>ToString[#[[1,1]]]<>")"}&/@LOGcancelledgenerators]//MatrixForm];]
]
];
Print[TimeObject[Now]];
Output
];
ToNonSparse[mat_]:=Map[If[Head[#]===Integer,{},ToExpression[#]]&,Normal[mat],{2}];
CancellationSparse[StructureMaps_,genset_,genstocancelInput_:0]:=CancellationSparse0[StructureMaps,genset,genstocancelInput];
(* Find "identity" labels *)
IdentityQ[v_]:=If[Length[v]==1,v[[1,1]]<=NS1Idem&&v[[1,2]]<=NS2Idem,False];
(* cancellation function *)
Cancellation::usage="Cancellation[StructureMaps,generators,genset] computes the complex obtained from StructureMaps by doing certain cancellations. 'generators' is the set of generators underlying 'StructureMaps'. 
The way cancellations are carried out can be affected by an optional third parameter, which takes one of the following values:
1) 1, or in fact any other integer; In this case, any identity morphism will be cancelled (if possible).
2) list of generator pairs {endgen,startgen}. Those cancellations will be performed if possible. Cancellation will stop at the first instance the conditions for cancellation are not satisfied. The indices here are those corresponding to 'generators'";
Cancellation0[StructureMaps_,genset_,genstocancelInput_]:=Module[{
MMit=StructureMaps,(* MMit[[y,x]], MMit[[All,i]]= column i *)
generatorsit=genset,
genstocancel=genstocancelInput,
diff2=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"Position", "[", 
RowBox[{"StructureMaps", ",", 
RowBox[{"_", "?", "IdentityQ"}], ",", 
RowBox[{"{", "2", "}"}]}], "]"}], 
RowBox[{"IntegerQ", "[", "genstocancelInput", "]"}]},
{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"genstocancelInput", "===", 
RowBox[{"{", "}"}]}], ",", 
RowBox[{"{", "}"}], ",", 
RowBox[{"{", 
RowBox[{"genstocancelInput", "[", 
RowBox[{"[", "1", "]"}], "]"}], "}"}]}], "]"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\),(* diff2: positions of honest differentials in MMtit*)
it=0,(* iterator counting the number of cancellations *)
coorit=1,(* iterator testing all positions *)
coorcond=0,(* flag. 1: generator pair at current coor can be cancelled; 0: we need to look further, so coorit=coorit+1 *)
coorexit=0,(* flag. 0: there are still untested generator pairs; 1: none of the generator pairs can be cancelled *)
cooritmax=0,(* auxiliary variable for flag coorexit *)
vb={},
vc={},
coor={1,2},
RemainingGenRange=Range[3,Length[StructureMaps]],
Output={{},{},{}},
LOGcancelledgenerators={}(* list of all cancelled generatorpairs {{enditerindex,startiterindex},{endgensetindex,startgensetindex}}; iterindex means the index of the generator in the iteration step in which it is cancelled. gensetindex menas its index in 'genset'. *)
},
If[Length[StructureMaps]!=Length[genset],Print["USER ERROR: Generator set cannot belong to the structure maps!"],
If[If[IntegerQ[genstocancelInput],False,(DuplicateFreeQ[Flatten[genstocancelInput]]&&SubsetQ[Range[1,Length[genset]],Flatten[genstocancelInput]])==False],Print["USER ERROR: Check you generator pairs you want cancelled!"],
cooritmax=Length[diff2];
(* test for generator pairs that can be cancelled *)
While[coorit<=cooritmax&&coorcond==0,
coor=diff2[[coorit]];
RemainingGenRange=Complement[Range[1,Length[MMit]],coor];
vc=MMit[[coor[[1]],RemainingGenRange]]; (* image of x *)
vb=MMit[[RemainingGenRange,coor[[2]]]];  (* "preimage" of y *)
If[IdentityQ[MMit[[coor[[1]],coor[[2]]]]]&&MMit[[coor[[1]],coor[[1]]]]==MMit[[coor[[2]],coor[[2]]]],coorcond=1,coorcond=0];
coorit=coorit+1;
];
If[coorcond==0,coorexit=1;Print["No Cancellation possible."],
(* feedback *)
Print[TimeObject[Now]];
(* cancellation iteration *)
Monitor[
While[diff2=!={}&&coorexit==0,
(* In case of a prescribed list of cancellation pairs, removing the next pair to be cancelled and shifting the indices *)
If[IntegerQ[genstocancel]==False,genstocancel=Map[GenIndexFromNewIndex[generatorsit,#,generatorsit[[RemainingGenRange]]]&,Drop[genstocancel,1],{2}]];
(* record which generators are cancelled in this iteration *)
LOGcancelledgenerators=Append[LOGcancelledgenerators,{coor,{GenIndexFromNewIndex[generatorsit,coor[[1]],genset],GenIndexFromNewIndex[generatorsit,coor[[2]],genset]}}];
(* Adding contribution from cancelled pair to remaining morphisms *)
MMit=MapThread[AlgAA,{MMit[[RemainingGenRange,RemainingGenRange]],Transpose[Outer[ArrowComp,vc,vb,1,1]]},2];
(* cancelling generators *)
generatorsit=generatorsit[[RemainingGenRange]];
(* find new generator pairs *)
diff2=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"Position", "[", 
RowBox[{"MMit", ",", 
RowBox[{"_", "?", "IdentityQ"}], ",", 
RowBox[{"{", "2", "}"}]}], "]"}], 
RowBox[{"IntegerQ", "[", "genstocancelInput", "]"}]},
{
RowBox[{"If", "[", 
RowBox[{
RowBox[{"genstocancel", "==", 
RowBox[{"{", "}"}]}], ",", 
RowBox[{"{", "}"}], ",", 
RowBox[{"{", 
RowBox[{"genstocancel", "[", 
RowBox[{"[", "1", "]"}], "]"}], "}"}]}], "]"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Reset the generator pair iterator *)
coorit=1;
(* Reset flag *)
coorcond=0;
(* Reset auxiliary variable for flag coorexit *)
cooritmax=Length[diff2];
(* test for generator pairs that can be cancelled *)
While[coorit<=cooritmax&&coorcond==0,
coor=diff2[[coorit]];
RemainingGenRange=Complement[Range[1,Length[MMit]],coor];
vc=MMit[[coor[[1]],RemainingGenRange]]; (* image of x *)
vb=MMit[[RemainingGenRange,coor[[2]]]];  (* "preimage" of y *)
If[IdentityQ[MMit[[coor[[1]],coor[[2]]]]]&&MMit[[coor[[1]],coor[[1]]]]==MMit[[coor[[2]],coor[[2]]]],coorcond=1,coorcond=0];
coorit=coorit+1;
];
If[coorcond==0,coorexit=1];
it=it+1;]
,
{"...Calculating...","Iteration",it,ArrayPlot[Map[Length,MMit[[1;;Min[200,Length[MMit]],1;;Min[200,Length[MMit]]]],{2}],ColorRules->{0->Green}]}//MatrixForm
];
];
Output=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"{", 
RowBox[{"MMit", ",", "generatorsit"}], "}"}], 
RowBox[{"genstocancel", "===", "0"}]},
{
RowBox[{"{", 
RowBox[{"MMit", ",", "generatorsit", ",", "LOGcancelledgenerators"}], "}"}], "True"}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(*More Feedback*)
Print[{"Finished calculation","after "<>ToString[it]<>" iteration"<>If[it==1,"","s"]<>".",ArrayPlot[Map[Length,MMit[[1;;Min[200,Length[MMit]],1;;Min[200,Length[MMit]]]],{2}],ColorRules->{0->Green}]}//MatrixForm];
(* Print remaining number of generators  *)
Print["There are ",Length[generatorsit]," generators left."];
(* If genstocancelInput=!=0, print a list of all cancelled generator pairs *)
If[genstocancelInput=!=0,Print["The cancelled generators were ",Transpose[{"("<>ToString[#[[1,2]]]<>")",ToString[#[[2,2]]],"\[DownArrow]",ToString[#[[2,1]]],"("<>ToString[#[[1,1]]]<>")"}&/@LOGcancelledgenerators]//MatrixForm];];
];
];
Output
];
Cancellation[StructureMaps_,genset_,genstocancelInput_:1]:=Cancellation0[StructureMaps,genset,genstocancelInput];
(*Two functions that help decide if a cancellation was good*)(*Find those generators which are connected by two arrows pointing in opposite directions*)FindDoubleArrows[mat_]:=Complement[Intersection[Reverse/@#,#]&@Position[mat,v_/;v!={},{2}],{#,#}&/@Range[Length[mat]]];
NoDoubleArrowsQ[mat_]:=Length[FindDoubleArrows[mat]]==0;
(*What does the diagonal look like?(Making sure there is only the identity action.)*)
SimpleDiagonal[mat_]:=Tally[Diagonal[mat],Length[#1]==Length[#2]&];
SimpleDiagonalQ[mat_]:=Length[SimpleDiagonal[mat]]==1;
CarefulCancellation[StructureMaps_,genset_]:=Module[{
MMit=StructureMaps,(* MMit[[y,x]], MMit[[All,i]]= column i *)
MMitnew,
generatorsit=genset,
diff2=Position[StructureMaps,_?IdentityQ,{2}],(* diff2: positions of honest differentials in MMtit*)
it=0,(* iterator counting the number of cancellations *)
coorit=1,(* iterator testing all positions *)
coorcond=0,(* flag. 1: generator pair at current coor can be cancelled; 0: we need to look further, so coorit=coorit+1 *)
coorexit=0,(* flag. 0: there are still untested generator pairs; 1: none of the generator pairs can be cancelled *)
cooritmax=0,(* auxiliary variable for flag coorexit *)
vb={},
vc={},
coor={1,2},
RemainingGenRange=Range[3,Length[StructureMaps]],
Output={{},{},{}},
LOGcancelledgenerators={}(* list of all cancelled generatorpairs {{enditerindex,startiterindex},{endgensetindex,startgensetindex}}; iterindex means the index of the generator in the iteration step in which it is cancelled. gensetindex menas its index in 'genset'. *)
},
If[
Length[StructureMaps]!=Length[genset],Print["USER ERROR: Generator set cannot belong to the structure maps!"],
If[
NoDoubleArrowsQ[StructureMaps]==False,
Print["StructureMaps already contains double arrows: ",FindDoubleArrows[StructureMaps]],
If[
SimpleDiagonalQ[StructureMaps]==False,
Print["StructureMaps already has a more complicated diagonal: ",SimpleDiagonal[StructureMaps]],
cooritmax=Length[diff2];
(* test for generator pairs that can be cancelled *)
While[coorit<=cooritmax&&coorcond==0,
coor=diff2[[coorit]];
RemainingGenRange=Complement[Range[1,Length[MMit]],coor];
vc=MMit[[coor[[1]],RemainingGenRange]]; (* image of x *)
vb=MMit[[RemainingGenRange,coor[[2]]]];  (* "preimage" of y *)
MMitnew=MapThread[AlgAA,{MMit[[RemainingGenRange,RemainingGenRange]],Transpose[Outer[ArrowComp,vc,vb,1,1]]},2];
If[IdentityQ[MMit[[coor[[1]],coor[[2]]]]]&&MMit[[coor[[1]],coor[[1]]]]==MMit[[coor[[2]],coor[[2]]]]&&NoDoubleArrowsQ[MMitnew]&&SimpleDiagonalQ[MMitnew],
coorcond=1,coorcond=0];
coorit=coorit+1;
];
If[coorcond==0,coorexit=1;Print["No Cancellation possible."],
(* feedback *)
Print[TimeObject[Now]];
(* cancellation iteration *)
Monitor[
While[diff2=!={}&&coorexit==0,
(* record which generators are cancelled in this iteration *)
LOGcancelledgenerators=Append[LOGcancelledgenerators,{coor,{GenIndexFromNewIndex[generatorsit,coor[[1]],genset],GenIndexFromNewIndex[generatorsit,coor[[2]],genset]}}];
(* Adding contribution from cancelled pair to remaining morphisms *)
MMit=MMitnew;
(* cancelling generators *)
generatorsit=generatorsit[[RemainingGenRange]];
(* find new generator pairs *)
diff2=Position[MMit,_?IdentityQ,{2}];
(* Reset the generator pair iterator *)
coorit=1;
(* Reset flag *)
coorcond=0;
(* Reset auxiliary variable for flag coorexit *)
cooritmax=Length[diff2];
(* test for generator pairs that can be cancelled *)
While[coorit<=cooritmax&&coorcond==0,
coor=diff2[[coorit]];
RemainingGenRange=Complement[Range[1,Length[MMit]],coor];
vc=MMit[[coor[[1]],RemainingGenRange]]; (* image of x *)
vb=MMit[[RemainingGenRange,coor[[2]]]];  (* "preimage" of y *)
MMitnew=MapThread[AlgAA,{MMit[[RemainingGenRange,RemainingGenRange]],Transpose[Outer[ArrowComp,vc,vb,1,1]]},2];
If[IdentityQ[MMit[[coor[[1]],coor[[2]]]]]&&MMit[[coor[[1]],coor[[1]]]]==MMit[[coor[[2]],coor[[2]]]]&&NoDoubleArrowsQ[MMitnew]&&SimpleDiagonalQ[MMitnew],
coorcond=1,coorcond=0];
coorit=coorit+1;
];
If[coorcond==0,coorexit=1];
it=it+1;]
,
{"...Calculating...","Iteration",it,ArrayPlot[Map[Length,MMit[[1;;Min[200,Length[MMit]],1;;Min[200,Length[MMit]]]],{2}],ColorRules->{0->Green}]}//MatrixForm
];
];
Output={MMit,generatorsit,LOGcancelledgenerators};
(*More Feedback*)
Print[{"Finished calculation","after "<>ToString[it]<>" iteration"<>If[it==1,"","s"]<>".",ArrayPlot[Map[Length,MMit[[1;;Min[200,Length[MMit]],1;;Min[200,Length[MMit]]]],{2}],ColorRules->{0->Green}]}//MatrixForm];
(* Print remaining number of generators  *)
Print["There are ",Length[generatorsit]," generators left."];
(* If genstocancelInput=!=0, print a list of all cancelled generator pairs *)
Print["The cancelled generators were ",Transpose[{"("<>ToString[#[[1,2]]]<>")",ToString[#[[2,2]]],"\[DownArrow]",ToString[#[[2,1]]],"("<>ToString[#[[1,1]]]<>")"}&/@LOGcancelledgenerators]//MatrixForm];
];
];
];
Output
];
(* cancellation only within a subset of generators *)
CarefulCancellationSubset[StructureMaps_,genset_,gensubset_]:=Module[{
MMit=StructureMaps,(* MMit[[y,x]], MMit[[All,i]]= column i *)
MMitnew,
generatorsit=genset,
diff2=Cases[Position[StructureMaps,_?IdentityQ,{2}],v_/;MemberQ[gensubset,v[[1]]]&&MemberQ[gensubset,v[[2]]]],(* diff2: positions of honest differentials in MMtit*)
it=0,(* iterator counting the number of cancellations *)
coorit=1,(* iterator testing all positions *)
coorcond=0,(* flag. 1: generator pair at current coor can be cancelled; 0: we need to look further, so coorit=coorit+1 *)
coorexit=0,(* flag. 0: there are still untested generator pairs; 1: none of the generator pairs can be cancelled *)
cooritmax=0,(* auxiliary variable for flag coorexit *)
vb={},
vc={},
coor={1,2},
RemainingGenRange=Range[3,Length[StructureMaps]],
Output={{},{},{}},
LOGcancelledgenerators={}(* list of all cancelled generatorpairs {{enditerindex,startiterindex},{endgensetindex,startgensetindex}}; iterindex means the index of the generator in the iteration step in which it is cancelled. gensetindex menas its index in 'genset'. *)
},
If[
Length[StructureMaps]!=Length[genset],Print["USER ERROR: Generator set cannot belong to the structure maps!"],
If[
NoDoubleArrowsQ[StructureMaps]==False,
Print["StructureMaps already contains double arrows: ",FindDoubleArrows[StructureMaps]],
If[
SimpleDiagonalQ[StructureMaps]==False,
Print["StructureMaps already has a more complicated diagonal: ",SimpleDiagonal[StructureMaps]],
cooritmax=Length[diff2];
(* test for generator pairs that can be cancelled *)
While[coorit<=cooritmax&&coorcond==0,
coor=diff2[[coorit]];
RemainingGenRange=Complement[Range[1,Length[MMit]],coor];
vc=MMit[[coor[[1]],RemainingGenRange]]; (* image of x *)
vb=MMit[[RemainingGenRange,coor[[2]]]];  (* "preimage" of y *)
MMitnew=MapThread[AlgAA,{MMit[[RemainingGenRange,RemainingGenRange]],Transpose[Outer[ArrowComp,vc,vb,1,1]]},2];
If[IdentityQ[MMit[[coor[[1]],coor[[2]]]]]&&MMit[[coor[[1]],coor[[1]]]]==MMit[[coor[[2]],coor[[2]]]]&&NoDoubleArrowsQ[MMitnew]&&SimpleDiagonalQ[MMitnew],
coorcond=1,coorcond=0];
coorit=coorit+1;
];
If[coorcond==0,coorexit=1;Print["No Cancellation possible."],
(* feedback *)
Print[TimeObject[Now]];
(* cancellation iteration *)
Monitor[
While[diff2=!={}&&coorexit==0,
(* record which generators are cancelled in this iteration *)
LOGcancelledgenerators=Append[LOGcancelledgenerators,{coor,{GenIndexFromNewIndex[generatorsit,coor[[1]],genset],GenIndexFromNewIndex[generatorsit,coor[[2]],genset]}}];
(* Adding contribution from cancelled pair to remaining morphisms *)
MMit=MMitnew;
(* cancelling generators *)
generatorsit=generatorsit[[RemainingGenRange]];
(* find new generator pairs *)
diff2=Cases[Position[MMit,_?IdentityQ,{2}],v_/;MemberQ[gensubset,v[[1]]]&&MemberQ[gensubset,v[[2]]]];
(* Reset the generator pair iterator *)
coorit=1;
(* Reset flag *)
coorcond=0;
(* Reset auxiliary variable for flag coorexit *)
cooritmax=Length[diff2];
(* test for generator pairs that can be cancelled *)
While[coorit<=cooritmax&&coorcond==0,
coor=diff2[[coorit]];
RemainingGenRange=Complement[Range[1,Length[MMit]],coor];
vc=MMit[[coor[[1]],RemainingGenRange]]; (* image of x *)
vb=MMit[[RemainingGenRange,coor[[2]]]];  (* "preimage" of y *)
MMitnew=MapThread[AlgAA,{MMit[[RemainingGenRange,RemainingGenRange]],Transpose[Outer[ArrowComp,vc,vb,1,1]]},2];
If[IdentityQ[MMit[[coor[[1]],coor[[2]]]]]&&MMit[[coor[[1]],coor[[1]]]]==MMit[[coor[[2]],coor[[2]]]]&&NoDoubleArrowsQ[MMitnew]&&SimpleDiagonalQ[MMitnew],
coorcond=1,coorcond=0];
coorit=coorit+1;
];
If[coorcond==0,coorexit=1];
it=it+1;]
,
{"...Calculating...","Iteration",it,ArrayPlot[Map[Length,MMit[[1;;Min[200,Length[MMit]],1;;Min[200,Length[MMit]]]],{2}],ColorRules->{0->Green}]}//MatrixForm
];
];
Output={MMit,generatorsit,LOGcancelledgenerators};
(*More Feedback*)
Print[{"Finished calculation","after "<>ToString[it]<>" iteration"<>If[it==1,"","s"]<>".",ArrayPlot[Map[Length,MMit[[1;;Min[200,Length[MMit]],1;;Min[200,Length[MMit]]]],{2}],ColorRules->{0->Green}]}//MatrixForm];
(* Print remaining number of generators  *)
Print["There are ",Length[generatorsit]," generators left."];
(* If genstocancelInput=!=0, print a list of all cancelled generator pairs *)
Print["The cancelled generators were ",Transpose[{"("<>ToString[#[[1,2]]]<>")",ToString[#[[2,2]]],"\[DownArrow]",ToString[#[[2,1]]],"("<>ToString[#[[1,1]]]<>")"}&/@LOGcancelledgenerators]//MatrixForm];
];
];
];
Output
];
];


Step5PostProcessing:=Module[{},
OutputPreparation;
HomotopyPreparation;
];


OutputPreparation:=Module[{},
(* First graphical representation of StructureMaps, via a Matrix *)
GResult[mat_,gens_]:=
If[Length[mat]!=Length[gens],Print["USER ERROR: Generator set cannot belong to the structure maps!"],Grid[Join[Transpose[{Join[{" "},{" "},{" "},Range[1,Length[mat]]]}],Transpose[{Join[{"  "},{"Idem"},{" "},S1IdemNameForGenQ[gens[[#]]]<>S2IdemNameForGenQ[gens[[#]]]&/@Range[1,Length[mat]]],Join[{" "},{" "},{" "},ConstantArray["|",Length[mat]]]}],Join[{Range[1,Length[mat]],S1IdemNameForGenQ[gens[[#]]]<>S2IdemNameForGenQ[gens[[#]]]&/@Range[1,Length[mat]],ConstantArray["-----",Length[mat]]},ReplacePart[ReplacePart[mat,Thread[Position[mat,v_/;v!={},{2}]->X]],Thread[Position[mat,v_/;v=={{1,1}},{2}]->Style[1,FontColor->Red,FontWeight->Bold]]]/.{}->Style[".",FontColor->Gray]],2],Background->{None,None,Join[Thread[#+{3,3}&/@Cases[Tuples[{Range[1,Length[mat]],Range[1,Length[mat]]}],v_/;(Part[mat,##]&@@v!={})&&(Part[mat,##]&@@Reverse[v]!={})]->Green],Thread[Table[{i+3,i+3},{i,1,Length[mat]}]->Yellow]]}]];
(* Second *)
ShowGraph0[StructureMaps_,gens_,pos_,CoordList_]:=Module[{npos=pos},
If[pos===0,npos=Range[Length[gens]]];AdjacencyGraph[Map[If[#=!={},1,0]&,Transpose[StructureMaps],{2}],DirectedEdges->True,{VertexCoordinates->CoordList,GraphHighlight->(#[[2]]->#[[1]]&)/@Position[StructureMaps,_?IdentityQ,{2}], EdgeLabels->Apply[((#2->#1)->AlgebraToLabels[StructureMaps[[#1,#2]]])&,Position[StructureMaps,v_/;v!={},{2}],{1}], VertexLabels->MapIndexed[(#2[[1]]->Placed[ToString[#1]<>":"<>"("<>S1IdemNameForGenQ[gens[[#1]]]<>S2IdemNameForGenQ[gens[[#1]]]<>")",Above])&,npos]}]];
(* include the follwing code in VertexCoordinates to include generator names 
If[Head[Aa]===Symbol,ToStringMS[gens[[#1]]],StringJoin[gens[[#1]]/.Aa]]
*)
ShowGraph[StructureMaps_,gens_,pos_:0,CoordList_:Automatic]:=
If[Length[StructureMaps]!=Length[gens],Print["USER ERROR: Generator set cannot belong to the structure maps!"],ShowGraph0[MapThread[AlgAA,{StructureMaps,AddS1S2Action[gens]},2],gens,pos,CoordList]];
ShowGraph::usage="ShowGraph[StructureMaps_,gens_,pos_:0,CoordList_:Automatic] draws a graphical representation of StructureMaps. 'gens' is the underlying set of generators corresponding to the StructureMaps. The identity action is implicit, i.e. should always be added.";
ShowSubGraph[mat_,gens_,pos_,CoordList_:Automatic]:=ShowGraph0[MapThread[AlgAA,{mat[[pos,pos]],AddS1S2Action[gens[[pos]]]},2],gens,pos,CoordList];
ShowSubGraph::usage="ShowSubGraph[mat_,gens_,pos_] displays the full subgraph of the complex specified by the generators with indices in the list pos.";
(* Third *)
PlotGraph[mat_]:=Print[ArrayPlot[Map[Length,mat,{2}],ColorRules->{0->Green}]];
PlotGraph::usage="PlotGraph[mat_] returs a graphical representation of the structure map 'mat' using ArrayPlot. Green means no labels, light grey few labels, black maximal number of labels in 'mat'.";
FindAllArrowsStartingAt[mat_,gens_,xindex_]:=If[Length[Flatten[mat[[All,xindex]]]]==0,Print["No arrows starting at generator "<>ToStringMS[xindex]<>"="<>ToStringMS[gens[[xindex]]]<>"."],Print@@Join[{"From "<>ToStringMS[xindex]<>"("<>S1IdemNameForGenQ[gens[[xindex]]]<>S2IdemNameForGenQ[gens[[xindex]]]<>")="<>StringJoin[If[Head[Aa]===Symbol,ToStringMS[gens[[xindex]]],gens[[xindex]]/.Aa]]<>"      algebra labels"},Flatten[{"\n To  "<>StringJoin[If[Head[Aa]===Symbol,ToStringMS[gens[[#]]],gens[[#]]/.Aa]]<>"="<>ToStringMS[#]<>"("<>S1IdemNameForGenQ[gens[[xindex]]]<>S2IdemNameForGenQ[gens[[xindex]]]<>") via ",Riffle[AlgebraToLabels[mat[[#,xindex]]],"+"]}&/@Position[mat[[All,xindex]],v_/;Length[v]!=0,{1}][[All,1]],2]]];
FindAllArrowsStartingAt::usage="FindAllArrowsStartingAt[mat_,gens_,xindex_] displays all arrows and their labels starting at the generator at position 'xindex' of 'gens' in the structure maps specified by 'mat'.";
FindAllArrowsEndingAt[mat_,gens_,yindex_]:=If[Length[Flatten[mat[[yindex]]]]==0,Print["No arrows ending at generator "<>ToStringMS[yindex]<>"="<>ToStringMS[gens[[yindex]]]<>"."],Print@@Join[{"   To "<>ToStringMS[yindex]<>"("<>S1IdemNameForGenQ[gens[[yindex]]]<>S2IdemNameForGenQ[gens[[yindex]]]<>")="<>StringJoin[If[Head[Aa]===Symbol,ToStringMS[gens[[yindex]]],gens[[yindex]]/.Aa]]<>"      algebra labels"},Flatten[{"\n From  "<>StringJoin[If[Head[Aa]===Symbol,ToStringMS[gens[[#]]],gens[[#]]/.Aa]]<>"="<>ToStringMS[#]<>"("<>S1IdemNameForGenQ[gens[[yindex]]]<>S2IdemNameForGenQ[gens[[yindex]]]<>") via ",Riffle[AlgebraToLabels[mat[[yindex,#]]],"+"]}&/@Position[mat[[yindex]],v_/;Length[v]!=0,{1}][[All,1]],2]]];
FindAllArrowsEndingAt::usage="FindAllArrowsEndingAt[mat_,gens_,xindex_] displays all arrows and their labels ending at the generator at position 'yindex' of 'gens' in the structure maps specified by 'mat'.";
IdemsForPartitioningPre[gens_,GenIndexPartitioningInput_] :=Map[({S1IdemNameForGenQ[gens[[#]]],S2IdemNameForGenQ[gens[[#]]]})&,GenIndexPartitioningInput,{2}];
IdemsForPartitioning0[gens_,GenIndexPartitioningInput_] :=Module[{GenIndexPartitioning=If[IntegerQ[GenIndexPartitioningInput],{Range[Length[gens]]},GenIndexPartitioningInput]},Function[{c},Join[{DeleteDuplicates[Flatten[c,1]]},Table[Count[#,DeleteDuplicates[Flatten[c,1]][[i]]],{i,Length[DeleteDuplicates[Flatten[c,1]]]}]&/@c]]@IdemsForPartitioningPre[gens,GenIndexPartitioningInput]//MatrixForm];
IdemsForPartitioning[gens_,GenIndexPartitioningInput_:0]:=IdemsForPartitioning0[gens,GenIndexPartitioningInput];
IdemsForPartitioning::usage="IdemsForPartitioning[gens_,GenIndexPartitioningInput_] takes a list of generators gens and a partitioning of these GenIndexPartitioningInput and displays the number of idempotents for each partition. GenIndexPartitioningInput is optional; The default is the trivial partitioning of gens with one partition.";
PossibleCancelsWithIdems[mat_,gens_]:=MatrixForm[{{S1IdemNameForGenQ[gens[[#[[1,1]]]]],S2IdemNameForGenQ[gens[[#[[1,1]]]]]},#}&/@GatherBy[Cases[ArrayRules[mat],v_/;IdentityQString[v[[2]]]][[All,1]],{S1IdemNameForGenQ[gens[[#[[1]]]]],S2IdemNameForGenQ[gens[[#[[1]]]]]}&]];
PossibleCancelsWithIdems::usage="PossibleCancelsWithIdems[mat_,gens_] displays a list of possible cancellation pairs in a complex with generators gens and structure maps mat, ordered by idempotents.";
GenTableWithIdems[gens_]:=MatrixForm[{{S1IdemNameForGenQ[gens[[#[[1]]]]],S2IdemNameForGenQ[gens[[#[[1]]]]]},#}&/@GatherBy[Range[Length[gens]],{S1IdemNameForGenQ[gens[[#]]],S2IdemNameForGenQ[gens[[#]]]}&]];
GenTableWithIdems::usage="GenTableWithIdems[gens_] displays a list of generators gens, ordered by idempotents.";
ListOfAllLabels[mat_]:=DeleteDuplicates[Flatten[{AlgebraToLabels[{#}][[1]],"\[LeftRightArrow]",#},1]&/@Sort[Flatten[mat,2]]]//MatrixForm;
ListOfAllLabels::usage="ListOfAllLabels[mat_] returns a list of all labels appearing in mat.";
];


HomotopyPreparation:=Module[{},
xfactor=10;
xfactor::usage="Set 'xfactor' equal to a real number between 1 and \[Infinity]. This value determines the complexity of the structure maps; 1 means all morphisms count the same; if it is higher, long sequences of algebra elements contribute less towards the complexity of the structure maps.";
(* Find those elements generating the quotient algebra *)
S1NiceSubAlg=Cases[S1SubalgebraBasis,v_/;StringFreeQ[ToString[v/.S1RepRuleLabels],"p"]||StringFreeQ[ToString[v/.S1RepRuleLabels],"q"]];
S2NiceSubAlg=Cases[S2SubalgebraBasis,v_/;StringFreeQ[ToString[v/.S2RepRuleLabels],"p"]||StringFreeQ[ToString[v/.S2RepRuleLabels],"q"]];
S1NiceSubAlgTable:=Print["S1NiceSubAlgTable=",MatrixForm[{S1NiceSubAlg,S1NiceSubAlg/.S1RepRuleLabels}]];
S2NiceSubAlgTable:=Print["S2NiceSubAlgTable=",MatrixForm[{S2NiceSubAlg,S2NiceSubAlg/.S2RepRuleLabels}]];
(* PosToBeCleaned is a list of all positions in StructureMaps that we want to clean up, i.e. homotope to 0 *)
PosToBeCleaned[GenIndexPartitioning_]:=
Flatten[Tuples[{GenIndexPartitioning[[#[[1]]]],GenIndexPartitioning[[#[[2]]]]}]&/@Complement[Tuples[{Range[Length[GenIndexPartitioning]],Range[Length[GenIndexPartitioning]]}],{#,#}&/@Range[Length[GenIndexPartitioning]]],1];
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{"MyLengthS1", "[", "v_", "]"}], ":=", 
RowBox[{
RowBox[{"MyLengthS1", "[", "v", "]"}], "=", 
RowBox[{
RowBox[{"Length", "[", "v", "]"}], "+", 
RowBox[{"Count", "[", 
RowBox[{"v", ",", 
RowBox[{"x_", "/;", 
RowBox[{"MemberQ", "[", 
RowBox[{"S1NiceSubAlg", ",", "x"}], "]"}]}]}], "]"}]}]}]}], 
RowBox[{"TypeS1", "===", "TypeA"}]},
{
RowBox[{
RowBox[{"MyLengthS1", "[", "v_", "]"}], ":=", 
RowBox[{
RowBox[{"MyLengthS1", "[", "v", "]"}], "=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{"MemberQ", "[", 
RowBox[{"S1NiceSubAlg", ",", "v"}], "]"}], ",", "xfactor", ",", "1"}], "]"}]}]}], 
RowBox[{"TypeS1", "===", "TypeD"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{"MyLengthS2", "[", "v_", "]"}], ":=", 
RowBox[{
RowBox[{"MyLengthS2", "[", "v", "]"}], "=", 
RowBox[{
RowBox[{"Length", "[", "v", "]"}], "+", 
RowBox[{"Count", "[", 
RowBox[{"v", ",", 
RowBox[{"x_", "/;", 
RowBox[{"MemberQ", "[", 
RowBox[{"S1NiceSubAlg", ",", "x"}], "]"}]}]}], "]"}]}]}]}], 
RowBox[{"TypeS2", "===", "TypeA"}]},
{
RowBox[{
RowBox[{"MyLengthS2", "[", "v_", "]"}], ":=", 
RowBox[{
RowBox[{"MyLengthS2", "[", "v", "]"}], "=", 
RowBox[{"If", "[", 
RowBox[{
RowBox[{"MemberQ", "[", 
RowBox[{"S2NiceSubAlg", ",", "v"}], "]"}], ",", "xfactor", ",", "1"}], "]"}]}]}], 
RowBox[{"TypeS2", "===", "TypeD"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
ComplexityQuot0[StructureMaps_,PosToBeCleaned0_]:=Module[{
PosToBeCleaned=If[
IntegerQ[PosToBeCleaned0],
Tuples[{Range[Length[StructureMaps]],Range[Length[StructureMaps]]}],
PosToBeCleaned0
]
},
(*
Length[Flatten[Apply[StructureMaps[[##]]&,PosToBeCleaned,1],1]]
*)
Plus@@(1/xfactor^(MyLengthS1[#[[1]]]+MyLengthS2[#[[2]]])&/@Flatten[Apply[StructureMaps[[##]]&,PosToBeCleaned,1],1])
];
ComplexityQuot[StructureMaps_,PosToBeCleaned_:0]:=ComplexityQuot0[StructureMaps,PosToBeCleaned];
Complexity0[StructureMaps_,PosToBeCleaned0_]:=Module[{
PosToBeCleaned=If[
IntegerQ[PosToBeCleaned0],
Tuples[{Range[Length[StructureMaps]],Range[Length[StructureMaps]]}],
PosToBeCleaned0
]
},
(*
Length[Flatten[Apply[StructureMaps[[##]]&,PosToBeCleaned,1],1]]
*)
Plus@@(1/xfactor^Plus@@Map[Length,#]&/@Flatten[Apply[StructureMaps[[##]]&,PosToBeCleaned,1],1])
];
Complexity[StructureMaps_,PosToBeCleaned_:0]:=Complexity0[StructureMaps,PosToBeCleaned];
Complexity::usage="Complexity[StructureMaps_,GenIndexPartitioningInput_] counts the number of algebra elements in the labelling of the arrows of StructureMaps that go between different generator sets, specified by 'GenIndexPartitioningInput'. If 'GenIndexPartitioningInput' has length 1 or is an integer, we count algebra elements on all arrows.";
(* inverse of M: Given a sequences seq, find all sequences of basic algebra elements, whose seq-component of S1M is 1. *)
S1Mm1diff[seq_]:=Mod2[Flatten[Tuples[ReplacePart[Function[{c},{c}]/@seq,#->Position[S1DiffList,seq[[#]]][[All,1]]]]&/@Range[1,Length[seq]],1]];
S1Mm1mul[seq_]:=Mod2[Flatten/@Flatten[Tuples[ReplacePart[Function[{c},{c}]/@seq,#->Reverse/@Position[S1MultTable,seq[[#]]]]]&/@Range[1,Length[seq]],1]];
S1Mm1[seq_]:=Join[S1Mm1mul[seq],S1Mm1diff[seq]];
(* Same for S2 *)
S2Mm1diff[seq_]:=Mod2[Flatten[Tuples[ReplacePart[Function[{c},{c}]/@seq,#->Position[S2DiffList,seq[[#]]][[All,1]]]]&/@Range[1,Length[seq]],1]];
S2Mm1mul[seq_]:=Mod2[Flatten/@Flatten[Tuples[ReplacePart[Function[{c},{c}]/@seq,#->Reverse/@Position[S2MultTable,seq[[#]]]]]&/@Range[1,Length[seq]],1]];
S2Mm1[seq_]:=Join[S2Mm1mul[seq],S2Mm1diff[seq]];
(* Now combine for S1 and S2 *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{"Mm1", "[", "alg_", "]"}], ":=", 
RowBox[{"Join", "[", 
RowBox[{
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
RowBox[{"S1Mm1", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "&"}], "/@", 
RowBox[{"S2Mm1", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]},
{
RowBox[{
RowBox[{"Mm1", "[", "alg_", "]"}], ":=", 
RowBox[{"Join", "[", 
RowBox[{
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
RowBox[{"S1Mm1", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "&"}], "/@", 
RowBox[{"S2DiffList", "[", 
RowBox[{"[", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}], "]"}]}]}], "]"}]}], 
StyleBox[
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}],
FontWeight->"Bold"]},
{
RowBox[{
RowBox[{"Mm1", "[", "alg_", "]"}], ":=", 
RowBox[{"Join", "[", 
RowBox[{
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
RowBox[{"S1DiffList", "[", 
RowBox[{"[", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}]}], ",", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "&"}], "/@", 
RowBox[{"S2Mm1", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]},
{
RowBox[{
RowBox[{"Mm1", "[", "alg_", "]"}], ":=", 
RowBox[{"Join", "[", 
RowBox[{
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
RowBox[{"S1DiffList", "[", 
RowBox[{"[", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}]}], ",", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "&"}], "/@", 
RowBox[{"S2DiffList", "[", 
RowBox[{"[", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}], "]"}]}]}], "]"}]}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Definition of Matrix after applying a homotopy.
input is a label and endgenindex and startgenindex *)
(* The condition we check here is slightly stronger than necessary, but quicker. *)
(* start\[NotEqual]end ensures that h^2 vanishes. Also, hD(h) and D(h)h need to vanish. Together with start\[NotEqual]end, StructureMaps[[start,end]]==={} is both necessary and sufficient. *)
CheckIfHomotopyIsAllowed[StructureMaps_,homotopy_,start_,end_]:=If[start!=end&&StructureMaps[[start,end]]==={},{homotopy,start,end},0];
(* Add differential to MatrixInput *)
HomotopyAddDiffOnly[MatrixInput_,homotopy_,start_,end_]:=MapAt[Function[{c},AlgAA[c,Mm1[homotopy]]],MatrixInput,{end,start}];
(* Matrix of contribution of precompositions of structure map with homotopy *)
PreComposeMatrix[MatrixInput_,homotopy_,start_,end_]:=Map[Function[{c},If[IntegerQ[c],{},c]],ArrayPad[Transpose[{Function[{c},ArrowComp[{homotopy},c]]/@MatrixInput[[All,end]]}],{{0},{start-1,Length[MatrixInput]-start}}],{2}];
(* Matrix of contribution of postcompositions of structure map with homotopy *)
PostComposeMatrix[MatrixInput_,homotopy_,start_,end_]:=Map[Function[{c},If[IntegerQ[c],{},c]],ArrayPad[{Function[{c},ArrowComp[c,{homotopy}]]/@MatrixInput[[start,All]]},{{end-1,Length[MatrixInput]-end},{0}}],{2}];
(* Matrix of contribution of pre- and postcompositions of structure map with homotopy *)
PreAndPostComposeMatrix[MatrixInput_,homotopy_,start_,end_]:=MapThread[AlgAA,{PostComposeMatrix[MatrixInput,homotopy,start,end],PreComposeMatrix[MatrixInput,homotopy,start,end]},2];
(* Homotopy without security checks *)
Homotopy0[StructureMaps_,homotopy_,start_,end_]:=
MapThread[AlgAA,{HomotopyAddDiffOnly[StructureMaps,homotopy,start,end],PreAndPostComposeMatrix[StructureMaps,homotopy,start,end]},2];
(* Homotopy with security checks; 
0. Check if genset  has same length as StructureMaps
1. Check if conditions for homotopy are satisfied
2. we do not want to use homotopies with wrong idempotents *)
Homotopy[StructureMaps_,genset_,homotopy_,start_,end_]:=
If[Length[StructureMaps]!=Length[genset],Print["USER ERROR: Generator set cannot belong to the structure maps!"],
If[IntegerQ[CheckIfHomotopyIsAllowed[StructureMaps,homotopy,start,end]],
Print["USER ERROR: the conditions for adding a homotopy are not satisfied."],
If[
S1IdemForGenQ[Position[generators,genset[[start]]][[1,1]]]==S1IdemForAlgIndStart[homotopy[[1]]]&&
S1IdemForGenQ[Position[generators,genset[[end]]][[1,1]]]==S1IdemForAlgIndEnd[homotopy[[1]]]&&
S2IdemForGenQ[Position[generators,genset[[start]]][[1,1]]]==S2IdemForAlgIndStart[homotopy[[2]]]&&
S2IdemForGenQ[Position[generators,genset[[end]]][[1,1]]]==S2IdemForAlgIndEnd[homotopy[[2]]],
Homotopy0[StructureMaps,homotopy,start,end],Print["USER ERROR: You are trying to use a homotopy with the wrong idempotents. Try again."]]]];
Homotopy::usage="Homotopy[StructureMaps,genset,homotopy,start,end] computes the new StuctureMaps obtained by applying a homotopy (=algebra element) to StructureMaps going from generator with index 'start' to generator with index 'end' in the underlying set of generators specified by 'genset'.";
(* Adding a homotpy such that a given label is killed from pre- or postcomposition with homotopy *)
(* First we need to find all splittings of the label. *)
(* Split a sequence of basic algebra elements into two  *)
S1SplitSeq[seq_]:=If[IntegerQ[seq],{{seq,seq}},Join[{{Position[S1AlgebraTableInd,First[seq]][[1,2]],seq}},({seq[[1;;#]],seq[[#+1;;]]})&/@Range[1,Length[seq]-1],{{seq,Position[S1AlgebraTableInd,Last[seq]][[1,1]]}}]];
S2SplitSeq[seq_]:=If[IntegerQ[seq],{{seq,seq}},Join[{{Position[S2AlgebraTableInd,First[seq]][[1,2]],seq}},({seq[[1;;#]],seq[[#+1;;]]})&/@Range[1,Length[seq]-1],{{seq,Position[S2AlgebraTableInd,Last[seq]][[1,1]]}}]];
(* SplitAlg gives a list of splittings, i.e. an entry is a pair of algebra elements whose arrow composition (=concatenation for type A and AA structures) is alg *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{
RowBox[{"SplitAlg", "[", "alg_", "]"}], ":=", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{
RowBox[{"S1SplitSeq", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}], ",", 
RowBox[{"S2SplitSeq", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "}"}], "]"}]}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"SplitAlg", "[", "alg_", "]"}], ":=", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{
RowBox[{"S1SplitSeq", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}], ",", 
RowBox[{"Reverse", "/@", 
RowBox[{"Position", "[", 
RowBox[{"S2MultTable", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}]}], "}"}], "]"}]}]}], ";"}], 
StyleBox[
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}],
FontWeight->"Bold"]},
{
RowBox[{
RowBox[{
RowBox[{"SplitAlg", "[", "alg_", "]"}], ":=", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{
RowBox[{"Reverse", "/@", 
RowBox[{"Position", "[", 
RowBox[{"S1MultTable", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ",", 
RowBox[{"S2SplitSeq", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "}"}], "]"}]}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"SplitAlg", "[", "alg_", "]"}], ":=", 
RowBox[{"Transpose", "/@", 
RowBox[{"Tuples", "[", 
RowBox[{"{", 
RowBox[{
RowBox[{"Reverse", "/@", 
RowBox[{"Position", "[", 
RowBox[{"S1MultTable", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ",", 
RowBox[{"Reverse", "/@", 
RowBox[{"Position", "[", 
RowBox[{"S2MultTable", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}]}], "}"}], "]"}]}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Need to find homotopies such that a given label at position pos in MatrixInput is killed from precomposition with this homotopy. pos is a list of the form {endcoord,startcoor,position of algebra element in labelling of this arrow}.
Output is a list of pairs of the form {endgenindex of homotopy,index of splitting to be used for this homotopy} *)
AllHomotopiesForPreComp0[MatrixInput_,pos_]:=Flatten[Apply[Tuples[{{#1},{pos[[2]]},Position[MatrixInput[[pos[[1]]]],#2,{2}][[All,1]]}]&,SplitAlg[Part[MatrixInput,##]&@@pos],{1}],1];
AllHomotopiesForPreComp[MatrixInput_,pos_]:=CheckIfHomotopyIsAllowedList[MatrixInput,AllHomotopiesForPreComp0[MatrixInput,pos]];
(* same for postcompositions.
Output is a list of pairs of the form {algebra element, startgenindex of homotopy,endgenindex of homotopy} *)
CheckIfHomotopyIsAllowedList[MatrixInput_,list_]:=DeleteCases[Apply[CheckIfHomotopyIsAllowed[MatrixInput,##]&,list,1],0];
AllHomotopiesForPostComp0[MatrixInput_,pos_]:=Flatten[Apply[Tuples[{{#2},Position[MatrixInput[[All,pos[[2]]]],#1,{2}][[All,1]],{pos[[1]]}}]&,SplitAlg[Part[MatrixInput,##]&@@pos],{1}],1];
AllHomotopiesForPostComp[MatrixInput_,pos_]:=
CheckIfHomotopyIsAllowedList[MatrixInput,AllHomotopiesForPostComp0[MatrixInput,pos]];
(* For type A structures: function M *)
S1Mdiff[seq_]:=Mod2[Flatten[Tuples/@Function[{c},MapAt[S1DiffList[[#[[1]]]]&,{#}&/@seq,{c}]]/@Range[1,Length[seq]],1]];
S1Mmulpre=#@@@Partition[#2,2,1]&;
S1Mmul[seq_]:=Mod2[DeleteCases[ReplacePart[Drop[seq,{#}],#->S1Mmulpre[S1IndexComp,seq][[#]]]&/@ Range[1,Length[seq]-1],v_/;MemberQ[v,0]]];
S1M[seq_]:=If[IntegerQ[seq],{},Join[S1Mmul[seq],S1Mdiff[seq]]];
(* Same for S2 *)
S2Mdiff[seq_]:=Mod2[Flatten[Tuples/@Function[{c},MapAt[S2DiffList[[#[[1]]]]&,{#}&/@seq,{c}]]/@Range[1,Length[seq]],1]];
S2Mmulpre=#@@@Partition[#2,2,1]&;
S2Mmul[seq_]:=Mod2[DeleteCases[ReplacePart[Drop[seq,{#}],#->S2Mmulpre[S2IndexComp,seq][[#]]]&/@ Range[1,Length[seq]-1],v_/;MemberQ[v,0]]];
S2M[seq_]:=If[IntegerQ[seq],{},Join[S2Mmul[seq],S2Mdiff[seq]]];
(* Now combine for S1 and S2 *)
\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{
RowBox[{
RowBox[{"M", "[", "alg_", "]"}], ":=", 
RowBox[{"Join", "[", 
RowBox[{
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
RowBox[{"S1M", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "&"}], "/@", 
RowBox[{"S2M", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], "]"}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"M", "[", "alg_", "]"}], ":=", 
RowBox[{"Join", "[", 
RowBox[{
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
RowBox[{"S1M", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "&"}], "/@", 
RowBox[{
RowBox[{"Position", "[", 
RowBox[{"S2DiffList", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "[", 
RowBox[{"[", 
RowBox[{"All", ",", "1"}], "]"}], "]"}]}]}], "]"}]}], ";"}], 
StyleBox[
RowBox[{
RowBox[{"TypeS1", "===", "TypeA"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}],
FontWeight->"Bold"]},
{
RowBox[{
RowBox[{
RowBox[{"M", "[", "alg_", "]"}], ":=", 
RowBox[{"Join", "[", 
RowBox[{
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
RowBox[{
RowBox[{"Position", "[", 
RowBox[{"S1DiffList", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "[", 
RowBox[{"[", 
RowBox[{"All", ",", "1"}], "]"}], "]"}]}], ",", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "&"}], "/@", 
RowBox[{"S2M", "[", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], "]"}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeA"}]}]},
{
RowBox[{
RowBox[{
RowBox[{"M", "[", "alg_", "]"}], ":=", 
RowBox[{"Join", "[", 
RowBox[{
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{"#", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], "/@", 
RowBox[{
RowBox[{"Position", "[", 
RowBox[{"S1DiffList", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "[", 
RowBox[{"[", 
RowBox[{"All", ",", "1"}], "]"}], "]"}]}], ",", 
RowBox[{
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"alg", "[", 
RowBox[{"[", "1", "]"}], "]"}], ",", "#"}], "}"}], "&"}], "/@", 
RowBox[{
RowBox[{"Position", "[", 
RowBox[{"S2DiffList", ",", 
RowBox[{"alg", "[", 
RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "[", 
RowBox[{"[", 
RowBox[{"All", ",", "1"}], "]"}], "]"}]}]}], "]"}]}], ";"}], 
RowBox[{
RowBox[{"TypeS1", "===", "TypeD"}], "&&", 
RowBox[{"TypeS2", "===", "TypeD"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
MNonvanishingQ[alg_]:=M[alg]=!={};
(* All Homotopies for killing by M/Mm1 *)
AllHomotopiesForDiff0[MatrixInput_,pos_]:=Tuples[{M[MatrixInput[[##]]&@@pos],{pos[[2]]},{pos[[1]]}}];
AllHomotopiesForDiff[MatrixInput_,pos_]:=CheckIfHomotopyIsAllowedList[MatrixInput,AllHomotopiesForDiff0[MatrixInput,pos]];
AllHomotopies[MatrixInput_,pos_]:=Join[
AllHomotopiesForDiff[MatrixInput,pos],
AllHomotopiesForPreComp[MatrixInput,pos],
AllHomotopiesForPostComp[MatrixInput,pos]];
AllHomotopies::usage="AllHomotopies[MatrixInput_,pos_] computes a list of all homotopies that can remove an arrow label specified by 'pos' in the Structure Maps 'MatrixInput'. A homotopy is a list {algebra element, startgenindex of homotopy, endgenindex of homotopy}.";
(* Check Subscript[A, \[Infinity]]-relations *)
AInfty[mat_]:=MapThread[AlgAA,{Map[Flatten[#,1]&,Map[Mm1,mat,{3}],{2}],Inner[ArrowComp[#2,#1]&,mat,mat,AlgAA]},2];
AInftyQ[mat_]:=Length[Flatten[AInfty[mat]]]==0;
AInftyQ::usage="AInftyQ[mat_] verifies \!\(\*SubscriptBox[\(A\), \(\[Infinity]\)]\)-relations, returning True if they hold and False otherwise.";
(* Could speed this up by parallelising, but if we need to check lots of them, ParallelTry should be faster *)
AllComplexities0[StructureMaps_,pos_,PosToBeCleaned_]:=
Apply[Complexity[Homotopy0[StructureMaps,##],PosToBeCleaned]&,
AllHomotopies[StructureMaps,pos]
,{1}];
AllComplexitiesQuot0[StructureMaps_,pos_,PosToBeCleaned_]:=
Apply[ComplexityQuot[Homotopy0[StructureMaps,##],PosToBeCleaned]&,
AllHomotopies[StructureMaps,pos]
,{1}];
AllComplexities[StructureMaps_,pos_,PosToBeCleaned_:0]:=AllComplexities0[StructureMaps,pos,PosToBeCleaned];
AllComplexitiesQuot[StructureMaps_,pos_,PosToBeCleaned_:0]:=AllComplexitiesQuot0[StructureMaps,pos,PosToBeCleaned];
(* Could speed this up by parallelising, but if we need to check lots of them, ParallelTry should be faster *)
AllComplexitiesParallel0[StructureMaps_,pos_,PosToBeCleaned_]:=
Parallelize[Apply[Complexity[Homotopy0[StructureMaps,##],PosToBeCleaned]&,
AllHomotopies[StructureMaps,pos]
,{1}]];
AllComplexitiesQuotParallel0[StructureMaps_,pos_,PosToBeCleaned_]:=
Parallelize[Apply[ComplexityQuot[Homotopy0[StructureMaps,##],PosToBeCleaned]&,
AllHomotopies[StructureMaps,pos]
,{1}]];
AllComplexitiesParallel[StructureMaps_,pos_,PosToBeCleaned_:0]:=AllComplexitiesParallel0[StructureMaps,pos,PosToBeCleaned];
AllComplexitiesQuotParallel[StructureMaps_,pos_,PosToBeCleaned_:0]:=AllComplexitiesQuotParallel0[StructureMaps,pos,PosToBeCleaned];
AllComplexities::usage="[StructureMaps_,pos_,PosToBeCleaned_] is a list of the complexities for the possible homotopies killing an alegbra element at position 'pos' in 'Structuremaps', in teh order of 'AllHomotopies'. As for the fucntion 'Complexity', PosToBeCleaned is an optional argument whose default is 0.";
ComplexityAlmostDecreasingQ[StructureMaps_,pos_,CurrentComplexity_,PosToBeCleaned_,almostvar_]:=
Min[AllComplexities[StructureMaps,pos,PosToBeCleaned]]<(1+almostvar/100)*CurrentComplexity;
ComplexityQuotAlmostDecreasingQ[StructureMaps_,pos_,CurrentComplexity_,PosToBeCleaned_,almostvar_]:=
Min[AllComplexitiesQuot[StructureMaps,pos,PosToBeCleaned]]<(1+almostvar/100)*CurrentComplexity;

ComplexityDecreasingQ[StructureMaps_,pos_,CurrentComplexity_,PosToBeCleaned_:0,almostvar_:0]:=ComplexityAlmostDecreasingQ[StructureMaps,pos,CurrentComplexity,PosToBeCleaned,almostvar];
ComplexityQuotDecreasingQ[StructureMaps_,pos_,CurrentComplexity_,PosToBeCleaned_:0,almostvar_:0]:=ComplexityQuotAlmostDecreasingQ[StructureMaps,pos,CurrentComplexity,PosToBeCleaned,almostvar];

ComplexityDecreasingParallelQ0[StructureMaps_,pos_,CurrentComplexity_,PosToBeCleaned_,almostvar_:0]:=
Min[AllComplexitiesParallel[StructureMaps,pos,PosToBeCleaned]]<CurrentComplexity;
ComplexityDecreasingParallelQ[StructureMaps_,pos_,CurrentComplexity_,PosToBeCleaned_:0,almostvar_:0]:=ComplexityDecreasingParallelQ0[StructureMaps,pos,CurrentComplexity,PosToBeCleaned,almostvar];

ComplexityDecreasingQ::usage="ComplexityDecreasing[StructureMaps_,pos_,CurrentComplexity_,PosToBeCleaned_] determines if there is a homotopy which removes the algebra element at position 'pos' in 'StructureMaps' and decreases the complexity of 'StructureMaps'. PosToBeCleaned is an optional argument as in the function 'Complexity'";
AutoSeparateGens::usage="AutoSeparateGens[StructureMaps_,gensets_,PosToBeCleaned_,MaxIterations_] tries to find a complex homotopic to the input complex which is the direct sum of some subcomplexes whose underlying sets of generators are give by 'PosToBeCleaned', a list of positions to be cleaned, usually generated by PosToBeCleaned. Output is of the form {NewStructureMaps,LOG=Record of added homotopies}";
AutoSeparateGens[StructureMaps_,gensets_,PosToBeCleaned_,MaxIterations_]:=Module[{
Output={{},{}},
NGens=Length[gensets],
PosOfArrowsToBeRemoved={},
CurrentComplexity=Complexity[StructureMaps,PosToBeCleaned],
StructureMapsit=StructureMaps,
iter=0,
pos,(* this is the position we want to clean up in an iteration *)
homotopyStartEnd={},
start=1,
end=1,
LOG={},
NRandomPicks=0,
cycliccounter=0,
timeused=0,
almostvar=1,
NRandomInd=0,
maxTally=0
},
(* First check if the input is sensible *)
Print[Now];
Monitor[
If[
Length[StructureMaps]!=NGens,Print["USER ERROR: Something is wrong about the generatorsets!"],
(* Calculate list of positions {endgen,startgen,position in label} *)
PosOfArrowsToBeRemoved=Flatten[Apply[Tuples[{{#1},{#2},Range[Length[StructureMapsit[[##]]]]}]&,PosToBeCleaned,{1}],1];
If[
Length[PosOfArrowsToBeRemoved]==0,
Print["This complex is already cleaned up. Done nothing."],
(* Main iteration *)
While[iter<MaxIterations,
NRandomInd=0;
timeused=Timing[
(* Pick an element in PosOfArrowsToBeRemoved, i.e. a position of an algebra element that we want to homotope away. We first search for one where there exists a homotopy which decreases the complexity of 'StructureMapsit'. *)
CurrentComplexity=Complexity[StructureMapsit,PosToBeCleaned];
(* Parallelize this *)
Off[ParallelTry::toofew];
pos=ParallelTry[If[ComplexityDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned],#,$Failed]&,PosOfArrowsToBeRemoved];
(*pos=SelectFirst[PosOfArrowsToBeRemoved,Length[AllHomotopies[StructureMapsit,#]]>0&,0];*)
(*
pos=SelectFirst[PosOfArrowsToBeRemoved,ComplexityDecreasingQ[StructureMapsit,#,CurrentComplexity,GenIndexPartitioning]&,0];
*)
(* If we have not found a homotopy, we look for one elsewhere that decreses complexity in PosToBeCleaned *)
If[pos===$Failed,
PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]!=0,{3}];
pos=ParallelTry[If[ComplexityDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned],#,$Failed]&,PosOfArrowsToBeRemoved];
];
(* If we still have not found a homotopy, we look for one elsewhere that decreses overall complexity *)
If[pos===$Failed,
PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]!=0,{3}];
pos=ParallelTry[If[ComplexityDecreasingQ[StructureMapsit,#,CurrentComplexity],#,$Failed]&,PosOfArrowsToBeRemoved];
];
(* If we still have not found a homotopy and we have not repeated the same homotopies too many times, we look for that almost decreases complexity in PosToBeCleaned *)
almostvar=1;
While[pos===$Failed&&almostvar<100&&maxTally<6,
PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]!=0,{3}];
pos=ParallelTry[If[ComplexityAlmostDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned,almostvar],#,$Failed]&,PosOfArrowsToBeRemoved];
almostvar=almostvar+1;
NRandomInd=NRandomInd+1;
];
If[NRandomInd=!=0,NRandomPicks=NRandomPicks+1];
(* If we do not find one that decreases complexity, we 
1) abort
2)pick a random algebra element to be removed; but make sure that it is actually removable i.e. that there exists a homotopy.  *)
If[pos===$Failed,
(*1*)
Print["Aborted after iteration "<>ToString[iter]];
iter=MaxIterations;
pos=0
(*2*)
(*
PosOfArrowsToBeRemoved=Select[PosOfArrowsToBeRemoved,Length[AllHomotopies[StructureMapsit,#]]>0&];
If[Length[PosOfArrowsToBeRemoved]\[Equal]0,
pos=0;
(* we exit while-loop *)
iter=MaxIterations,
pos=RandomChoice[PosOfArrowsToBeRemoved];
NRandomPicks=NRandomPicks+1
]*)
];

If[
(* If we could not choose any arrow to homotope away (i.e. p=0; this should rarely happen), then leave out this loop and we exit while-loop. *)
pos=!=0,
cycliccounter=cycliccounter+Position[PosOfArrowsToBeRemoved,pos][[1,1]];
(* Select the homotopy with lowest complexity. *)
homotopyStartEnd=AllHomotopies[StructureMapsit,pos][[Position[#,Min[#]][[1,1]]]]&@AllComplexitiesParallel[StructureMapsit,pos,PosToBeCleaned];
(* Add the homotopy *)
StructureMapsit=Homotopy0[StructureMapsit,##]&@@homotopyStartEnd;
(* Record what we have just done *)
LOG=Append[LOG,homotopyStartEnd];
(* Compute the new list of arrows which go between different generator sets *)
PosOfArrowsToBeRemoved=Flatten[Apply[Tuples[{{#1},{#2},Range[Length[StructureMapsit[[##]]]]}]&,PosToBeCleaned,{1}],1];
PosOfArrowsToBeRemoved=RotateLeft[PosOfArrowsToBeRemoved,cycliccounter];
(* Compute maximum number of identical homotopies *)
maxTally=Max[Tally[LOG][[All,2]]];
(* Check if we need to repeat. *)
If[Length[PosOfArrowsToBeRemoved]==0,
iter=MaxIterations;
Print["Success! All Cleaned Up. Could I have some ice cream?"],
(* otherwise increment iter *)
iter=iter+1
];
]
][[1]]
]
]
],{{"Iteration "<>ToString[iter+1]},{"---------------"},{"Remaining Positions"},{ArrayPlot[SparseArray[(#[[1;;2]]->#[[3]])&/@Reverse[PosOfArrowsToBeRemoved],{NGens,NGens},0],ColorRules->{0->Green}]},{"Complexity"},{N[Complexity[StructureMapsit,PosToBeCleaned]]},{"Number of random picks"},{NRandomPicks},{"Time used in previous iteration"},{timeused}}//MatrixForm
];
Print["I increased complexity ",NRandomPicks," times." ];
Print[Now];
Output={StructureMapsit,LOG}
];
AutoSeparateGensQuot[StructureMaps_,gensets_,PosToBeCleaned_,MaxIterations_]:=Module[{
Output={{},{}},
NGens=Length[gensets],
PosOfArrowsToBeRemoved={},
CurrentComplexity=ComplexityQuot[StructureMaps,PosToBeCleaned],
StructureMapsit=StructureMaps,
iter=0,
pos,(* this is the position we want to clean up in an iteration *)
homotopyStartEnd={},
start=1,
end=1,
LOG={},
NRandomPicks=0,
cycliccounter=0,
timeused=0,
almostvar=1,
NRandomInd=0,
maxTally=0
},
(* First check if the input is sensible *)
Print[Now];
Monitor[
If[
Length[StructureMaps]!=NGens,Print["USER ERROR: Something is wrong about the generatorsets!"],
(* Calculate list of positions {endgen,startgen,position in label} *)
PosOfArrowsToBeRemoved=Flatten[Apply[Tuples[{{#1},{#2},Range[Length[StructureMapsit[[##]]]]}]&,PosToBeCleaned,{1}],1];
If[
Length[PosOfArrowsToBeRemoved]==0,
Print["This complex is already cleaned up. Done nothing."],
(* Main iteration *)
While[iter<MaxIterations,
NRandomInd=0;
timeused=Timing[
(* Pick an element in PosOfArrowsToBeRemoved, i.e. a position of an algebra element that we want to homotope away. We first search for one where there exists a homotopy which decreases the complexity of 'StructureMapsit'. *)
CurrentComplexity=ComplexityQuot[StructureMapsit,PosToBeCleaned];
(* Parallelize this *)
Off[ParallelTry::toofew];
pos=ParallelTry[If[ComplexityQuotDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned],#,$Failed]&,PosOfArrowsToBeRemoved];
(*pos=SelectFirst[PosOfArrowsToBeRemoved,Length[AllHomotopies[StructureMapsit,#]]>0&,0];*)
(*
pos=SelectFirst[PosOfArrowsToBeRemoved,ComplexityDecreasingQ[StructureMapsit,#,CurrentComplexity,GenIndexPartitioning]&,0];
*)
(* If we have not found a homotopy, we look for one elsewhere that decreses complexity in PosToBeCleaned *)
If[pos===$Failed,
PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]!=0,{3}];
pos=ParallelTry[If[ComplexityQuotDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned],#,$Failed]&,PosOfArrowsToBeRemoved];
];
(* If we still have not found a homotopy, we look for one elsewhere that decreses overall complexity *)
If[pos===$Failed,
PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]!=0,{3}];
pos=ParallelTry[If[ComplexityQuotDecreasingQ[StructureMapsit,#,CurrentComplexity],#,$Failed]&,PosOfArrowsToBeRemoved];
];
(* If we still have not found a homotopy and we have not repeated the same homotopies too many times, we look for that almost decreases complexity in PosToBeCleaned *)
almostvar=1;
While[pos===$Failed&&almostvar<100&&maxTally<6,
PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]!=0,{3}];
pos=ParallelTry[If[ComplexityQuotAlmostDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned,almostvar],#,$Failed]&,PosOfArrowsToBeRemoved];
almostvar=almostvar+1;
NRandomInd=NRandomInd+1;
];
If[NRandomInd=!=0,NRandomPicks=NRandomPicks+1];
(* If we do not find one that decreases complexity, we 
1) abort
2)pick a random algebra element to be removed; but make sure that it is actually removable i.e. that there exists a homotopy.  *)
If[pos===$Failed,
(*1*)
Print["Aborted after iteration "<>ToString[iter]];
iter=MaxIterations;
pos=0
(*2*)
(*
PosOfArrowsToBeRemoved=Select[PosOfArrowsToBeRemoved,Length[AllHomotopies[StructureMapsit,#]]>0&];
If[Length[PosOfArrowsToBeRemoved]\[Equal]0,
pos=0;
(* we exit while-loop *)
iter=MaxIterations,
pos=RandomChoice[PosOfArrowsToBeRemoved];
NRandomPicks=NRandomPicks+1
]*)
];
If[
(* If we could not choose any arrow to homotope away (i.e. p=0; this should rarely happen), then leave out this loop and we exit while-loop. *)
pos=!=0
,
cycliccounter=cycliccounter+Position[PosOfArrowsToBeRemoved,pos][[1,1]];
(* Select the homotopy with lowest complexity. *)
homotopyStartEnd=AllHomotopies[StructureMapsit,pos][[Position[#,Min[#]][[1,1]]]]&@AllComplexitiesQuotParallel[StructureMapsit,pos,PosToBeCleaned];
(* Add the homotopy *)
StructureMapsit=Homotopy0[StructureMapsit,##]&@@homotopyStartEnd;
(* Record what we have just done *)
LOG=Append[LOG,homotopyStartEnd];
(* Compute the new list of arrows which go between different generator sets *)
PosOfArrowsToBeRemoved=Flatten[Apply[Tuples[{{#1},{#2},Range[Length[StructureMapsit[[##]]]]}]&,PosToBeCleaned,{1}],1];
PosOfArrowsToBeRemoved=RotateLeft[PosOfArrowsToBeRemoved,cycliccounter];
(* Compute maximum number of identical homotopies *)
maxTally=Max[Tally[LOG][[All,2]]];
(* Check if we need to repeat. *)
If[Length[PosOfArrowsToBeRemoved]==0,
iter=MaxIterations;
Print["Success! All Cleaned Up. Could I have some ice cream?"],
(* otherwise increment iter *)
iter=iter+1
];
]
][[1]]
]
]
]
,
{{"Iteration "<>ToString[iter+1]},{"---------------"},{"Remaining Positions"},{ArrayPlot[SparseArray[(#[[1;;2]]->#[[3]])&/@Reverse[PosOfArrowsToBeRemoved],{NGens,NGens},0],ColorRules->{0->Green}]},{"ComplexityQuot"},{N[ComplexityQuot[StructureMapsit,PosToBeCleaned]]},{"Number of random picks"},{NRandomPicks},{"Time used in previous iteration"},{timeused}}//MatrixForm
];
Print["I increased complexity ",NRandomPicks," times." ];
Print[Now];
Output={StructureMapsit,LOG}
];
(*Clean up position*)AutoCleanPos::usage="AutoCleanPos[StructureMaps_,gensets_,PosToBeCleaned_,MaxIterations_]: tries to clean up a single position in StructureMaps, specified by PosToBeCleaned.";
AutoCleanPos[StructureMaps_,gensets_,PosToBeCleaned_,MaxIterations_]:=Module[{Output={{},{}},NGens=Length[gensets],PosOfArrowsToBeRemoved={},CurrentComplexity=ComplexityQuot[StructureMaps,{PosToBeCleaned}],StructureMapsit=StructureMaps,iter=0,pos,homotopyStartEnd={},start=1,end=1,LOG={},NRandomPicks=0,timeused=0,almostvar=1,NRandomInd=0,maxTally=0,numberofhomotopies=1,Mm1MHomotopyResults,possibleCandidates,i=0},(*First check if the input is sensible*)Print[Now];
Monitor[If[Length[StructureMaps]!=NGens,Print["USER ERROR: Something is wrong about the generatorsets!"],(*Calculate list of positions {endgen,startgen,position in label}*)PosOfArrowsToBeRemoved=Append[PosToBeCleaned,#]&/@Range[Length[Part[StructureMapsit,##]&@@PosToBeCleaned]];
If[Length[PosOfArrowsToBeRemoved]==0,Print["This complex is already cleaned up. Done nothing."],(*Main iteration*)While[iter<MaxIterations,NRandomInd=0;
timeused=Timing[(*Pick an element in PosOfArrowsToBeRemoved,i.e.a position of an algebra element that we want to homotope away.We first search for one where there exists a homotopy which decreases the complexity of'StructureMapsit'.*)CurrentComplexity=ComplexityQuot[StructureMapsit,{PosToBeCleaned}];
(*Parallelize this*)(*Off[ParallelTry::toofew];
pos=ParallelTry[If[ComplexityDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned],#,$Failed]&,PosOfArrowsToBeRemoved];*)numberofhomotopies=1;
pos=SelectFirst[PosOfArrowsToBeRemoved,ComplexityQuotDecreasingQ[StructureMapsit,#,CurrentComplexity,{PosToBeCleaned}]&,$Failed];
If[pos===$Failed,numberofhomotopies=2;
(*pos=SelectFirst[PosOfArrowsToBeRemoved,ComplexityDecreasingQ[StructureMapsit,#,CurrentComplexity,GenIndexPartitioning]&,0];*)(*If we still have not found a homotopy,we look for one elsewhere that decreses overall complexity*)(*If[pos===$Failed,PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]\[NotEqual]0,{3}];
pos=SelectFirst[PosOfArrowsToBeRemoved,ComplexityDecreasingQ[StructureMapsit,#,Complexity[StructureMapsit,1]]&,$Failed];];*)(*Look for double homotopies that reduce the complexity as PosToBeCleaned*)Mm1MHomotopyResults=Flatten[StructureMapsit/@M[#],1]&/@(StructureMapsit[[##]]&@@PosToBeCleaned);
possibleCandidates=Complement[Select[Tuples[{#,#}]&@Range[Length[StructureMapsit[[##]]&@@PosToBeCleaned]],IntersectingQ[Mm1MHomotopyResults[[#[[1]]]],Mm1MHomotopyResults[[#[[2]]]]]&],{#,#}&/@Range[Length[StructureMapsit[[##]]&@@PosToBeCleaned]]];
For[i=1,i<=Length[possibleCandidates],i++,Off[ParallelTry::toofew];
homotopyStartEnd=ParallelTry[If[(ComplexityQuot[Function[{y1,y2,y3},Homotopy0[Function[{x1,x2,x3},Homotopy0[StructureMapsit,x1,x2,x3]]@@#[[1]],y1,y2,y3]]@@#[[2]],{PosToBeCleaned}]<CurrentComplexity)==True,#,$Failed]&,Tuples[{AllHomotopiesForDiff[StructureMapsit,Join[PosToBeCleaned,{possibleCandidates[[i,1]]}]],AllHomotopiesForDiff[StructureMapsit,Join[PosToBeCleaned,{possibleCandidates[[i,2]]}]]}]];
If[homotopyStartEnd=!=$Failed,i=Length[possibleCandidates]+1];];
If[Length[possibleCandidates]==0,homotopyStartEnd=$Failed];
If[homotopyStartEnd=!=$Failed,pos=1];];
(*If we still have not found a homotopy and we have not repeated the same homotopies too many times,we look for that almost decreases complexity in PosToBeCleaned*)(*almostvar=1;
While[pos===$Failed&&almostvar<100&&maxTally<6,PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]\[NotEqual]0,{3}];
pos=SelectFirst[PosOfArrowsToBeRemoved,ComplexityAlmostDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned,almostvar]&,$Failed];
almostvar=almostvar+1;
NRandomInd=NRandomInd+1;];
If[NRandomInd=!=0,NRandomPicks=NRandomPicks+1];*)(*If we do not find one that decreases complexity,we 1) abort 2)pick a random algebra element to be removed;
but make sure that it is actually removable i.e.that there exists a homotopy.*)If[pos===$Failed,(*1*)Print["Aborted after iteration "<>ToString[iter]];
iter=MaxIterations;
pos=0
(*2*)
(*PosOfArrowsToBeRemoved=Select[PosOfArrowsToBeRemoved,Length[AllHomotopies[StructureMapsit,#]]>0&];
If[Length[PosOfArrowsToBeRemoved]\[Equal]0,pos=0;
(*we exit while-loop*)iter=MaxIterations,pos=RandomChoice[PosOfArrowsToBeRemoved];
NRandomPicks=NRandomPicks+1]*)];

If[(*If we could not choose any arrow to homotope away (i.e.p=0;this should rarely happen),then leave out this loop and we exit while-loop.*)pos=!=0,(*Select the homotopy with lowest complexity.*)(*Add the homotopy*)If[numberofhomotopies==1,homotopyStartEnd=AllHomotopies[StructureMapsit,pos][[Position[#,Min[#]][[1,1]]]]&@AllComplexitiesQuot[StructureMapsit,pos,{PosToBeCleaned}];
StructureMapsit=Homotopy0[StructureMapsit,##]&@@homotopyStartEnd,StructureMapsit=Function[{y1,y2,y3},Homotopy0[Function[{x1,x2,x3},Homotopy0[StructureMapsit,x1,x2,x3]]@@#[[1]],y1,y2,y3]]@@#[[2]]&@homotopyStartEnd];
(*Record what we have just done*)LOG=Append[LOG,homotopyStartEnd];
(*Compute the new list of arrows which go between different generator sets*)PosOfArrowsToBeRemoved=Append[PosToBeCleaned,#]&/@Range[Length[Part[StructureMapsit,##]&@@PosToBeCleaned]];
(*Compute maximum number of identical homotopies*)maxTally=Max[Tally[LOG][[All,2]]];
(*Check if we need to repeat.*)If[Length[PosOfArrowsToBeRemoved]==0,iter=MaxIterations;
Print["Success! All Cleaned Up. Could I have some ice cream?"],(*otherwise increment iter*)iter=iter+1];]][[1]]]]],{{"Iteration "<>ToString[iter+1]},{"---------------"},{{numberofhomotopies,i}},{Length[Part[StructureMapsit,##]&@@PosToBeCleaned]},{"Remaining Positions"},{ArrayPlot[SparseArray[(#[[1;;2]]->#[[3]])&/@Reverse[PosOfArrowsToBeRemoved],{NGens,NGens},0],ColorRules->{0->Green}]},{"Complexity"},{N[ComplexityQuot[StructureMapsit,{PosToBeCleaned}]]},{"Number of random picks"},{NRandomPicks},{"Time used in previous iteration"},{timeused}}//MatrixForm];
Print["I increased complexity ",NRandomPicks," times."];
Print[Now];
Output={StructureMapsit,LOG}];
(*Clean up position*)
AutoCleanPosns::usage="AutoCleanPos[StructureMaps_,gensets_,PosToBeCleaned_,MaxIterations_]: tries to clean up a set of positions in StructureMaps, specified by PosToBeCleaned.";
AutoCleanPosns[StructureMaps_,gensets_,PosToBeCleaned_,MaxIterations_]:=Module[{
Output={{},{}},
NGens=Length[gensets],
PosOfArrowsToBeRemoved={},
CurrentComplexity=ComplexityQuot[StructureMaps,PosToBeCleaned],
StructureMapsit=StructureMaps,
iter=0,
pos,
homotopyStartEnd={},
start=1,
end=1,
LOG={},
NRandomPicks=0,
timeused=0,
almostvar=1,
NRandomInd=0,
maxTally=0,
numberofhomotopies=1,
Mm1MHomotopyResults,
i=0,
ii=0,
iii=0,
PosnsToBeCleaned,
sPosToBeCleaned,
After1Homotopy,
All1Homotopies,
All2Homotopies={}
},(*First check if the input is sensible*)
Print[Now];
Monitor[If[Length[StructureMaps]!=NGens,Print["USER ERROR: Something is wrong about the generatorsets!"],(*Calculate list of positions {endgen,startgen,position in label}*)PosOfArrowsToBeRemoved=RandomSample[Flatten[Apply[Tuples[{{#1},{#2},Range[Length[StructureMapsit[[##]]]]}]&,PosToBeCleaned,{1}],1],UpTo[10^5]];
If[Length[PosOfArrowsToBeRemoved]==0,Print["This complex is already cleaned up. Done nothing."],(*Main iteration*)While[iter<MaxIterations,NRandomInd=0;
timeused=Timing[(*Pick an element in PosOfArrowsToBeRemoved,i.e.a position of an algebra element that we want to homotope away.We first search for one where there exists a homotopy which decreases the complexity of'StructureMapsit'.*)
(* initialize values *)
numberofhomotopies=1;
All2Homotopies={};
i=0;
ii=0;
iii=0;
CurrentComplexity=ComplexityQuot[StructureMapsit,PosToBeCleaned];
(*
Off[ParallelTry::toofew];
pos=ParallelTry[If[ComplexityQuotDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned],#,$Failed]&,PosOfArrowsToBeRemoved];
*)
pos=SelectFirst[PosOfArrowsToBeRemoved,ComplexityQuotDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned]&,$Failed];
(* if we haven't found a homotopy yet, we look for two consecutive homotopies *)
If[pos===$Failed,
(* We now try to do two consecutive homotopies which might lower complexity *)
numberofhomotopies=2;
(* initialize value *)
homotopyStartEnd=$Failed;
(* make list of all positions that we want to clean up, ordered by decreasing number of labels *)
PosnsToBeCleaned=DeleteCases[SortBy[PosToBeCleaned,Length[Part[StructureMapsit,##]&@@#]&],v_/;Length[Part[StructureMapsit,##]&@@v]==0];
(* Now try each of these positions *)
For[i=1,i<=Length[PosnsToBeCleaned],i++,
sPosToBeCleaned=PosnsToBeCleaned[[i]];
(* i is the label index on the arrow specified by sPosToBeCleaned *)
For[ii=1,ii<=Length[StructureMapsit[[##]]&@@sPosToBeCleaned],ii++,
(* Compute all possible [first] homotopies for each label on arrow specified by 'sPosToBeCleaned' *)
All1Homotopies=AllHomotopiesForDiff[StructureMapsit,Join[sPosToBeCleaned,{ii}]];
(* iii is the label index on the arrow specified by sPosToBeCleaned *)
For[iii=1,iii<= Length[All1Homotopies],iii++,
(* Compute new structure maps from current first homotopy *)
After1Homotopy=Function[{x1,x2,x3},Homotopy0[StructureMapsit,x1,x2,x3]]@@All1Homotopies[[iii]];
(* Compute all possible [second] homotopies for each label on arrow specified by 'sPosToBeCleaned' *)
All2Homotopies=Flatten[AllHomotopiesForDiff[After1Homotopy,Join[sPosToBeCleaned,{#}]]&/@Range[Length[After1Homotopy[[##]]&@@sPosToBeCleaned]],1];
(* Now search for those homotopies which lower complexity *)
If[
Length[All2Homotopies]>0,
homotopyStartEnd=SelectFirst[All2Homotopies,(ComplexityQuot[Function[{x1,x2,x3},Homotopy0[After1Homotopy,x1,x2,x3]]@@#,PosToBeCleaned]<CurrentComplexity)&,$Failed]
(*
Off[ParallelTry::toofew];
homotopyStartEnd=
ParallelTry[If[(ComplexityQuot[Function[{x1,x2,x3},Homotopy0[After1Homotopy,x1,x2,x3]]@@#,PosToBeCleaned]<CurrentComplexity)\[Equal]True,#,$Failed]&,All2Homotopies]
*)
];
If[homotopyStartEnd=!=$Failed,
homotopyStartEnd={All1Homotopies[[iii]],homotopyStartEnd};
i=Length[PosnsToBeCleaned]+1;
ii=Length[StructureMapsit[[##]]&@@sPosToBeCleaned]+1;
iii=Length[All1Homotopies]+1;
];
]
]
];
If[homotopyStartEnd=!=$Failed,pos=1];
];
(*If we still have not found a homotopy and we have not repeated the same homotopies too many times,we look for that almost decreases complexity in PosToBeCleaned*)(*almostvar=1;
While[pos===$Failed&&almostvar<100&&maxTally<6,PosOfArrowsToBeRemoved=Position[StructureMapsit,v_/;Length[v]\[NotEqual]0,{3}];
pos=SelectFirst[PosOfArrowsToBeRemoved,ComplexityAlmostDecreasingQ[StructureMapsit,#,CurrentComplexity,PosToBeCleaned,almostvar]&,$Failed];
almostvar=almostvar+1;
NRandomInd=NRandomInd+1;];
If[NRandomInd=!=0,NRandomPicks=NRandomPicks+1];*)(*If we do not find one that decreases complexity,we 1) abort 2)pick a random algebra element to be removed;
but make sure that it is actually removable i.e.that there exists a homotopy.*)If[pos===$Failed,(*1*)Print["Aborted after iteration "<>ToString[iter]];
iter=MaxIterations;
pos=0
(*2*)
(*PosOfArrowsToBeRemoved=Select[PosOfArrowsToBeRemoved,Length[AllHomotopies[StructureMapsit,#]]>0&];
If[Length[PosOfArrowsToBeRemoved]\[Equal]0,pos=0;
(*we exit while-loop*)iter=MaxIterations,pos=RandomChoice[PosOfArrowsToBeRemoved];
#{\[Alpha]-circles}=NRandomPicks=NRandomPicks+1]*)];
If[(*If we could not choose any arrow to homotope away (i.e.p=0;this should rarely happen),then leave out this loop and we exit while-loop.*)pos=!=0,(*Select the homotopy with lowest complexity.*)(*Add the homotopy*)If[numberofhomotopies==1,
homotopyStartEnd=AllHomotopies[StructureMapsit,pos][[Position[#,Min[#]][[1,1]]]]&@AllComplexitiesQuot[StructureMapsit,pos,PosToBeCleaned];
StructureMapsit=Homotopy0[StructureMapsit,##]&@@homotopyStartEnd
,
StructureMapsit=Function[{y1,y2,y3},Homotopy0[Function[{x1,x2,x3},Homotopy0[StructureMapsit,x1,x2,x3]]@@#[[1]],y1,y2,y3]]@@#[[2]]&@homotopyStartEnd];
(*Record what we have just done*)LOG=Append[LOG,homotopyStartEnd];
(*Compute the new list of arrows which go between different generator sets*)PosOfArrowsToBeRemoved=RandomSample[Flatten[Apply[Tuples[{{#1},{#2},Range[Length[StructureMapsit[[##]]]]}]&,PosToBeCleaned,{1}],1],UpTo[10^5]];
(*Compute maximum number of identical homotopies*)maxTally=Max[Tally[LOG][[All,2]]];
(*Check if we need to repeat.*)
If[Length[PosOfArrowsToBeRemoved]==0,iter=MaxIterations;
Print["Success! All Cleaned Up. Could I have some ice cream?"],(*otherwise increment iter*)iter=iter+1];]][[1]]]]],{{"Iteration "<>ToString[iter+1]},{"---------------"},{ToString[numberofhomotopies]<>","<>ToString[i]<>"/"<>ToString[Length[PosnsToBeCleaned]]<>","<>ToString[ii]<>"/"<>ToString[Length[StructureMapsit[[##]]&@@sPosToBeCleaned]]<>","<>ToString[iii]<>"/"<>ToString[Length[All1Homotopies]]<>","<>ToString[Length[All2Homotopies]]},{If[numberofhomotopies==1," ----- ",{sPosToBeCleaned}]},{"Remaining Positions"},{ArrayPlot[SparseArray[(#[[1;;2]]->#[[3]])&/@Reverse[PosOfArrowsToBeRemoved],{NGens,NGens},0],ColorRules->{0->Green}]},{"Complexity"},{N[ComplexityQuot[StructureMapsit,PosToBeCleaned]]},{"Number of random picks"},{NRandomPicks},{"Time used in previous iteration"},{timeused}}//MatrixForm];
Print["I increased complexity ",NRandomPicks," times."];
Print[Now];
Output={StructureMapsit,LOG}];
];
